/************************************************************************
 *																		*
 *	Copyright (c) 2001 by			David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/


#include <afx.h>
#include "msgs.h"


CString	make_message(int nCommandCode, CStringList & cslArgs)
{
	CString		strMessage;
	POSITION	posP;
	CString		strArg;
	CString		strTemp;
	int			nI;
	int			nJ;
	BOOL		bArgAlreadyQuoted;
	char		c;

//	Do the command code.

	strMessage.Format("%u", (nCommandCode & 0x00FFFF));

//	If there are arguments, append a space and one of the arguments (with quote
//	handling applied) until the list is exhausted.

	posP = cslArgs.GetHeadPosition();
	while (posP != NULL)
	{
		strMessage += " ";
		strArg = cslArgs.GetNext(posP);
		bArgAlreadyQuoted = FALSE;

//	Quoting rules:
//	1.	If the argument contains '\\', it must be quoted and each '\\' character
//		must be escaped with '\\'.
//	2.	If the argument contains '\"', each '\"' character must be escaped with
//		'\\'.  If rule 1 has not already been applied, the argument must also be
//		quoted.
//	3.	If the argument contains space and rules 1 and 2 have not already been
//		applied, it must be quoted.

		if (strArg.Find('\\') >= 0)
		{

//	Apply rule 1.

			strTemp = "\"";
			for (nI = 0; nI < strArg.GetLength(); ++nI)
			{
				if ((c = strArg.GetAt(nI)) == '\\')
				{
					strTemp += '\\';
				}
				strTemp += c;
			}
			strTemp += '\"';
			strArg = strTemp;
			bArgAlreadyQuoted = TRUE;
		}
		if (strArg.Find('\"') >= 0)
		{

//	Apply rule 2.

			if (bArgAlreadyQuoted)
			{

//	Remove outer quotes before searching for quotes.
				nI = 1;
				nJ = strArg.GetLength() - 2;
			}
			else
			{
				nI = 0;
				nJ = strArg.GetLength();
			}
			strArg = strArg.Mid(nI, nJ);
			strTemp = "\"";
			for (nI = 0; nI < strArg.GetLength(); ++nI)
			{
				if ((c = strArg.GetAt(nI)) == '\"')
				{
					strTemp += '\\';
				}
				strTemp += c;
			}
			strTemp += '\"';
			strArg = strTemp;
			bArgAlreadyQuoted = TRUE;
		}
		if (strArg.Find(' ') >= 0)
		{
			if (!bArgAlreadyQuoted)
			{

//	Apply rule 3.

				if (strArg.Find(' ') >= 0)
				{
					strTemp = "\"" + strArg + "\"";
					strArg = strTemp;
				}
			}
		}
		strMessage += strArg;
	}
	strMessage += "\n";
	return strMessage;
}


BOOL	parse_message(CString strMessage,
						int & nCommandCode,
						CStringList & cslArgs)
{
	CString		strTemp;
	CString		strArg;
	int			nI;
	char		c;
	BOOL		bEscape;

	cslArgs.RemoveAll();

//	Test for linefeed somewhere in strMessage.  Also ensure string starts with
//	a digit.

	if (strMessage.Find('\n') < 0) return FALSE;
	if (!isdigit(strMessage[0])) return FALSE;

//	Get command code.

	nCommandCode = (atoi(strMessage) & 0x00FFFF);
	for (nI = 0; isdigit(strMessage[nI]); ++nI)
		;
	if (strMessage[nI] != ' ') return TRUE;		//	no arguments -- done.
	++nI;

//	Message has arguments.  Parse them, de-quoting if necessary.

	while ((c = strMessage.GetAt(nI)) != '\n')
	{
		strArg = "";
		if (c != '\"')
		{

//	Simple case -- just look for a space.  If we find '\\' or '\"' it's a syntax
//	error, though.

			while (((c = strMessage.GetAt(nI)) != '\n') && (c != ' '))
			{
				if ((c == '\\') || (c == '\"')) return FALSE;
				strArg += c;
				nI++;
			}
			if (c == ' ')
				++nI;
		}
		else
		{

//	Quoted string case -- un-escape quotes and backslashes and strip terminal
//	quote character.

			++nI;
			bEscape = FALSE;
			for (;;)
			{
				if ((c = strMessage.GetAt(nI)) == '\"')
				{
					if (!bEscape) break;
					else
					{
						strArg += c;
						bEscape = FALSE;
						++nI;
					}
				}
				else if (c == '\\')
				{
					if (!bEscape)
					{
						bEscape = TRUE;
					}
					else
					{
						strArg += c;
						bEscape = FALSE;
					}
					++nI;
				}
				else
				{
					strArg += c;
					bEscape = FALSE;
					++nI;
				}
				if (c == '\n') return FALSE;	//	synatx error!
			}

//	Un-escaped quote. End of quoted string.  See if next character is linefeed
//	or space.  If not, syntax error.

			++nI;
			if (((c = strMessage.GetAt(nI)) != '\n') && (c != ' '))
				return FALSE;
			if (c == ' ')
				++nI;
		}
		cslArgs.AddTail(strArg);
	}
	return TRUE;
}

