/************************************************************************
 *																		*
 *	Copyright (c) 2001 by			David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/


#include "stdafx.h"
#include "tapeui.h"
#include "tapeuiDlg.h"
#include "msgs.h"
#include <process.h>


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/*======================================================================*
 |																		|
 |	Function prototype of the thread proc which handles inbound pipe	|
 |																		|
 *======================================================================*/

static UINT __stdcall	RxMsgThreadFunc(void * pParams);	// Thread 2 proc.

#define MUTEX_NAME	"InMessageTxtMutex"

/*======================================================================*
 |																		|
 |	Global variables:	for use between multiple threads (unless I got	|
 |						sloppy).  Since all of the thread #2 code is	|
 |						contained in this file, these variables are		|
 |						given file scope.								|
 |																		|
 *======================================================================*/

static PIPE_CONTROL	stPipeControl[2];				// Contains state info and
													// handles for I/O to pipes
static HANDLE			hEvent[3];					// The first two events are
													// used to signal I/O
													// complete on pipes; the
													// third one is used to
													// signal the input thread
													// to stop because we're
													// gonna wipe out the dialog
													// and hence the app.
static HANDLE			hInputThread;				// thread which reads
													// messages to window
static HANDLE			hmCommMutex;				// Interlocks the two
													// threads
static char 			szInMessageTxt[
							MAX_MESSAGE_LENGTH];	// Incoming message text
													// from thread 2
static BOOL				bMessageBufferAvailable;	// Above buffer ready for
													// re-use


/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTapeuiDlg dialog

CTapeuiDlg::CTapeuiDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CTapeuiDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CTapeuiDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_bConnected = FALSE;
	m_bPlayerMounted = FALSE;
	m_bPlayLocked = FALSE;
	m_bRecorderMounted = FALSE;
	m_bRecordLocked = FALSE;
	m_strH8Environment = "";
	hmCommMutex = NULL;
	hInputThread = NULL;
	hEvent[0] = INVALID_HANDLE_VALUE;
	hEvent[1] = INVALID_HANDLE_VALUE;
	hEvent[2] = INVALID_HANDLE_VALUE;
	memset(&stPipeControl[0], 0, sizeof(PIPE_CONTROL));
	stPipeControl[0].ePipeMode = PIPEMODE_UNDEFINED;
	memset (&stPipeControl[0].overControl, 0, sizeof(OVERLAPPED)); 
	stPipeControl[0].overControl.hEvent = INVALID_HANDLE_VALUE;
 	stPipeControl[0].hPipe = INVALID_HANDLE_VALUE;
	stPipeControl[0].ePipeState = PIPESTATE_CLOSED;
	memset(&stPipeControl[1], 0, sizeof(PIPE_CONTROL));
	stPipeControl[1].ePipeMode = PIPEMODE_UNDEFINED;
	memset (&stPipeControl[1].overControl, 0, sizeof(OVERLAPPED)); 
	stPipeControl[1].overControl.hEvent = INVALID_HANDLE_VALUE;
	stPipeControl[1].ePipeState = PIPESTATE_CLOSED;
 	stPipeControl[1].hPipe = INVALID_HANDLE_VALUE;
}

void CTapeuiDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTapeuiDlg)
	DDX_Control(pDX, IDC_PLAY_LIGHT, m_PlayLightFrame);
	DDX_Control(pDX, IDC_RECORD_LIGHT, m_RecordLightFrame);
	DDX_Control(pDX, IDC_PLAY_EJECT, m_PlayEjectBtn);
	DDX_Control(pDX, IDC_RECORD_EJECT, m_RecordEjectBtn);
	DDX_Control(pDX, IDC_RECORDER_WND, m_RecorderFrame);
	DDX_Control(pDX, IDC_PLAYER_WND, m_PlayerFrame);
	DDX_Control(pDX, IDC_PWR_LIGHT, m_PwrLightFrame);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CTapeuiDlg, CDialog)
	//{{AFX_MSG_MAP(CTapeuiDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_PLAY_EJECT, OnPlayEject)
	ON_BN_CLICKED(IDC_RECORD_EJECT, OnRecordEject)
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP

//	Message handlers for signals from Thread Two

    ON_MESSAGE(UM_INCOMING, OnIncoming)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTapeuiDlg message handlers

BOOL CTapeuiDlg::OnInitDialog()
{
	CRect		rWnd;
	CRect		rLight;
	int			nWidth;
	int			nHeight;
	CStringList	cslArgs;

	CDialog::OnInitDialog();

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		pSysMenu->EnableMenuItem(SC_MAXIMIZE, MF_BYCOMMAND | MF_GRAYED);
		pSysMenu->EnableMenuItem(SC_SIZE, MF_BYCOMMAND | MF_GRAYED);
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	m_bmpMounted.LoadBitmap(IDB_MOUNTED);
	m_bmpUnmounted.LoadBitmap(IDB_UNMOUNTED);
	m_PlayerFrame.GetWindowRect(&rWnd);
	ScreenToClient(&rWnd);
	m_PlayLightFrame.GetWindowRect(&rLight);
	nWidth = rLight.Width();
	rLight.left = (rWnd.left + rWnd.right) / 2 - nWidth / 2;
	rLight.right = rLight.left + nWidth;
	nHeight = rLight.Height();
	rLight.bottom = rWnd.bottom - (rWnd.Height() / 6);
	rLight.top = rLight.bottom - nHeight;
	m_PlayLightFrame.MoveWindow(&rLight);
	m_RecorderFrame.GetWindowRect(&rWnd);
	ScreenToClient(&rWnd);
	m_RecordLightFrame.GetWindowRect(&rLight);
	nWidth = rLight.Width();
	rLight.left = (rWnd.left + rWnd.right) / 2 - nWidth / 2;
	rLight.right = rLight.left + nWidth;
	nHeight = rLight.Height();
	rLight.bottom = rWnd.bottom - (rWnd.Height() / 6);
	rLight.top = rLight.bottom - nHeight;
	m_RecordLightFrame.MoveWindow(&rLight);
	m_PlayEjectBtn.EnableWindow(FALSE);
	m_RecordEjectBtn.EnableWindow(FALSE);
	while (!ConnectCommunications())
	{
		if (AfxMessageBox("Power up the H-8 "
							"and try again?", MB_RETRYCANCEL) == IDCANCEL) break;

	}
	if (m_bConnected)
	{
		m_bConnected = FALSE;
		EmitMessage(CONTEXT_INQUIRY, "0");
	}
	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CTapeuiDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CTapeuiDlg::OnPaint() 
{
	CPaintDC dc(this); // device context for painting

	if (IsIconic())
	{
		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CRect		rWnd;
		CRect		rWnd2;
		int			nHeight;
		CBrush *	pRedBrush = new CBrush(RGB(255, 0, 0));
		CBrush *	pDarkRedBrush = new CBrush(RGB(102, 0, 0));
		CBrush *	pGreenBrush = new CBrush(RGB(0, 255, 0));
		CBrush *	pDarkGreenBrush = new CBrush(RGB(0, 51, 0));
		CBrush *	pYellowBrush = new CBrush(RGB(255, 255, 0));
		CBrush *	pDarkYellowBrush = new CBrush(RGB(102, 102, 0));
		CDC 		memDC;
		CBitmap *	pBmpOld;
		CString		strLabel;

		m_PlayEjectBtn.EnableWindow(!m_bPlayLocked && m_bConnected);
		m_RecordEjectBtn.EnableWindow(!m_bRecordLocked && m_bConnected);

		CDialog::OnPaint();

		memDC.CreateCompatibleDC(&dc);
		m_PlayerFrame.GetWindowRect(&rWnd);
		ScreenToClient(&rWnd);
		m_RecorderFrame.GetWindowRect(&rWnd2);
		ScreenToClient(&rWnd2);
		rWnd.left = rWnd.right;
		rWnd.right = rWnd2.left;
		dc.SelectStockObject(BLACK_PEN);
		dc.SelectStockObject(GRAY_BRUSH);
		dc.Rectangle(&rWnd);
		m_PwrLightFrame.GetWindowRect(&rWnd);
		ScreenToClient(&rWnd);
		dc.SelectObject(m_bConnected ? pGreenBrush : pDarkGreenBrush);
		dc.Ellipse(&rWnd);
		m_PlayLightFrame.GetWindowRect(&rWnd);
		ScreenToClient(&rWnd);
		dc.SelectObject(m_bPlayLocked ? pYellowBrush : pDarkYellowBrush);
		dc.Ellipse(&rWnd);
		m_RecordLightFrame.GetWindowRect(&rWnd);
		ScreenToClient(&rWnd);
		dc.SelectObject(m_bRecordLocked ? pRedBrush : pDarkRedBrush);
		dc.Ellipse(&rWnd);
		dc.SelectStockObject(NULL_BRUSH);
		m_PlayerFrame.GetWindowRect(&rWnd);
		ScreenToClient(&rWnd);
		dc.Rectangle(&rWnd);
		rWnd.top += 20;
		rWnd.left += 20;
		rWnd.right -= 20;
		nHeight = rWnd.Width() * 50 / 75;
		rWnd.bottom = rWnd.top + nHeight;
		if (m_bPlayerMounted)
		{
			pBmpOld = memDC.SelectObject(&m_bmpMounted);
		}
		else
		{
			pBmpOld = memDC.SelectObject(&m_bmpUnmounted);
		}
		dc.StretchBlt(rWnd.left,
						rWnd.top,
						rWnd.Width(),
						rWnd.Height(),
						&memDC,
						0,
						0,
						75,
						50,
						SRCCOPY);
		memDC.SelectObject(pBmpOld);
		dc.SetTextAlign(TA_CENTER | TA_TOP);
		dc.SetBkMode(TRANSPARENT);
		dc.TextOut((rWnd.left + rWnd.right) / 2, rWnd.bottom + 10, "Player");
		if (m_bPlayerMounted)
		{
			strLabel = m_strPlayerFileName.Left(12);
			strLabel.MakeLower();
			dc.TextOut((rWnd.left + rWnd.right) / 2, rWnd.top + 15,
						strLabel);
		}
		m_RecorderFrame.GetWindowRect(&rWnd);
		ScreenToClient(&rWnd);
		dc.Rectangle(&rWnd);
		rWnd.top += 20;
		rWnd.left += 20;
		rWnd.right -= 20;
		nHeight = rWnd.Width() * 50 / 75;
		rWnd.bottom = rWnd.top + nHeight;
		if (m_bRecorderMounted)
		{
			pBmpOld = memDC.SelectObject(&m_bmpMounted);
		}
		else
		{
			pBmpOld = memDC.SelectObject(&m_bmpUnmounted);
		}
		dc.StretchBlt(rWnd.left,
						rWnd.top,
						rWnd.Width(),
						rWnd.Height(),
						&memDC,
						0,
						0,
						75,
						50,
						SRCCOPY);
		memDC.SelectObject(pBmpOld);
		dc.TextOut((rWnd.left + rWnd.right) / 2, rWnd.bottom + 10, "Recorder");
		if (m_bRecorderMounted)
		{
			strLabel = m_strRecorderFileName.Left(12);
			strLabel.MakeLower();
			dc.TextOut((rWnd.left + rWnd.right) / 2, rWnd.top + 15,
						strLabel);
		}
		delete pRedBrush;
		delete pDarkRedBrush;
		delete pGreenBrush;
		delete pDarkGreenBrush;
		delete pYellowBrush;
		delete pDarkYellowBrush;


	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CTapeuiDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CTapeuiDlg::OnPlayEject() 
{
	static char BASED_CODE	szFilter[] = "H8-5 Tape Files (*.tape)"
										"|*.tape||";
	static char BASED_CODE	szDir[MAX_PATH];
	CStringList				cslArgs;

	CFileDialog myDlg(TRUE, "tape", "in.tape", 0, szFilter);
	if (m_strH8Environment.IsEmpty())
	{
		strcpy(szDir, "c:\\h8\\media\\tapes");
	}
	else
	{
		strcpy(szDir, m_strH8Environment + "\\media\\tapes");
	}
	myDlg.m_ofn.lpstrInitialDir = szDir;
	cslArgs.RemoveAll();
	cslArgs.AddTail("0");
	if (!m_bPlayerMounted)
	{	
		if (myDlg.DoModal() == IDOK)
		{
			cslArgs.AddTail(myDlg.GetPathName());
			EmitMessage(CASSETTE_CHANGE, cslArgs);
			m_strPlayerFileName = myDlg.GetFileTitle();
			m_bPlayerMounted = TRUE;
			RedrawWindow();
		}
	}
	else
	{
		EmitMessage(CASSETTE_CHANGE, cslArgs);
		m_bPlayerMounted = FALSE;
		RedrawWindow();
	}
}

void CTapeuiDlg::OnRecordEject() 
{
	static char BASED_CODE	szFilter[] = "H8-5 Tape Files (*.tape)"
										"|*.tape||";
	static char BASED_CODE	szDir[MAX_PATH];
	CStringList				cslArgs;

	CFileDialog myDlg(FALSE, "tape", "out.tape",
						OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, szFilter);
	if (m_strH8Environment.IsEmpty())
	{
		strcpy(szDir, "c:\\h8\\media\\tapes");
	}
	else
	{
		strcpy(szDir, m_strH8Environment + "\\media\\tapes");
	}
	myDlg.m_ofn.lpstrInitialDir = szDir;
	cslArgs.RemoveAll();
	cslArgs.AddTail("1");
	if (!m_bRecorderMounted)
	{	
		if (myDlg.DoModal() == IDOK)
		{
			cslArgs.AddTail(myDlg.GetPathName());
			EmitMessage(CASSETTE_CHANGE, cslArgs);
			m_strRecorderFileName = myDlg.GetFileTitle();
			m_bRecorderMounted = TRUE;
			RedrawWindow();
		}
	}
	else
	{
		EmitMessage(CASSETTE_CHANGE, cslArgs);
		m_bRecorderMounted = FALSE;
		RedrawWindow();
	}
}


afx_msg LRESULT CTapeuiDlg :: OnIncoming(WPARAM /* wParam */, LPARAM lParam)
{
	POSITION	posP = (POSITION) lParam;
	CString		strTemp;

	strTemp = m_cslMessages.GetAt(posP);
	m_cslMessages.RemoveAt(posP);
	MessageDispatch(strTemp);
 	return 0L;
}

void	CTapeuiDlg :: MessageDispatch(CString strMsg)
{
	int			nCommandCode;
	CStringList	cslArgs;
	CString		strArg[MAX_ARG_COUNT];
	BOOL *		pFlag;
	CRect		rWnd;

	if (parse_message(strMsg, nCommandCode, cslArgs))
	{
		switch(nCommandCode)
		{
		case SHUTDOWN_NOW_CMD:
			m_bConnected = FALSE;
			OnClose();	//	pipes are going to close by themselves.
			break;
		case NORMAL_STOP:
			OnClose();
			break;
		case CASSETTE_LOCK:
			if (cslArgs.GetCount() == 2)
			{
				strArg[0] = cslArgs.RemoveHead();
				strArg[1] = cslArgs.RemoveHead();
				switch(atoi(strArg[0]))
				{
				case 0:
					pFlag = &m_bPlayLocked;
					break;
				case 1:
					pFlag = &m_bRecordLocked;
					break;
				default:
					AfxMessageBox("CASSETTE_LOCK: invalid arg[0] value.");
					return;
					break;
				}
				*pFlag = (atoi(strArg[1]) != 0);
				m_PlayLightFrame.GetWindowRect(&rWnd);
				ScreenToClient(&rWnd);
				RedrawWindow(&rWnd);
				m_RecordLightFrame.GetWindowRect(&rWnd);
				ScreenToClient(&rWnd);
				RedrawWindow(&rWnd);
			}
			else
			{
				AfxMessageBox("CASSETTE_LOCK: Wrong number of args.");
			}
			break;
		case CONTEXT_VALUE:
			if (cslArgs.GetCount() == 2)
			{
				strArg[0] = cslArgs.RemoveHead();
				strArg[1] = cslArgs.RemoveHead();
				switch(atoi(strArg[0]))
				{
				case 0:
					m_strH8Environment = strArg[1];
					m_bConnected = TRUE;

					RedrawWindow();
					break;
				default:
					AfxMessageBox("CONTEXT_VALUE: invalid arg[0] value.");
					break;
				}
			}
			else
			{
				AfxMessageBox("CONTEXT_VALUE: Wrong number of args.");
			}
			break;
		default:

//	Unrecognized message.  Ignore it.

			AfxMessageBox("Unrecognized message.");
			break;
		}
	}
	else
	{
		AfxMessageBox("Syntax error in message.");
	}
}

void	CTapeuiDlg :: EmitMessage(int nCommandCode, char * szArg)
{
	CStringList	cslArgs;

	cslArgs.RemoveAll();
	if (szArg != NULL) cslArgs.AddTail(szArg);
	EmitMessage(nCommandCode, cslArgs);
}


void	CTapeuiDlg :: EmitMessage(int nCommandCode, CStringList & cslArgs)
{
	CString	strMessage = make_message(nCommandCode, cslArgs);
	DWORD			dwErrno;
	char			szErrMsg[81];

//	If Pipe[0] is not connected or has been closed or is broken, do not attempt
//	to emit message, we're operating in stand-alone mode.

	if ((stPipeControl[0].hPipe == INVALID_HANDLE_VALUE) ||
		(stPipeControl[0].ePipeState == PIPESTATE_BROKEN) ||
		(stPipeControl[0].ePipeState == PIPESTATE_CLOSED)) return;

	if (strMessage.GetLength() > (sizeof(stPipeControl[0].szBuffer) - 3))
	{
		AfxMessageBox("Program error: EmitMessage -- attempting to send a "
						"message bigger than buffer size.");
	}
	else
	{

//	Ensure we own the outgoing message buffer before we proceed...

		WaitForSingleObject(stPipeControl[0].overControl.hEvent, INFINITE);

//	Initialize the buffer to padding character

		memset((void *) &(stPipeControl[0].szBuffer[0]), '\n',
				MAX_MESSAGE_LENGTH);
		strcpy(stPipeControl[0].szBuffer, strMessage);
		stPipeControl[0].dwBufferLength = strMessage.GetLength();
		if (WriteFile(stPipeControl[0].hPipe,
					  stPipeControl[0].szBuffer,
					  stPipeControl[0].dwBufferLength,
					  &stPipeControl[0].dwXferCount,
					  &stPipeControl[0].overControl) == FALSE)
		{
			if ((dwErrno = GetLastError()) !=  ERROR_IO_PENDING)
			{
				sprintf(szErrMsg, "Write to pipe failed.  Error was %d", dwErrno);
				AfxMessageBox(szErrMsg);
			}
		}
	}
}

BOOL CTapeuiDlg :: ConnectCommunications()
{
	UINT	dwThreadId;

	if (m_bConnected)
	{

//	Already connected -- don't do anything, just return TRUE.

		return TRUE;
	}

//	Initialize Pipe Control and data structures to "mostly harmless".

	memset(&stPipeControl[0], 0, sizeof(PIPE_CONTROL));
	stPipeControl[0].ePipeMode = PIPEMODE_UNDEFINED;
	memset (&stPipeControl[0].overControl, 0, sizeof(OVERLAPPED)); 
	stPipeControl[0].overControl.hEvent = INVALID_HANDLE_VALUE;
 	stPipeControl[0].hPipe = INVALID_HANDLE_VALUE;
	stPipeControl[0].ePipeState = PIPESTATE_CLOSED;

	memset(&stPipeControl[1], 0, sizeof(PIPE_CONTROL));
	stPipeControl[1].ePipeMode = PIPEMODE_UNDEFINED;
	memset (&stPipeControl[1].overControl, 0, sizeof(OVERLAPPED)); 
	stPipeControl[1].overControl.hEvent = INVALID_HANDLE_VALUE;
	stPipeControl[1].ePipeState = PIPESTATE_CLOSED;
 	stPipeControl[1].hPipe = INVALID_HANDLE_VALUE;

//	Open the outbound pipe for write using stPipeControl[0].
//	Open the inbound pipe ("<thing>.REC") for read using stPipeControl[1].
//	As the pipe is opened, wait for the pipe.  Time out after 10 seconds and
//	complain that the server appears to be not running.

	if (WaitNamedPipe(CASSETTE_TX, PIPE_DEFAULT_TIMEOUT) != TRUE)
	{
		return FALSE;
	}
	if ((stPipeControl[0].hPipe =
		CreateFile(CASSETTE_TX,
	   				GENERIC_WRITE,
	   				0,
	   				NULL,
       				OPEN_EXISTING,	// fail if no such pipe
	   				FILE_FLAG_OVERLAPPED,
	   				NULL)
		)  == INVALID_HANDLE_VALUE )
	{

//	Create failed.

		return FALSE;
	}

//	Outbound pipe created okay.  Set pipe control structure consistent with that.

	stPipeControl[0].ePipeMode = PIPEMODE_CLIENT_WRITE_OVERLAPPED;
	stPipeControl[0].ePipeState = PIPESTATE_IDLE;

//	Now open the inbound pipe.

	if (WaitNamedPipe(CASSETTE_RX,
  	  					PIPE_DEFAULT_TIMEOUT) != TRUE)
	{
		return FALSE;
	}
	if ((stPipeControl[1].hPipe =
		CreateFile(CASSETTE_RX,
	   				GENERIC_READ,
	   				0,
	   				NULL,
       				OPEN_EXISTING,	// fail if no such pipe
	   				FILE_FLAG_OVERLAPPED,
	   				NULL)
		)  == INVALID_HANDLE_VALUE )
	{
		DisconnectPipes();
		m_bConnected = FALSE;
		return FALSE;
	}

//	Inbound pipe created okay.  Set the pipe control structure for this pipe .

	stPipeControl[1].ePipeMode = PIPEMODE_CLIENT_READ_OVERLAPPED;
	stPipeControl[1].ePipeState = PIPESTATE_IDLE;

//	Create manually-reset Event #0 in signalled state and assign it to pipe #0.

	if (hEvent[0] == INVALID_HANDLE_VALUE)
	{
		if ((hEvent[0] = CreateEvent (NULL, TRUE, TRUE, NULL))
				== NULL )
		{
			AfxMessageBox("Unable to create an event for overlapped read of "
							"inbound pipe.");
			DisconnectPipes();
			m_bConnected = FALSE;
			return FALSE;
		}
	}
	stPipeControl[0].overControl.hEvent = hEvent[0];

//	Create manually-reset Event #1 in signalled state and assign it to pipe #1.

	if (hEvent[1] == INVALID_HANDLE_VALUE)
	{
		if ((hEvent[1] = CreateEvent (NULL, TRUE, TRUE, NULL))
				== NULL )
		{
			AfxMessageBox("Unable to create an event for overlapped write of "
							"outbound pipe.");
			DisconnectPipes();
			m_bConnected = FALSE;
			return FALSE;
		}
	}
	stPipeControl[1].overControl.hEvent = hEvent[1];

//	Create comm mutex object if not already created.
	
	if (hmCommMutex == NULL)
	{
		if ((hmCommMutex = CreateMutex(NULL, FALSE, MUTEX_NAME)) == NULL)
		{
			AfxMessageBox("Could not create Mutex object \""
						MUTEX_NAME
						"\"");
			DisconnectPipes();
			m_bConnected = FALSE;
			return FALSE;
		}
	}

//	Create auto-reset Event #2 in unsignalled state.  We signal event #2 if we need
//	to terminate the second thread.

	if (hEvent[2] == INVALID_HANDLE_VALUE)
	{
		if ((hEvent[2] =
				CreateEvent(NULL, FALSE, FALSE, NULL)) == NULL)
		{

/*	Could not create the event.	*/

			AfxMessageBox("Program error -- could not create an event to signal "
							"termination request to Thread 2 with.");
			DisconnectPipes();
			m_bConnected = FALSE;
			return FALSE;
		}
	}

//	Start thread #2 to receive messages from pipe and send WM_USER-based
//	messages to GUI code, assuming the thread is not already running.

	if (hInputThread == NULL)
	{
		if((hInputThread = (HANDLE)
			:: _beginthreadex(
					NULL,				// No security Attribs
	    			0L,					// Default stack
					&RxMsgThreadFunc,	// address of thread function
					&(AfxGetApp()->
						m_pMainWnd),	// argument for new thread 
					0,					// default creation flags (running, I hope!)
					&dwThreadId			// address of returned thread ID 
				)) == NULL)
		{
			AfxMessageBox("Program error -- Unable to Create Thread 2.");
			DisconnectPipes();
			m_bConnected = FALSE;
			return FALSE;
		}
	}
	m_bConnected = TRUE;
	return TRUE;
}


/*----------------------------------------------------------------------*
 |																		|
 |	DisconnectPipes:		Close both of our pipes and cause Thread #2	|
 |							to terminate itself.						|
 |																		|
 *----------------------------------------------------------------------*/

BOOL	CTapeuiDlg :: DisconnectPipes()
{
	BOOL		bResult = TRUE;

//	Terminate thread #2.  This will elliminate the read function for the pipes.

	if (hInputThread != NULL)
	{
		SetEvent(hEvent[2]);
		if (WaitForSingleObject(hInputThread,
						2000) == WAIT_FAILED)	// give thread 2 two seconds...
		{
			AfxMessageBox("Thread 2 did not exit within 2 sec.  "
							"Killing it anyway.");
			CloseHandle(hInputThread);
			hInputThread = NULL;
			bResult = FALSE;
		}
		hInputThread = NULL;
	}

//	Destroy three events and mutex objects

	if (hmCommMutex != NULL)
	{
		CloseHandle(hmCommMutex);
		hmCommMutex = NULL;
	}

	if (hEvent[2] != INVALID_HANDLE_VALUE)
	{
		CloseHandle(hEvent[2]);
		hEvent[2] = INVALID_HANDLE_VALUE;
	}

	if (hEvent[1] != INVALID_HANDLE_VALUE)
	{
		CloseHandle(hEvent[1]);
		hEvent[1] = INVALID_HANDLE_VALUE;
		stPipeControl[1].overControl.hEvent = INVALID_HANDLE_VALUE;
	}

	if (hEvent[0] != INVALID_HANDLE_VALUE)
	{
		CloseHandle(hEvent[0]);
		hEvent[0] = INVALID_HANDLE_VALUE;
		stPipeControl[0].overControl.hEvent = INVALID_HANDLE_VALUE;
	}

//	Disconnect from named pipes.  Destroy associated events.

	if (stPipeControl[0].hPipe != INVALID_HANDLE_VALUE)
	{
		CloseHandle(stPipeControl[0].hPipe);
		stPipeControl[0].hPipe = INVALID_HANDLE_VALUE;
		stPipeControl[0].ePipeState = PIPESTATE_CLOSED;
	}
	if (stPipeControl[0].overControl.hEvent != INVALID_HANDLE_VALUE)
	{
		CloseHandle(stPipeControl[0].overControl.hEvent);
		stPipeControl[0].overControl.hEvent = INVALID_HANDLE_VALUE;
	}
	if (stPipeControl[1].hPipe != INVALID_HANDLE_VALUE)
	{
		CloseHandle(stPipeControl[1].hPipe);
		stPipeControl[1].hPipe = INVALID_HANDLE_VALUE;
		stPipeControl[1].ePipeState = PIPESTATE_CLOSED;
	}
	if (stPipeControl[1].overControl.hEvent != INVALID_HANDLE_VALUE)
	{
		CloseHandle(stPipeControl[1].overControl.hEvent);
		stPipeControl[1].overControl.hEvent = INVALID_HANDLE_VALUE;
	}
	return bResult;
}


/*----------------------------------------------------------------------+
 |																		|
 |	RxMsgThreadFunc:	Message Receiver Thread (known elsewhere as		|
 |						"thread #2" -- gets messages.
 |																		|
 |		The Message Receiver code must send a Windows user message so	|
 |	that the GUI thread can do the actual work.							|
 |																		|
 +----------------------------------------------------------------------*/

UINT __stdcall	RxMsgThreadFunc(void * pParams)
{
	{
		DWORD			dwIndex;
		DWORD 			dwErrno;
		POSITION		posP;
		CTapeuiDlg *	pDlg = *((CTapeuiDlg **) pParams);

		while(1)
		{
			dwIndex = WaitForMultipleObjects(2, &hEvent[1], FALSE, INFINITE);
			if (dwIndex == WAIT_OBJECT_0 + 0)
			{

//		The pipe read has completed or needs to be done.

				switch(stPipeControl[1].ePipeState)
				{
				case PIPESTATE_CLOSED:
					AfxMessageBox("Program error -- Thread 2 attempted to "
									"read from closed pipe.",
									MB_TOPMOST);
					pDlg ->	m_bConnected = FALSE;
					pDlg -> RedrawWindow();
					:: _endthreadex(0);
					break;
				case PIPESTATE_IDLE:

//	Do a read -- this is usual for first wake-up.

					if (ReadFile(stPipeControl[1].hPipe,
							 stPipeControl[1].szBuffer,
							 MAX_MESSAGE_LENGTH,
							 &stPipeControl[1].dwXferCount,
							 &stPipeControl[1].overControl) == FALSE)
					{
						if ((dwErrno = GetLastError()) != ERROR_IO_PENDING)
						{

//	The pipe got an error.  Assume it broke.

							stPipeControl[1].ePipeState = PIPESTATE_BROKEN;
							pDlg ->	m_bConnected = FALSE;
							pDlg -> RedrawWindow();
						}
						else
						{

//	The read is incomplete.

							stPipeControl[1].ePipeState = PIPESTATE_XFER;
						}
					}
					else
					{

//	The read completed on first try.

						stPipeControl[1].ePipeState = PIPESTATE_READY;
					}
					break;
				case PIPESTATE_XFER:

//	Check OverlappedResult -- if done, fine.

 					if (GetOverlappedResult(stPipeControl[1].hPipe,
										&stPipeControl[1].overControl,
										&stPipeControl[1].dwXferCount,
										FALSE) == FALSE)
					{
				 		if ((dwErrno = GetLastError()) == ERROR_IO_INCOMPLETE) 
						{

//	Incomplete is okay, though it's strange we woke up.  Go back to sleep.

							break;
						}
						else
						{

//	The pipe broke.

							stPipeControl[1].ePipeState = PIPESTATE_BROKEN;
							pDlg ->	m_bConnected = FALSE;
							pDlg -> RedrawWindow();
						}
					}
					else
					{

//	The IO is complete.  Set Pipe State to READY.

						stPipeControl[1].ePipeState = PIPESTATE_READY;
					}
					break;
				case PIPESTATE_READY:
					stPipeControl[1].szBuffer[stPipeControl[1].dwXferCount] = 0;
					strcpy(szInMessageTxt, stPipeControl[1].szBuffer);

//	Post message to data packet dispatcher.

					pDlg -> m_cslMessages.AddTail(szInMessageTxt);
					posP = pDlg -> m_cslMessages.GetTailPosition();

//	Pass the decoding job to the primary thread.  PostMessage will "stack" the
//	command, and each command has its own message buffer.

					if ((*((CWnd **) pParams)) != NULL)
					{
						if (IsWindow((*((CWnd **) pParams)) -> m_hWnd))
						{
							(*((CWnd **) pParams)) ->
								PostMessage(UM_INCOMING, 0, (LPARAM) posP);
							Sleep(0);
						}
					}

//	Now that we've disposed of the data, set the pipe state to IDLE to
//	trigger another read operation.

					stPipeControl[1].ePipeState = PIPESTATE_IDLE;
					break;
				case PIPESTATE_BROKEN:

//	The pipe's busted.  Must die.

					stPipeControl[1].ePipeState = PIPESTATE_BROKEN;
					pDlg ->	m_bConnected = FALSE;
					pDlg -> RedrawWindow();
					:: _endthreadex(0);
					break;
				default:

//	How did we ever get here?

					AfxMessageBox("Program error -- incorrect enum value in "
									"pipe control structure.",
									MB_SETFOREGROUND);
					stPipeControl[1].ePipeState = PIPESTATE_IDLE;
					break;
				}
			}
			else if (dwIndex == WAIT_OBJECT_0 + 1)
			{

//		We've been commanded to exit.

				pDlg ->	m_bConnected = FALSE;
				pDlg -> RedrawWindow();
				:: _endthreadex(0);
			}
			else if (dwIndex == WAIT_FAILED)
			{
				if ((dwIndex = GetLastError()) == ERROR_INVALID_HANDLE)
				{

//		This can happen when we do a stop server.  just exit the thread.

					pDlg ->	m_bConnected = FALSE;
					pDlg -> RedrawWindow();
					:: _endthreadex(0);
				}
				else
				{

//		This wait failure is unplanned for

					char szWaitFailureMessage[MAX_PATH];
					sprintf(szWaitFailureMessage,
						"Program error -- wait failed.  Error was %lu",
						GetLastError());
					AfxMessageBox(szWaitFailureMessage,
									MB_TOPMOST);
					pDlg ->	m_bConnected = FALSE;
					pDlg -> RedrawWindow();
					:: _endthreadex(0);
				}
			}
			else
			{

//	can't happen

				AfxMessageBox("Program error -- event index out of range.",
								MB_TOPMOST);
				pDlg ->	m_bConnected = FALSE;
				pDlg -> RedrawWindow();
				:: _endthreadex(0);
			}
		}
		pDlg ->	m_bConnected = FALSE;
		pDlg -> RedrawWindow();
	}
	:: _endthreadex(0);
	return 0L;
}



void CTapeuiDlg::OnClose() 
{
	if (m_bConnected)
	{
		EmitMessage(APP_IS_CLOSING);
		Sleep(1000);
		DisconnectPipes();
	}
	SetEvent(hEvent[2]);
	CDialog::OnCancel();
}
