/************************************************************************
 *																		*
 *	Copyright (c) 1999 by			David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/


#include "stdafx.h"

#define	EXTERN	extern

#include "virtual_h8.h"
#include "RegionDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// RegionDlg dialog


RegionDlg::RegionDlg(CWnd* pParent /*=NULL*/)
	: CDialog(RegionDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(RegionDlg)
	m_bEnabledOnCreate = FALSE;
	m_strEnd = _T("");
	m_strStart = _T("");
	//}}AFX_DATA_INIT
	m_bShowCheckbox = FALSE;
}


void RegionDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(RegionDlg)
	DDX_Control(pDX, IDC_ENABLED_CK, m_RegionCk);
	DDX_Check(pDX, IDC_ENABLED_CK, m_bEnabledOnCreate);
	DDX_Text(pDX, IDC_REGION_END, m_strEnd);
	DDX_Text(pDX, IDC_REGION_START, m_strStart);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(RegionDlg, CDialog)
	//{{AFX_MSG_MAP(RegionDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// RegionDlg message handlers

void RegionDlg::OnOK() 
{
	int		nOffset;
	CString	strTemp;

	if (UpdateData(TRUE))
	{

//	Validate that the start address is good split-octal.

		if (!ValidateSplitOctal(m_strStart))
		{
			AfxMessageBox("Invalid Split-Octal Address Format.");
			GotoDlgCtrl(GetDlgItem(IDC_REGION_START));
		}
		else
		{
			m_nStartAddress = SplitOctalToDecimal(m_strStart);

//	Validate that the end address is either good split octal or that the
//	value in the field, when added to the start address produces good split
//	octal.  (If so, add it so that the end is an address instead of a number.)

			if (ValidateSplitOctal(m_strEnd))
			{
				m_nEndAddress = SplitOctalToDecimal(m_strEnd);
			}
			else if (((nOffset = atoi(m_strEnd)) > 0) && (nOffset <= 65535))
			{
				strTemp = AddOffsetToAddress(m_strStart, m_strEnd);
				if (SplitOctalToDecimal(strTemp) <
					SplitOctalToDecimal(m_strStart))
				{
					AfxMessageBox("Invalid End Address or Offset value.");
					GotoDlgCtrl(GetDlgItem(IDC_REGION_END));
					return;
				}
				else m_nEndAddress = SplitOctalToDecimal(strTemp);
			}
			else
			{
				AfxMessageBox("Invalid End Address or Offset value.");
				GotoDlgCtrl(GetDlgItem(IDC_REGION_END));
				return;
			}
			CDialog::OnOK();
		}
	}
}

BOOL RegionDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	if (m_bShowCheckbox)
	{
		m_RegionCk.ShowWindow(SW_SHOW);
	}
	else
	{
		m_RegionCk.ShowWindow(SW_HIDE);
	}
	UpdateData(FALSE);
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


BOOL		RegionDlg :: ValidateSplitOctal(CString strValue)
{
	char	c;
	int		nIndex;

	if (strValue.GetLength() < 7) return FALSE;
	for (nIndex = 0; nIndex <  7; ++nIndex)
	{
		c = strValue.GetAt(nIndex);
		if (nIndex == 3)
		{
			if (c != '.')
			{
				return FALSE;
			}
			else
			{
				continue;
			}
		}
		else if ((nIndex == 0) || (nIndex == 4))
		{
			if ((c < '0') || (c > '3'))
			{
				return FALSE;
			}
			else
			{
				continue;
			}
		}
		else if ((c < '0') || (c > '7'))
		{
			return FALSE;
		}
	}
	return TRUE;
}


CString		RegionDlg :: AddOffsetToAddress(CString strAddress,
											CString strOffset)
{
	int		nAddr = SplitOctalToDecimal(strAddress);
	int		nInc = atoi((LPCSTR) strOffset);
	int		nValue;

	nValue = nAddr + nInc;
	return DecimalToSplitOctal(nValue);
}


CString		RegionDlg :: DecimalToSplitOctal(int nValue)
{
	char	c[8];
	CString	strTemp;

	nValue = nValue & 0x00FFFF;

	c[0] = (char) ((nValue >> 14) + '0');
	c[1] = (char) (((nValue >> 11) % 8) + '0');
	c[2] = (char)(((nValue >> 8) % 8) + '0');
	c[3] = '.';
	c[4] = (char)(((nValue >> 6) % 4) + '0');
	c[5] = (char)(((nValue >> 3) % 8) + '0');
	c[6] = (char)((nValue % 8) + '0');
	c[7] = 0;
	strTemp = c;
	return strTemp;
}


int			RegionDlg::SplitOctalToDecimal(CString strValue)
{
	char	c[8];
	int		n;

	strcpy(c, (LPCSTR) strValue);

	n = ((c[0] - '0') << 14) +
		((c[1] - '0') << 11) +
		((c[2] - '0') << 8) +
		((c[4] - '0') << 6) +
		((c[5] - '0') << 3) +
		(c[6] - '0');
	return n;
}
