/************************************************************************
 *																		*
 *	H-19 EMULATOR Application (H-8 EMULATOR Project) --	SCREENWINDOW.H	*
 *																		*
 *	This object manages the screen display and does character I/O	 	*
 *	translation (including escape sequences).  Despite the name (which	*
 *	implies that it's the display side of the emulation), this module	*
 *	is the "guts" of the terminal emulator.								*
 *																		*
 *	------------------------------------------------------------------	*
 *																		*
 *	Copyright (c) 2000 - 2001 by	David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/


#ifndef _SCREENWINDOW_H_
#define	_SCREENWINDOW_H_

/////////////////////////////////////////////////////////////////////////////
// ScreenWindow window

#define	CUSOR_TIMER_PERIOD	250		//	H19 cursor blinks twice per second

//	The H19 characters were 8 wide by 10 high.  In PC displays, we need to make
//	them twice as high in order for the letters have the correct aspect ratio.
//	So we'll simply stretch the bitmap vertically by a factor of 2.

#define	CHAR_GEN_HEIGHT	10
#define CHAR_GEN_WIDTH	8
#define	CHAR_WIDTH		8
#define	CHAR_HEIGHT		(2 * 10)
#define	SCREEN_WIDTH	(80 * CHAR_WIDTH)
#define	SCREEN_HEIGHT	(25 * CHAR_HEIGHT)
#define SCREEN_VERT_BORDER	9
#define SCREEN_HORIZ_BORDER	9


#define	ATTRIB_INVERSE	0x01	//	inverse video
#define ATTRIB_GRAPHIC	0x02	//	graphic character
#define ATTRIB_GREEN_FG	0x10	//	"enhancement" -- use color like Z-100 did
#define	ATTRIB_RED_FG	0x08	//	"enhancement" -- use color like Z-100 did
#define	ATTRIB_BLUE_FG	0x04	//	"enhancement" -- use color like Z-100 did
#define ATTRIB_GREEN_BG	0x80	//	"enhancement" -- use color like Z-100 did
#define	ATTRIB_RED_BG	0x40	//	"enhancement" -- use color like Z-100 did
#define	ATTRIB_BLUE_BG	0x20	//	"enhancement" -- use color like Z-100 did

//	Some important control-character codes:

#define	BEL				0x007
#define	BS				0x008
#define	HT				0x009
#define	LF				0x00A
#define	VT				0x00B
#define	FF				0x00C
#define	CR				0x00D
#define	XON				0x011
#define	XOFF			0x013
#define	CAN				0x018
#define	ESC				0x01B
#define	DEL				0x07F


enum TTY_STATE {
	BASE = 0,
	ESCAPE,
	ARG1,
	ARG2
};

class ScreenWindow : public CWnd
{
//	Construction / Destruction:

public:
	ScreenWindow();
	virtual ~ScreenWindow();

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(ScreenWindow)
	//}}AFX_VIRTUAL

	// Generated message map functions
protected:
	//{{AFX_MSG(ScreenWindow)
	afx_msg void OnPaint();
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()




//	Methods:

public:
	void	CharacterIn(char cData);
	void	CharacterOut(char cData, BOOL bInterlock = FALSE);
	void	SendBreak();
	void	ReceiveBreak();
	void	RedrawCursor();
	void	Keystroke(int nCode);
	void	RepaintScreen();
	void	SetPowerupConfiguration(BOOL bRedraw = TRUE);

#ifdef _DEBUG
	void	DisplayTest();
#endif
protected:

private:
	CRect	CharacterGeneratorCoordinates(char cData, char cAttribute);
	CRect	DisplayCoordinates(int nLine, int nChar);
	void	DrawAt(int nLine, int nColumn, BOOL bRepaint = TRUE);
	void	RepaintLine(int nLine, BOOL bRepaint = TRUE);
	void	HomeCursor();
	void	SetCursorPosition(int nLine, int nColumn);
	void	SaveCursorPosition();
	void	RestoreCursorPosition();
	void	CursorRight();
	void	CursorLeft();
	void	CursorDown();
	void	CursorUp();
	void	ClearToEndOfScreen(BOOL bRedraw = TRUE);
	void	ClearToStartOfScreen();
	void	ClearToEndOfLine();
	void	ClearToStartOfLine();
	void	ClearLine(BOOL bRedraw = TRUE);
	void	ScrollUp();
	void	InsertLine();
	void	DeleteLine();
	void	DeleteCharacter();
	void	WriteToDisplay(char cData);
	void	SendOut(char cData, BOOL bInterlock);
	void	MoveScreen(int nFirstLine, int nLastLine, int nDirection);
	void	TransmitCharacter(int nLine,
								int nColumn,
								BOOL bInterlock = FALSE,
								BOOL bUseHistory = FALSE);
	void	TransmitLine(int nLine);
	void	TransmitPage();

#ifdef _DEBUG
	void	SetTextLine(int nLine, char * szData);
	void	SetInverseTextLine(int nLine, char * szData);
#endif


// Attributes:

public:
	BOOL	m_bReverseVideo;		//	Render entire display black-on-white
	double	m_dMagnification;		//	Magnify the window and font size by this
	BOOL	m_bCursorBlinkState;
	BOOL	m_bLocal;				//	"online" button is up
	BOOL	m_bShiftedMode;			//	Keypad shifted
	BOOL	m_bAlternateMode;		//	Keypad alternate
	BOOL	m_bAnsiMode;			//	ANSI escape sequences
	BOOL	m_bICSent;				//	IC escape was sent; send Not-IC escape next time
	BOOL	m_bInsertCharMode;		//	IC escape was received.
protected:

private:
	CBitmap	m_bmpCharacterSet;
	char	m_cRaster[25][80][2];	//  25 x 80 display plus attribute bits.
	UINT	m_nBG;					//	Default background color index
	UINT	m_nFG;					//	Default foreground color index
	int		m_nCursorLine;
	int		m_nCursorColumn;

//	The H-19 has several "mode" settings, which can occur in combinations:

	BOOL	m_bGraphicsMode;		//	Incoming characters are graphics
	BOOL	m_bWrapMode;			//	Wrap lines
	BOOL	m_bAutoCR;				//	Perform CR on receipt of LF
	BOOL	m_bAutoLF;				//	Perform LF on receipt of CR
	BOOL	m_b25thLineMode;		//	25th line is on
	BOOL	m_bInverseVideoMode;	//	This is distinct from "reverse video"
	BOOL	m_bHoldScreenMode;		//	scrolling locked
	BOOL	m_bShowCursor;			//	Show / Hide cursor
	BOOL	m_bCursorIsBlock;		//	Cursor is Block / Underline
	int		m_nSavedCursorLine;
	int		m_nSavedCursorColumn;
	BOOL	m_bMemDCCreated;		//	flags when we need to create the memDC
	CDC 	m_MemDC;				//	the memDC for display
	CDC 	m_ChargenDC;			//	the memDC for the character generator
	CDC 	m_CellDC;				//	the memDC for a single character
	CDC		m_GreenDC;				//	the memDC needed for "green screen" mode
	CDC		m_FgDC;					//	the memDC used by color mode for foreground
	CDC		m_BgDC;					//	the memDC used by color mode for background
	BOOL	m_bInhibitCursor;		//	temp. suppress cursor
	int		m_nPlanes;				//	color depth -- planes
	int		m_nBits;				//	color depth -- bits
	BOOL	m_bGreenCRT;			//	Simulate a Green Phosphor (H19A) CRT
	BOOL	m_bEnhancedEmulation;	//	Allow extra graphic characters and color
};

/////////////////////////////////////////////////////////////////////////////
#endif	//	#ifndef _SCREENWINDOW_H_
