/************************************************************************
 *																		*
 *	Copyright (c) 1998 - 2001 by	David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/


#include "stdafx.h"
#include "uart.h"
#include "bus.h"
#include "Virtual_h8.h"
#include "Virtual_h8Doc.h"
#include "Virtual_h8View.h"
#define EXTERN extern
#include "globals.h"

		Bus :: Bus()
{
	InitializeCriticalSection(&m_critBus);
	InitializeCriticalSection(&m_critTime);

//	The following signals are only initialized at power-up time, never on reset:

	m_bSuspend =				TRUE;
	m_hTimeRuns = CreateEvent(NULL, TRUE, TRUE, NULL);
	m_bH19DCD = FALSE;
	m_bH19bRI = FALSE;
	m_bH19DSR = FALSE;
	m_bH19CTS = FALSE;
	m_bH19CauseBreak = FALSE;
	m_nH19Cursor = 0;
	m_hH19 = CreateEvent(NULL, TRUE, TRUE, NULL);
	m_bH19DTR = FALSE;
	m_bH19OUT1 = FALSE;
	m_bH19OUT2 = FALSE;
	m_bH19RTS = FALSE;
	m_bH19SendBreak = FALSE;

//	Call reset method to initialize the remaining signals.

	Reset();				
}


		Bus :: ~Bus()
{
	DeleteCriticalSection(&m_critBus);
	DeleteCriticalSection(&m_critTime);
	CloseHandle(m_hTimeRuns);
	CloseHandle(m_hH19);
}


void	Bus :: Reset()
{

//	These signals are reset every time there is a master clear:

//	synchronization  signals:

	m_bReset =					FALSE;
	m_bNewInput =				FALSE;
	m_bNewInterruptResponse =	FALSE;

//	Interrupt requests:

	m_bINT10 =					FALSE;
	m_bRTMSignal =				FALSE;
	m_bINT20 =					FALSE;
	m_bINT30 =					FALSE;
	m_bINT40 =					FALSE;
	m_bINT50 =					FALSE;
	m_bINT60 =					FALSE;
	m_bINT70 =					FALSE;
	m_bInterruptRequest =		FALSE;

//	address and data bus:

	m_nBusAddress =				0;
	m_nBusReadData =			0;
	m_nBusWriteData =			0;
	
//	cycle type control signals:

	m_bIORead =					FALSE;
	m_bIOWrite =				FALSE;
	m_bIntAck =					FALSE;
	
//	processor status signals:

	m_bInterruptsEnabled =		FALSE;
	m_bRun =					FALSE;
	m_bM1 =						FALSE;

//	These signals are not "connected":

	m_bNMI =					FALSE;
	m_bSIN =					FALSE;
	m_bSOUT =					FALSE;
	m_bRST5_5 =					FALSE;
	m_bRST6_5 =					FALSE;
	m_bRST7_5 =					FALSE;
	m_bH19CauseBreak =			FALSE;
	m_nH19Cursor =					0;
	m_cslH19Messages.RemoveAll();
	m_ConsoleUART.Reset();
}


void	Bus :: SetInterruptRequests()
{

//	This code looks at all peripheral device simulations that can signal an
//	interrupt request and sets the interrupt request for that device to the
//	state of the device's interrupt request signal.

//	m_bINT10 = (real-time interrupt; done elsewhere.)
//	m_bINT20 = (single-step interrupt; done elsewhere.)
	m_bINT30 = m_ConsoleUART.InterruptSignal();
//	m_bINT40 = (nonstandard)
//	m_bINT50 = m_H47.InterruptState();
//	m_bINT60 = (nonstandard)
//	m_bINT70 = (reserved -- RST7 is HDOS SYSCALL macro.)

}


void	Bus :: UpdateSerialIO()
{
	char	cRx = 0;
	char	cTx = 0;
	BOOL	bUpdateData;
	CString	strMessage;

	EnterCriticalSection(&m_critBus);

//	This code looks at all serial device simulations and causes data exchange
//	with the corresponding UART if there is pending IO in either direction.

//	1.	Console UART and message stream

	bUpdateData = (m_bH19CauseBreak || !m_cslH19Messages.IsEmpty());
	if (bUpdateData)
	{
		if (WaitForSingleObject(m_hH19, 0) != WAIT_TIMEOUT)
		{

//		get the character (or maybe CauseBreak is set)

			if (!m_bH19CauseBreak)
			{

//	Get the current message and the character at the current cursor.
			
				strMessage = m_cslH19Messages.GetHead();
				cRx = strMessage.GetAt(m_nH19Cursor);
				++m_nH19Cursor;
				if (m_nH19Cursor >= strMessage.GetLength())
				{

//	Now at end of message.  Remove message at head. Set cursor to zero.
//	Schedule 33 ms delay.

					m_cslH19Messages.RemoveHead();
					m_nH19Cursor = 0;
					if (!theApp.ScheduleWakeup(&m_hH19, 33))
					{
						AfxMessageBox("Schedule 33ms delay failed.");
					}
				}
				else
				{

//	Schedule 1ms delay

					if (!theApp.ScheduleWakeup(&m_hH19, 1))
					{
						AfxMessageBox("Schedule 1ms delay failed.");
					}
				}
			}
			else
			{
				cRx = 0;
			}			
			m_ConsoleUART.UpdateUARTFromDevice(TRUE,	//	always update modem bits
												m_bH19DCD,
												m_bH19bRI,
												m_bH19DSR,
												m_bH19CTS,
												bUpdateData,
												cRx,
												FALSE,	//	m_bH19CauseOE,
												FALSE,	//	m_bH19CausePE,
												FALSE,	//	m_bH19CauseFE,
												m_bH19CauseBreak);
			m_bH19CauseBreak = FALSE;
		}
	}
	if (m_ConsoleUART.UpdateDeviceFromUART(m_bH19DTR,
											m_bH19RTS,
											m_bH19OUT1,
											m_bH19OUT2,
											cTx,
											m_bH19SendBreak))
	{
		if (m_bH19SendBreak)
		{
			((CVirtual_h8View *) theApp.m_pViewWnd) ->
				PostMessage(UM_BREAK_TO_H19, 0, 0);
		}
		else
		{
			((CVirtual_h8View *) theApp.m_pViewWnd) ->
				PostMessage(UM_CHAR_TO_H19, (WPARAM) cTx, 0);
		}
	}

//	2.	[TBD]

	LeaveCriticalSection(&m_critBus);
}


void	Bus :: H19MessageIn(CString strText)
{
	if (!strText.IsEmpty())
	{
		EnterCriticalSection(&m_critBus);
		m_cslH19Messages.AddTail(strText);
		LeaveCriticalSection(&m_critBus);
	}
}


void	Bus :: H19Break()
{
	EnterCriticalSection(&m_critBus);
	m_bH19CauseBreak = TRUE;
	LeaveCriticalSection(&m_critBus);
}