/************************************************************************
 *																		*
 *	Copyright (c) 1999 - 2001 by	David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/


#include "stdafx.h"

#define	EXTERN extern
#include "h8types.h"
#include "resource.h"
#include "virtual_h8.h"
#include "globals.h"
#include "processor.h"
#include "bus.h"
#include "pam.h"
#include "display.h"
#include "panel.h"
#include "logicanalyzer.h"
#include "regiondlg.h"


//	Global functions:


CString	SplitOctal(int n)
{
	CString strTemp;

	int	i[2];
	i[0] = (n >> 8) & 0x00FF;
	i[1] = n &0x00FF;
	strTemp.Format("%s.%s", (LPCSTR) Octal(i[0]), (LPCSTR) Octal(i[1]));
	return strTemp;
}


CString	Octal(int n)
{
	CString	strTemp;
	int i[3];

	i[0] = (n >> 6) & 0x0003;
	i[1] = (n >> 3) & 0x0007;
	i[2] = n & 0x0007;
	strTemp.Format("%d%d%d", i[0], i[1], i[2]);
	return strTemp;
}


BOOL	IsBranchOp(int n)
{
	switch(n & 0x00FF)
	{
	case 0300:			//	RNZ
	case 0302:			//	JNZ
	case 0303:			//	JMP
	case 0304:			//	CNZ
	case 0307:			//	RST 0
	case 0310:			//	RZ
	case 0311:			//	RET
	case 0312:			//	JZ
	case 0314:			//	CZ
	case 0315:			//	CALL
	case 0317:			//	RST	1
	case 0320:			//	RNC
	case 0322:			//	JNC
	case 0324:			//	CNC
	case 0327:			//	RST	2
	case 0330:			//	RC
	case 0332:			//	JC
	case 0334:			//	CC
	case 0337:			//	RST 3
	case 0340:			//	RPO
	case 0342:			//	JPO
	case 0344:			//	CPO
	case 0347:			//	RST 4
	case 0350:			//	RPE
	case 0351:			//	PCHL
	case 0352:			//	JPE
	case 0354:			//	CPE
	case 0357:			//	RST 5
	case 0360:			//	RP
	case 0362:			//	JP
	case 0364:			//	CP
	case 0367:			//	RST 6
	case 0370:			//	RM
	case 0372:			//	JM
	case 0374:			//	CM
	case 0377:			//	RST 7
		return TRUE;
		break;
	default:
		return FALSE;
		break;
	}
}


void	MemoryDump()
{
}

void	TraceBus(BOOL bReaction)
{
	int			nAddr;
	int			nData;
	static int	nCycle = 0;

	if (theApp.m_Bus.m_bReset) nCycle = 0;
	if (g_bLACycleTrace)
	{
		fprintf(tfp, "Cycle %d%c:\n", nCycle, bReaction? 'b' : 'a');
		if (bReaction) ++nCycle;
		fprintf(tfp, "\tRESET INT10 INT20 INT30 INT40 INT50 INT60 INT70 NMI RST5.5 RST6.5 RST7.5 IEN IRQ\n");
		fprintf(tfp, "\t  %d     %d     %d     %d     %d     %d     %d     %d    %d    %d      %d      %d     %d   %d\n",
				theApp.m_Bus.m_bReset, theApp.m_Bus.m_bINT10, theApp.m_Bus.m_bINT20, theApp.m_Bus.m_bINT30, theApp.m_Bus.m_bINT40, theApp.m_Bus.m_bINT50, theApp.m_Bus.m_bINT60, theApp.m_Bus.m_bINT70,
				 theApp.m_Bus.m_bNMI, theApp.m_Bus.m_bRST5_5, theApp.m_Bus.m_bRST6_5, theApp.m_Bus.m_bRST7_5, theApp.m_Bus.m_bInterruptsEnabled, theApp.m_Bus.m_bInterruptRequest);
		fprintf(tfp, "\tIACK IOR IOW RUN SIN SOUT NewInput NewIntResp Suspend Address ReadData WriteData\n");
		fprintf(tfp, "\t %d    %d   %d   %d   %d    %d      %d          %d       %d     %3s     %3s      %3s\n",
				theApp.m_Bus.m_bIntAck, theApp.m_Bus.m_bIORead, theApp.m_Bus.m_bIOWrite, theApp.m_Bus.m_bRun, theApp.m_Bus.m_bSIN, theApp.m_Bus.m_bSOUT,
				theApp.m_Bus.m_bNewInput, theApp.m_Bus.m_bNewInterruptResponse, theApp.m_Bus.m_bSuspend, (LPCSTR) Octal((theApp.m_Bus.m_nBusAddress) & 0x00FF), (LPCSTR) Octal(theApp.m_Bus.m_nBusReadData), (LPCSTR) Octal(theApp.m_Bus.m_nBusWriteData));
		fflush(tfp);
	}
	else if (!bReaction)
	{
		nAddr = theApp.m_Bus.m_nBusAddress;
		if ((tfp != NULL) && (g_bLAIRQCycleTrace) && theApp.m_Bus.m_bInterruptRequest)
		{

//	Log the interrupt request cycle.

			fprintf(tfp, "Interrupt request cycle initiated.  "
							"Current bus interrupt state is: %s\n",
						(LPCSTR) DecodeBusInterruptState());
			fflush(tfp);
		}
		if ((tfp != NULL) && (g_bLAIACKCycleTrace) && theApp.m_Bus.m_bIntAck)
		{

//	Log the interrupt ACK cycle.

			fprintf(tfp, "Interrupt Ack cycle: %s synthesized\n",
						(LPCSTR) Octal(theApp.m_Bus.m_nBusReadData));
			fflush(tfp);
		}
		if ((tfp != NULL) && (g_bLAIOReadTrace) && theApp.m_Bus.m_bIORead)
		{

//	Log the IO Read cycle.

			nData = theApp.m_Bus.m_nBusReadData;
			fprintf(tfp, "Input from port %s\tData = %s\n",
						(LPCSTR) Octal(nAddr),
						(LPCSTR) Octal(nData));
			fflush(tfp);
		}
	}
	else if (bReaction)
	{
		nAddr = theApp.m_Bus.m_nBusAddress;
		if ((tfp != NULL) && (g_bLAIOWriteTrace) && theApp.m_Bus.m_bIOWrite)
		{

//	Log the IO write cycle.

			nData = theApp.m_Bus.m_nBusWriteData;
			fprintf(tfp, "Output to port %s\tData = %s\n",
						(LPCSTR) Octal(nAddr),
						(LPCSTR) Octal(nData));
			fflush(tfp);
		}
	}
}


void	Trace()
{

//	This routine handles all instruction-fetch-level trace operations.

//	First, we see if the trace flag is set in memory.  This takes precedence
//	over the "global" option to trace branches, since the debug trace saves a
//	more complete snapshot of the processor state.

	if (theApp.m_Processor.m_MemMap.GetDebugFlags(theApp.m_Processor.PC) & DEBUG_TRACE_FLAG)
	{
		fprintf(tfp, "(PC) = %s\tData = %s %s %s\n"
						"\t\t\t(A) = %s, (BC) = %s, (DE) = %s\n"
						"\t\t\t(HL) = %s, (SP) = %s, (M) = %s %s\n",
				(LPCSTR) SplitOctal(theApp.m_Processor.PC),
				(LPCSTR) Octal(theApp.m_Processor.m_MemMap.Read(theApp.m_Processor.PC)),
				(LPCSTR) Octal(theApp.m_Processor.m_MemMap.Read((PWORD)(theApp.m_Processor.PC + 1))),
				(LPCSTR) Octal(theApp.m_Processor.m_MemMap.Read((PWORD)(theApp.m_Processor.PC + 2))),
				(LPCSTR) Octal(theApp.m_Processor.A),
				(LPCSTR) SplitOctal(theApp.m_Processor.rpBC.regpair),
				(LPCSTR) SplitOctal(theApp.m_Processor.rpDE.regpair),
				(LPCSTR) SplitOctal(theApp.m_Processor.rpHL.regpair),
				(LPCSTR) SplitOctal(theApp.m_Processor.SP),
				(LPCSTR) Octal(theApp.m_Processor.m_MemMap.Read(theApp.m_Processor.rpHL.regpair)),
				(LPCSTR) Octal(theApp.m_Processor.m_MemMap.Read((PWORD)
													(theApp.m_Processor.rpHL.regpair + 1)))
				);
		fflush(tfp);
	}
	else
	{
		if (g_bLABranchTrace)
		{

//	We should snapshot the opcode if it's a branch-type operation.

			if (IsBranchOp(theApp.m_Processor.m_MemMap.Read(theApp.m_Processor.PC)))
			{
				fprintf(tfp, "(PC) = %s\tData = %s %s %s\n"
								"\t\t\t(A) = %s, (SP) = %s\n",
					(LPCSTR) SplitOctal(theApp.m_Processor.PC),
					(LPCSTR) Octal(theApp.m_Processor.m_MemMap.Read(theApp.m_Processor.PC)),
					(LPCSTR) Octal(theApp.m_Processor.m_MemMap.Read((PWORD)(theApp.m_Processor.PC + 1))),
					(LPCSTR) Octal(theApp.m_Processor.m_MemMap.Read((PWORD)(theApp.m_Processor.PC + 2))),
					(LPCSTR) Octal(theApp.m_Processor.A),
					(LPCSTR) SplitOctal(theApp.m_Processor.SP),
					(LPCSTR) Octal(theApp.m_Processor.m_MemMap.Read(theApp.m_Processor.rpHL.regpair)),
					(LPCSTR) Octal(theApp.m_Processor.m_MemMap.Read((PWORD)
														(theApp.m_Processor.rpHL.regpair + 1)))
					);
				fflush(tfp);
			}
		}
		if ((g_bLAIRQCycleTrace || g_bLAIACKCycleTrace || g_bLAIENCycleTrace) && 
			((theApp.m_Processor.m_MemMap.Read(theApp.m_Processor.PC) == 0373) ||
			(theApp.m_Processor.m_MemMap.Read(theApp.m_Processor.PC) == 0363)))
		{
			fprintf(tfp, "(PC) = %s\tOpcode = %s\n",
					(LPCSTR) SplitOctal(theApp.m_Processor.PC),
					(theApp.m_Processor.m_MemMap.Read(theApp.m_Processor.PC) == 0363) ? "DI" : "EI");
				fflush(tfp);
		}
	}

//	finally, we should trap if the trap flag is set.

	if (theApp.m_Processor.m_MemMap.GetDebugFlags(theApp.m_Processor.PC) & DEBUG_TRAP_FLAG)
	{
	}
}



CString	DecodeBusInterruptState()
{
	CString	strRet;
	
	strRet = "";
	if (theApp.m_Bus.m_bINT10) strRet += "INT10 ";
	if (theApp.m_Bus.m_bINT20) strRet += "INT20 ";
	if (theApp.m_Bus.m_bINT30) strRet += "INT30 ";
	if (theApp.m_Bus.m_bINT40) strRet += "INT40 ";
	if (theApp.m_Bus.m_bINT50) strRet += "INT50 ";
	if (theApp.m_Bus.m_bINT60) strRet += "INT60 ";
	if (theApp.m_Bus.m_bINT70) strRet += "INT70 ";
	if (strRet.IsEmpty()) strRet = "none";
	return strRet;
}


/////////////////////////////////////////////////////////////////////////////
// LogicAnalyzer dialog


//	Arrays of control IDs for the multiple panels of this dialog:

static UINT	nControls1[] =	{
								IDC_PG1GB1,
								IDC_PG1GB2,
								IDC_LOG_ALL_CYCLES,
								IDC_LOG_M1,
								IDC_LOG_IRQ,
								IDC_LOG_IACK,
								IDC_LOG_IEN,
								IDC_LOG_RESETS,
								IDC_LOG_IOR,
								IDC_PORT_QAUAL_RD,
								IDC_IO_READ_ADDRESS,
								IDC_LOG_IOW,
								IDC_PORT_QAUAL_WR,
								IDC_IO_WRITE_ADDRESS,
								IDC_TRACEJMP,
								IDC_PAMDUMP
							};
static UINT	nControls2[] =	{
								IDC_PG2_GB1,
								IDC_TRACE_LIST,
								IDC_NEW_TRACE_BTN,
								IDC_CLEAR_TRACE_BTN
							};
static UINT	nControls3[] =	{
								IDC_PG3_GB1,
								IDC_DUMP_LIST,
								IDC_NEW_DUMP_BTN,
								IDC_CLEAR_DUMP_BTN
							};
static UINT	nControls4[] =	{
								IDC_PG4_GB1,
								IDC_BREAK_LIST,
								IDC_NEW_BREAK_BTN,
								IDC_CLEAR_BREAK_BTN
							};


LogicAnalyzer::LogicAnalyzer(CWnd* pParent /*=NULL*/)
	: CDialog(LogicAnalyzer::IDD, pParent)
{
	//{{AFX_DATA_INIT(LogicAnalyzer)
	m_bLogResets = FALSE;
	m_bDumpPAMWorkArea = FALSE;
	m_bTraceBranches = FALSE;
	m_nIOReadPort = 0;
	m_nIOWritePort = 0;
	m_bLogAllCycles = FALSE;
	m_bLogIACK = FALSE;
	m_bLogIntEnable = FALSE;
	m_bLogIORead = FALSE;
	m_bLogIOWrite = FALSE;
	m_bLogIRQ = FALSE;
	m_bLogM1 = FALSE;
	m_bQualifyIORad = FALSE;
	m_bQualifyIOWrite = FALSE;
	//}}AFX_DATA_INIT
	m_nPage = 0;
}


void LogicAnalyzer::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(LogicAnalyzer)
	DDX_Control(pDX, IDC_PORT_QAUAL_WR, m_IOWriteQualCheck);
	DDX_Control(pDX, IDC_PORT_QAUAL_RD, m_IOReadQualCheck);
	DDX_Control(pDX, IDC_IO_WRITE_ADDRESS, m_IOWriteEdit);
	DDX_Control(pDX, IDC_IO_READ_ADDRESS, m_IOReadEdit);
	DDX_Control(pDX, IDC_TRACE_LIST, m_TraceList);
	DDX_Control(pDX, IDC_DUMP_LIST, m_DumpList);
	DDX_Control(pDX, IDC_BREAK_LIST, m_BreakList);
	DDX_Control(pDX, IDOK, m_FinishBtn);
	DDX_Control(pDX, IDC_NEXT, m_NextBtn);
	DDX_Control(pDX, IDC_BACK, m_BackBtn);
	DDX_Check(pDX, IDC_LOG_RESETS, m_bLogResets);
	DDX_Check(pDX, IDC_PAMDUMP, m_bDumpPAMWorkArea);
	DDX_Check(pDX, IDC_TRACEJMP, m_bTraceBranches);
	DDX_Text(pDX, IDC_IO_READ_ADDRESS, m_nIOReadPort);
	DDV_MinMaxUInt(pDX, m_nIOReadPort, 0, 377);
	DDX_Text(pDX, IDC_IO_WRITE_ADDRESS, m_nIOWritePort);
	DDV_MinMaxUInt(pDX, m_nIOWritePort, 0, 377);
	DDX_Check(pDX, IDC_LOG_ALL_CYCLES, m_bLogAllCycles);
	DDX_Check(pDX, IDC_LOG_IACK, m_bLogIACK);
	DDX_Check(pDX, IDC_LOG_IEN, m_bLogIntEnable);
	DDX_Check(pDX, IDC_LOG_IOR, m_bLogIORead);
	DDX_Check(pDX, IDC_LOG_IOW, m_bLogIOWrite);
	DDX_Check(pDX, IDC_LOG_IRQ, m_bLogIRQ);
	DDX_Check(pDX, IDC_LOG_M1, m_bLogM1);
	DDX_Check(pDX, IDC_PORT_QAUAL_RD, m_bQualifyIORad);
	DDX_Check(pDX, IDC_PORT_QAUAL_WR, m_bQualifyIOWrite);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(LogicAnalyzer, CDialog)
	//{{AFX_MSG_MAP(LogicAnalyzer)
	ON_BN_CLICKED(IDC_BACK, OnBack)
	ON_BN_CLICKED(IDC_NEXT, OnNext)
	ON_BN_CLICKED(IDC_CLEAR_BREAK_BTN, OnClearBreakBtn)
	ON_BN_CLICKED(IDC_CLEAR_DUMP_BTN, OnClearDumpBtn)
	ON_BN_CLICKED(IDC_CLEAR_TRACE_BTN, OnClearTraceBtn)
	ON_BN_CLICKED(IDC_NEW_BREAK_BTN, OnNewBreakBtn)
	ON_BN_CLICKED(IDC_NEW_DUMP_BTN, OnNewDumpBtn)
	ON_BN_CLICKED(IDC_NEW_TRACE_BTN, OnNewTraceBtn)
	ON_NOTIFY(LVN_ITEMCHANGED, IDC_BREAK_LIST, OnItemchangedBreakList)
	ON_NOTIFY(LVN_ITEMCHANGED, IDC_DUMP_LIST, OnItemchangedDumpList)
	ON_NOTIFY(LVN_ITEMCHANGED, IDC_TRACE_LIST, OnItemchangedTraceList)
	ON_BN_CLICKED(IDC_DELETE_BREAK_BTN, OnDeleteBreakBtn)
	ON_BN_CLICKED(IDC_DELETE_DUMP_BTN, OnDeleteDumpBtn)
	ON_BN_CLICKED(IDC_DELETE_TRACE_BTN, OnDeleteTraceBtn)
	ON_BN_CLICKED(IDC_DISABLE_ALL_BREAK_BTN, OnDisableAllBreakBtn)
	ON_BN_CLICKED(IDC_DISABLE_BREAK_BTN, OnDisableBreakBtn)
	ON_BN_CLICKED(IDC_ENABLE_ALL_BREAK_BTN, OnEnableAllBreakBtn)
	ON_BN_CLICKED(IDC_ENABLE_BREAK_BTN, OnEnableBreakBtn)
	ON_BN_CLICKED(IDC_PORT_QAUAL_RD, OnPortQaualRd)
	ON_BN_CLICKED(IDC_PORT_QAUAL_WR, OnPortQaualWr)
	ON_BN_CLICKED(IDC_LOG_IOR, OnLogIor)
	ON_BN_CLICKED(IDC_LOG_IOW, OnLogIow)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// LogicAnalyzer message handlers

BOOL LogicAnalyzer::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	m_PanelManager.Create(this,				//	window pointer
							&m_BackBtn,		//	"Back" button
							&m_NextBtn,		//	"Next" button
							&m_FinishBtn);	//	"Finish" button
	m_PanelManager.AddPanel(0,									//	delta x
							0,									//	delta y
							sizeof(nControls1) / sizeof(UINT),	//	number of controls
							nControls1,							//	control id array
							-1);								//	id of control to get focus
	m_PanelManager.AddPanel(-310,
							0,
							sizeof(nControls2) / sizeof(UINT), 
							nControls2,
							-1);
	m_PanelManager.AddPanel(0,
							-220,
							sizeof(nControls3) / sizeof(UINT), 
							nControls3,
							-1);
	m_PanelManager.AddPanel(-310,
							-220,
							sizeof(nControls4) / sizeof(UINT), 
							nControls4,
							-1);
	m_nPage = 0;
	m_PanelManager.RepositionPanels();
	m_PanelManager.PositionControl(&m_FinishBtn, 0, -30);
	m_PanelManager.ResizeWindow(310, 185);	//	width and height of final window.
	
//	Must paginate before initializing the controls because we might want to
//	hide/show some controls after picking the panel.

	m_PanelManager.Paginate(m_nPage);

//	Initialize the list controls.

	SetListControlHeadings();
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

////////////////////////////////////////////////////////////////////////////////


void	LogicAnalyzer :: SetListControlHeadings()
{
	CString	strColText[6];
	CString	strMaxDataString[6];
	int		nWLabel;
	int		nWData;
	int		nW;
	int		nI;

//	Set the column headings common to all list controls.

	strColText[0] = "Region No.";
	strMaxDataString[0] = "65536";
	strColText[1] = "Start Address";
	strMaxDataString[1] = "377.377";
	strColText[2] = "Length";
	strMaxDataString[2] = "377.377";

//	Set the column headings specific to trace lists:

	strColText[3] = "";

//	Initialize the trace list control.	
	
	for (nI = 0; nI < 6; ++nI)
	{
		if (strColText[nI].IsEmpty()) break;
		nWLabel = m_TraceList.GetStringWidth((LPCSTR) strColText[nI]);
		nWData = m_TraceList.GetStringWidth((LPCSTR) strMaxDataString[nI]);
		nW = max(nWLabel, nWData) + 20;
		m_TraceList.InsertColumn(nI,
								(LPCSTR) strColText[nI],
								LVCFMT_LEFT,
								nW,
								nI);
	}
  
//	Set the specific column headings of the dump list control:

	strColText[3] = "";

//	Initialize the dump list control.	
	
	for (nI = 0; nI < 6; ++nI)
	{
		if (strColText[nI].IsEmpty()) break;
		nWLabel = m_DumpList.GetStringWidth((LPCSTR) strColText[nI]);
		nWData = m_DumpList.GetStringWidth((LPCSTR) strMaxDataString[nI]);
		nW = max(nWLabel, nWData) + 20;
		m_DumpList.InsertColumn(nI,
								(LPCSTR) strColText[nI],
								LVCFMT_LEFT,
								nW,
								nI);
	}
  
  
//	Set the specific column headings of the breakpoint list control:

	strColText[3] = "Status";
	strMaxDataString[3] = "Inactive";
	strColText[4] = "";

//	Initialize the breakpoint list control.	
	
	for (nI = 0; nI < 6; ++nI)
	{
		if (strColText[nI].IsEmpty()) break;
		nWLabel = m_BreakList.GetStringWidth((LPCSTR) strColText[nI]);
		nWData = m_BreakList.GetStringWidth((LPCSTR) strMaxDataString[nI]);
		nW = max(nWLabel, nWData) + 20;
		m_BreakList.InsertColumn(nI,
								(LPCSTR) strColText[nI],
								LVCFMT_LEFT,
								nW,
								nI);
	}
}


void	LogicAnalyzer :: LoadTraceRegionsControl()
{
	LV_ITEM			lviItem[3];
	int				nI;
	CString			strText1;
	CString			strText2;
	CString			strText3;
	TraceRegion *	pT;

	m_TraceList.DeleteAllItems();
	m_TraceList.SetItemCount(rgnTrace.GetCount());
	for (nI = 0; nI < rgnTrace.GetCount(); ++nI)
	{
		pT = rgnTrace.GetAt(rgnTrace.FindIndex(nI));
		strText1.Format("%d", nI);
		strText2 = SplitOctal(pT -> nStartAddress);
		strText3.Format("%d", pT -> nLength);
		lviItem[0].mask = LVIF_TEXT | LVIF_STATE;
		lviItem[0].iItem = nI;
		lviItem[0].iSubItem = 0;
		lviItem[0].state = 0;
		lviItem[0].stateMask = 0;
		lviItem[0].pszText = (LPSTR) (LPCSTR) strText1;
		lviItem[0].cchTextMax = 0;
		lviItem[0].iImage = -1;
		lviItem[1].mask = LVIF_TEXT;
		lviItem[1].iItem = nI;
		lviItem[1].iSubItem = 1;
		lviItem[1].state = 0;
		lviItem[1].stateMask = 0;
		lviItem[1].pszText = (LPSTR) (LPCSTR) strText2;
		lviItem[1].cchTextMax = 0;
		lviItem[1].iImage = 0;
		lviItem[2].mask = LVIF_TEXT;
		lviItem[2].iItem = nI;
		lviItem[2].iSubItem = 2;
		lviItem[2].state = 0;
		lviItem[2].stateMask = 0;
		lviItem[2].pszText = (LPSTR) (LPCSTR) strText3;
		lviItem[2].cchTextMax = 0;
		lviItem[2].iImage = 0;
		m_TraceList.InsertItem(&lviItem[0]);
		m_TraceList.SetItem(&lviItem[1]);
		m_TraceList.SetItem(&lviItem[2]);
	}
}



void	LogicAnalyzer :: LoadDumpRegionsControl()
{
	LV_ITEM			lviItem[3];
	int				nI;
	CString			strText1;
	CString			strText2;
	CString			strText3;
	DumpRegion *	pD;

	m_DumpList.DeleteAllItems();
	m_DumpList.SetItemCount(rgnDump.GetCount());
	for (nI = 0; nI < rgnDump.GetCount(); ++nI)
	{
		pD = rgnDump.GetAt(rgnDump.FindIndex(nI));
		strText1.Format("%d", nI);
		strText2 = SplitOctal(pD -> nStartAddress);
		strText3.Format("%d", pD -> nLength);
		lviItem[0].mask = LVIF_TEXT | LVIF_STATE;
		lviItem[0].iItem = nI;
		lviItem[0].iSubItem = 0;
		lviItem[0].state = 0;
		lviItem[0].stateMask = 0;
		lviItem[0].pszText = (LPSTR) (LPCSTR) strText1;
		lviItem[0].cchTextMax = 0;
		lviItem[0].iImage = -1;
		lviItem[1].mask = LVIF_TEXT;
		lviItem[1].iItem = nI;
		lviItem[1].iSubItem = 1;
		lviItem[1].state = 0;
		lviItem[1].stateMask = 0;
		lviItem[1].pszText = (LPSTR) (LPCSTR) strText2;
		lviItem[1].cchTextMax = 0;
		lviItem[1].iImage = 0;
		lviItem[2].mask = LVIF_TEXT;
		lviItem[2].iItem = nI;
		lviItem[2].iSubItem = 2;
		lviItem[2].state = 0;
		lviItem[2].stateMask = 0;
		lviItem[2].pszText = (LPSTR) (LPCSTR) strText3;
		lviItem[2].cchTextMax = 0;
		lviItem[2].iImage = 0;
		m_DumpList.InsertItem(&lviItem[0]);
		m_DumpList.SetItem(&lviItem[1]);
		m_DumpList.SetItem(&lviItem[2]);
	}
}



void	LogicAnalyzer :: LoadBreakRegionsControl()
{
	LV_ITEM			lviItem[3];
	int				nI;
	CString			strText1;
	CString			strText2;
	CString			strText3;
	CString			strText4;
	BreakRegion *	pB;

	m_BreakList.DeleteAllItems();
	m_BreakList.SetItemCount(rgnBreak.GetCount());
	for (nI = 0; nI < rgnBreak.GetCount(); ++nI)
	{
		pB = rgnBreak.GetAt(rgnBreak.FindIndex(nI));
		strText1.Format("%d", nI);
		strText2 = SplitOctal(pB -> nStartAddress);
		strText3.Format("%d", pB -> nLength);
		strText4 = (pB -> nType) ? "Active" : "Inactive";
		lviItem[0].mask = LVIF_TEXT | LVIF_STATE;
		lviItem[0].iItem = nI;
		lviItem[0].iSubItem = 0;
		lviItem[0].state = 0;
		lviItem[0].stateMask = 0;
		lviItem[0].pszText = (LPSTR) (LPCSTR) strText1;
		lviItem[0].cchTextMax = 0;
		lviItem[0].iImage = -1;
		lviItem[1].mask = LVIF_TEXT;
		lviItem[1].iItem = nI;
		lviItem[1].iSubItem = 1;
		lviItem[1].state = 0;
		lviItem[1].stateMask = 0;
		lviItem[1].pszText = (LPSTR) (LPCSTR) strText2;
		lviItem[1].cchTextMax = 0;
		lviItem[1].iImage = 0;
		lviItem[2].mask = LVIF_TEXT;
		lviItem[2].iItem = nI;
		lviItem[2].iSubItem = 2;
		lviItem[2].state = 0;
		lviItem[2].stateMask = 0;
		lviItem[2].pszText = (LPSTR) (LPCSTR) strText3;
		lviItem[2].cchTextMax = 0;
		lviItem[2].iImage = 0;
		lviItem[3].mask = LVIF_TEXT;
		lviItem[3].iItem = nI;
		lviItem[3].iSubItem = 2;
		lviItem[3].state = 0;
		lviItem[3].stateMask = 0;
		lviItem[3].pszText = (LPSTR) (LPCSTR) strText4;
		lviItem[3].cchTextMax = 0;
		lviItem[3].iImage = 0;
		m_BreakList.InsertItem(&lviItem[0]);
		m_BreakList.SetItem(&lviItem[1]);
		m_BreakList.SetItem(&lviItem[2]);
		m_BreakList.SetItem(&lviItem[3]);
	}
}


BOOL	LogicAnalyzer :: ValidatePage(int nPage)
{
	switch(nPage)
	{
	case 0:

//	validate paths for logfile, input settings and output settings.

		break;
	default:

//	nothing in need of validation.

		break;
	}
	return TRUE;
}

////////////////////////////////////////////////////////////////////////////////



void LogicAnalyzer::OnBack() 
{
	m_nPage--;
	if (m_nPage < 0) m_nPage = 0;
	m_PanelManager.Paginate(m_nPage);
	
}

void LogicAnalyzer::OnNext() 
{
	if (ValidatePage(m_nPage))
	{
		m_nPage++;
		if (m_nPage > 3) m_nPage = 3;
		m_PanelManager.Paginate(m_nPage);
	}	
}

void LogicAnalyzer::OnOK() 
{
	if (ValidatePage(m_nPage))
	{
		CDialog::OnOK();
	}
}


void LogicAnalyzer::OnClearBreakBtn() 
{
	// TODO: Add your control notification handler code here
	
}

void LogicAnalyzer::OnClearDumpBtn() 
{
	// TODO: Add your control notification handler code here
	
}

void LogicAnalyzer::OnClearTraceBtn() 
{
	// TODO: Add your control notification handler code here
	
}

void LogicAnalyzer::OnNewBreakBtn() 
{
	RegionDlg	myDlg;

	myDlg.m_bShowCheckbox = TRUE;
	if (myDlg.DoModal() == IDOK)
	{
	}
}

void LogicAnalyzer::OnNewDumpBtn() 
{
	RegionDlg	myDlg;

	myDlg.m_bShowCheckbox = FALSE;
	if (myDlg.DoModal() == IDOK)
	{
	}
}

void LogicAnalyzer::OnNewTraceBtn() 
{
	RegionDlg	myDlg;
	CString		strStart;
	CString		strEnd;

	myDlg.m_bShowCheckbox = FALSE;
	if (myDlg.DoModal() == IDOK)
	{

		strStart = DecimalToSplitOctal(myDlg.m_nStartAddress);
		strEnd = DecimalToSplitOctal(myDlg.m_nEndAddress);

		AfxMessageBox("Start = " + strStart + " End = " + strEnd);

//	Convert dialog data to items and insert them.

//		m_TraceList.InsertItem()
	}
}


CString	LogicAnalyzer :: DecimalToSplitOctal(int nValue)
{
	char	c[8];
	int		nTemp = nValue;

	c[3] = '.';
	c[7] = 0;
	c[6] = (char)('0' + nTemp % 8);
	nTemp = (nTemp >> 3) & 0x1FFF;
	c[5] = (char)('0' + nTemp % 8);
	nTemp = (nTemp >> 3) & 0x03FF;
	c[4] = (char)('0' + nTemp % 4);
	nTemp = (nTemp >> 2) & 0x00FF;
	c[2] = (char)('0' + nTemp % 8);
	nTemp = (nTemp >> 3) & 0x00FF;
	c[1] = (char)('0' + nTemp % 8);
	nTemp = (nTemp >> 3) & 0x00FF;
	c[0] = (char)('0' + nTemp % 4);
	return c;
}


void LogicAnalyzer::OnItemchangedBreakList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	// TODO: Add your control notification handler code here
	
	*pResult = 0;
}

void LogicAnalyzer::OnItemchangedDumpList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	// TODO: Add your control notification handler code here
	
	*pResult = 0;
}

void LogicAnalyzer::OnItemchangedTraceList(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;

//	If the state of some item goes from unselected to selected or vice-versa,
//	add or remove the item from our list of selections.

	if ((pNMListView -> uChanged) & LVIF_STATE)
	{
		if (((pNMListView -> uNewState) &  LVIS_SELECTED) &&
			!((pNMListView -> uOldState) &  LVIS_SELECTED))
		{

//	An item is being selected.  Add it to the list.

			SelectTraceItem(pNMListView -> iItem);
		}
		else if (!((pNMListView -> uNewState) &  LVIS_SELECTED) &&
					((pNMListView -> uOldState) &  LVIS_SELECTED))
		{

//	An item is being deselected.  Remove it from the list.

			DeSelectTraceItem(pNMListView -> iItem);
		}
	}
	*pResult = 0;
}


void	LogicAnalyzer::SelectTraceItem(int /* nItem */)
{
}


void	LogicAnalyzer::DeSelectTraceItem(int /* nItem */)
{
}


void	LogicAnalyzer::SelectDumpItem(int /* nItem */)
{
}


void	LogicAnalyzer::DeSelectDumpItem(int /* nItem */)
{
}


void	LogicAnalyzer::SelectBreakItem(int /* nItem */)
{
}


void	LogicAnalyzer::DeSelectBreakItem(int /* nItem */)
{
}

void LogicAnalyzer::OnDeleteBreakBtn() 
{
	// TODO: Add your control notification handler code here
	
}

void LogicAnalyzer::OnDeleteDumpBtn() 
{
	// TODO: Add your control notification handler code here
	
}

void LogicAnalyzer::OnDeleteTraceBtn() 
{
	// TODO: Add your control notification handler code here
	
}

void LogicAnalyzer::OnDisableAllBreakBtn() 
{
	// TODO: Add your control notification handler code here
	
}

void LogicAnalyzer::OnDisableBreakBtn() 
{
	// TODO: Add your control notification handler code here
	
}

void LogicAnalyzer::OnEnableAllBreakBtn() 
{
	// TODO: Add your control notification handler code here
	
}

void LogicAnalyzer::OnEnableBreakBtn() 
{
	// TODO: Add your control notification handler code here
	
}

void LogicAnalyzer::OnPortQaualRd() 
{
	// TODO: Add your control notification handler code here
	
}

void LogicAnalyzer::OnPortQaualWr() 
{
	// TODO: Add your control notification handler code here
	
}

void LogicAnalyzer::OnLogIor() 
{
	// TODO: Add your control notification handler code here
	
}

void LogicAnalyzer::OnLogIow() 
{
	// TODO: Add your control notification handler code here
	
}
