/************************************************************************
 *																		*
 *	Copyright (c) 1998 - 2001 by	David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/


#include "stdafx.h"

#define	EXTERN	extern

#include "h8types.h"
#include "Virtual_h8.h"
#include "globals.h"
#include "memmap.h"
#include "logicanalyzer.h"

#ifndef _DEBUG
#define	LOC(x)	(x)
#else
#define	LOC(x)	(x)][0
#endif


//	Memory map of ROM origins and sizes:

static const int	nRoms = 2;
static const PWORD	nRomOrgs[nRoms] = {
										PAM37_ORG,
										H17_ORG
									  };
static const PWORD	nRomSizes[nRoms] = {
										PAM37_SIZE,
										H17_SIZE
									   };



	MemoryMap :: MemoryMap()
{
	m_bRomDisable = false;
	memset(m_szMemory, 0, sizeof(m_szMemory));
	memset(m_szROMs, 0, sizeof(m_szROMs));
}

	MemoryMap :: ~MemoryMap()
{
}

void MemoryMap :: SetRomDisable(bool bSetting)
{
	m_bRomDisable = bSetting;
}
	
PBYTE	MemoryMap :: Read(PWORD nAddress)
{

//	The hardware reads from ROM if the ROM is not disabled.

	if (IsRom(nAddress))
	{
		return m_szROMs[LOC(nAddress)];
	}
	else
	{
		return m_szMemory[LOC(nAddress)];
	}
}


PWORD	MemoryMap :: Read2(PWORD nAddress)
{
	union	{
		PBYTE	c[2];
		PWORD	w;
	} work;
	
//	The hardware reads from ROM if the ROM is not disabled.

	if (IsRom(nAddress))
	{
		work.c[0] = m_szROMs[LOC(nAddress++)];
		work.c[1] = m_szROMs[LOC(nAddress)];
	}
	else
	{
		work.c[0] = m_szMemory[LOC(nAddress++)];
		work.c[1] = m_szMemory[LOC(nAddress)];
	}
	return work.w;
}


void	MemoryMap :: Write(PWORD nAddress, PBYTE nData)
{

//	The hardware does not trap an attempt to write to ROM; instead (if there
//	is RAM overlapping the ROM, as became the case in latter-day systems),
//	the write goes to the RAM at the same location.


#ifdef _DEBUG

	if (tfp != NULL)
	{
		if (GetDebugFlags(nAddress) & DEBUG_DUMP_FLAG)
		{
			fprintf(tfp, ">>\tWrote %s at %s\n",
					(LPCSTR) Octal(nData),
					(LPCSTR) SplitOctal(nAddress));
			fflush(tfp);
		}
	}

#endif

	m_szMemory[LOC(nAddress)] = nData;
}


void	MemoryMap :: Write2(PWORD nAddress, PWORD nData)
{
	union	{
		PBYTE	c[2];
		PWORD	w;
	} work;

	work.w = nData;

#ifdef _DEBUG

	if (tfp != NULL)
	{
		if (GetDebugFlags(nAddress) & DEBUG_DUMP_FLAG)
		{
			fprintf(tfp, ">>\tWrote %s at %s\n",
					(LPCSTR) SplitOctal(nData),
					(LPCSTR) SplitOctal(nAddress));
			fflush(tfp);
		}
	}

#endif

	m_szMemory[LOC(nAddress++)] = work.c[0];
	m_szMemory[LOC(nAddress)] = work.c[1];
}


void	MemoryMap :: LoadProm(PWORD nAddress, PBYTE nData)
{

//	This routine loads the ROMs (called only from the PromLoad function):

	m_szROMs[LOC(nAddress)] = nData;
}


int		MemoryMap :: GetDebugFlags(PWORD nAddress)
{

#ifndef _DEBUG

	return 0;

#else

	if (IsRom(nAddress))
	{
		return m_szROMs[nAddress][1];
	}
	else
	{
		return m_szMemory[nAddress][1];
	}

#endif

}


void	MemoryMap :: SetDebugFlags(PWORD nAddress, PBYTE nData)
{

#ifdef _DEBUG

	m_szMemory[nAddress & 0x00FFFF][1] = nData;

#endif

}


bool	MemoryMap :: IsRom(PWORD nAddress)
{

	if (m_bRomDisable) return false;

#if 0

//	This is the right way to code this, but it takes too long:

	for (int nI = 0; nI < nRoms; ++nI)
	{
		if (((nAddress) >= nRomOrgs[nI]) &&
			((nAddress) < (nRomOrgs[nI] + nRomSizes[nI]))) return true;
	}
	return false;

#else

//	This is the shortcut -- it's based on knowing where the roms are
//	in relationship to each other and knowing that there are only two:

	if (nAddress > H17_ORG + H17_SIZE) return false;
	else if ((nAddress < H17_ORG) &&
				(nAddress > PAM37_ORG + PAM37_SIZE)) return false;
	else return true;

#endif

}