/************************************************************************
 *																		*
 *	H8 EMULATOR Project --	PAM.H										*
 *																		*
 *	A simulation of the Heathkit H-8 control panel logic				*
 *																		*
 *	------------------------------------------------------------------	*
 *																		*
 *	Copyright (c) 1998 - 2001 by	David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/


#ifndef	_PAM_H_
#define	_PAM_H_

//	============================================================================
//
//	The memory map for the H-17 and PAM-37 ROMs:

#define	PAM37_ORG				0x0000	//	000.000
#define	PAM37_SIZE				0x1000	//	020.000
#define	H17_ORG					0x1800	//	030.000
#define	H17_SIZE				0x0800	//	010.000


//	============================================================================
//
//	Memory map for control panel's RAM work area:

#define	PAM_WORKSPACE			0x2000
#define PAM_WORKSPACE_SIZE		0x0040


//	============================================================================
//
//	The 8251 USART:

#define	USART_CMD_HUNT			0x0080
#define	USART_CMD_RESET			0x0040
#define	USART_CMD_RTS			0x0020
#define	USART_CMD_ERROR_RESET	0x0010
#define	USART_CMD_BREAK			0x0008
#define	USART_CMD_RX_ENABLE		0x0004
#define	USART_CMD_DTR			0x0002
#define	USART_CMD_TX_ENABLE		0x0001

#define	USART_STATUS_DSR		0x0080
#define	USART_STATUS_SYNDET		0x0040
#define	USART_STATUS_FE			0x0020
#define	USART_STATUS_OE			0x0010
#define	USART_STATUS_PE			0x0008
#define	USART_STATUS_TXE		0x0004
#define	USART_STATUS_RXRDY		0x0002
#define	USART_STATUS_TXRDY		0x0001

struct USART {					//	8251 emulation
	BOOL			bCommand_State;
	unsigned char	nMode;
	unsigned char	nCommand;
	unsigned char	nStatus;
};


//	============================================================================
//
//	I/O Ports of the Control Panel Card:

#define	PANEL_SEGMENT_SELECT	0361	//	write
#define	PANEL_KEYBOARD			0361	//	read
#define	PANEL_DIGIT_SELECT		0360	//	write -- low-order nibble
#define	PANEL_CONTROL			0360	//	write -- high-order nibble

//	The PANEL_CONTROL port is divided into two pieces:

#define	PANEL_DIGIT_SELECT_MASK	0x000F
#define	PANEL_CONTROL_MASK		0x00F0


//	============================================================================
//
//	MAN-1 segment (IOW361) bit defs:
//
//				A
//		   ------------
//		F  /		  /
//		  /          /  B
//		 /		G	/
//		------------
//	    /		   /
//	 E /		  /
//	  /			 /  C
//	 /		D	/
//	------------	* P
//
//
//	Note: the segment select is actually active-low (i.e.: zero -> segment ON).

#define	PANEL_SEGMENT_A	0x0002
#define	PANEL_SEGMENT_B	0x0004
#define	PANEL_SEGMENT_C	0x0008
#define	PANEL_SEGMENT_D	0x0010
#define	PANEL_SEGMENT_E	0x0020
#define	PANEL_SEGMENT_F	0x0040
#define	PANEL_SEGMENT_G	0x0001
#define	PANEL_SEGMENT_P	0x0080

//	Some useful combinations of the above segments as characters:

#define	PANEL_MAN1_0		~0x007E
#define	PANEL_MAN1_1		~0x000C
#define	PANEL_MAN1_2		~0x0037
#define	PANEL_MAN1_3		~0x001F
#define	PANEL_MAN1_4		~0x004D
#define	PANEL_MAN1_5		~0x005B
#define	PANEL_MAN1_6		~0x007B
#define	PANEL_MAN1_7		~0x000E
#define	PANEL_MAN1_8		~0x007F
#define	PANEL_MAN1_9		~0x004F
#define	PANEL_MAN1_A		~0x006F
#define	PANEL_MAN1_B		~0x0079
#define	PANEL_MAN1_C		~0x0072
#define	PANEL_MAN1_D		~0x003D
#define	PANEL_MAN1_E		~0x0073
#define	PANEL_MAN1_F		~0x0063
#define	PANEL_MAN1_H		~0x006D
#define PANEL_MAN1_L		~0x0070
#define	PANEL_MAN1_N		~0x0029
#define	PANEL_MAN1_P		~0x0067
#define	PANEL_MAN1_R		~0x0021
#define	PANEL_MAN1_U		~0x007C
#define PANEL_MAN1_X		~0x0049
#define PANEL_MAN1_Y		~0x005D
#define PANEL_MAN1_TICK		~0x0040
#define	PANEL_MAN1_DASH		~0x0001
#define	PANEL_MAN1_SPACE	~0x0000
#define PANEL_MAN1_UNDERBAR	~0x0010


//	============================================================================
//
//	Bit definitions for IOW360

#define	PANEL_DIGIT_1	0x0001
#define	PANEL_DIGIT_2	0x0002
#define	PANEL_DIGIT_3	0x0003
#define	PANEL_DIGIT_4	0x0004
#define	PANEL_DIGIT_5	0x0005
#define	PANEL_DIGIT_6	0x0006
#define	PANEL_DIGIT_7	0x0007
#define	PANEL_DIGIT_8	0x0008
#define	PANEL_DIGIT_9	0x0009
#define	PANEL_INT_20	0x0010
#define	PANEL_MON		0x0020
#define	PANEL_INT_10	0x0040
#define	PANEL_SPKR		0x0080


//	============================================================================
//
//	Bit definitions for IOR360

#define	PANEL_KEY_0		0x00FE
#define	PANEL_KEY_1		0x00FC
#define	PANEL_KEY_2		0x00FA
#define	PANEL_KEY_3		0x00F8
#define	PANEL_KEY_4		0x00F6
#define	PANEL_KEY_5		0x00F4
#define	PANEL_KEY_6		0x00F2
#define	PANEL_KEY_7		0x00F0
#define	PANEL_KEY_8		0x00EF
#define	PANEL_KEY_9		0x00CF
//	the traditional (PAM-8) names for the non-numeric keys:
#define	PANEL_KEY_PLUS	0x00AF
#define	PANEL_KEY_DASH	0x008F
#define	PANEL_KEY_STAR	0x006F
#define	PANEL_KEY_SLASH	0x004F
#define	PANEL_KEY_MESH	0x002F
#define	PANEL_KEY_DOT	0x000F
//	the non-traditional (PAM-37) names for the non-numeric keys:
#define	PANEL_KEY_A		0x00AF
#define	PANEL_KEY_B		0x008F
#define	PANEL_KEY_C		0x006F
#define	PANEL_KEY_D		0x004F
#define	PANEL_KEY_E		0x002F
#define	PANEL_KEY_F		0x000F


//	============================================================================
//
//	Note:	INT10 signal to the processor is pulsed by RTM key and also by timer
//			(see below) while INT10 enable is high.  Any write to port 361
//			resets this flop.
//			INT20 signal is set after two instruction fetches while EI is TRUE
//			and INT20 request is set and resets two fetches after EI goes false.
//			The control panel also includes a 2MS timer.  The timer does a
//			hardware clear of the segment select registers (limits duty cycle of
//			LEDs) and also of the MON led.  The other flip-flops are only cleared
//			by RESET.
//
//	(Hopefully, we will not need to simulate the hardware this closely!)
//
//	There is a 2-ms timer and a 1000 HZ squarewave circuit on the panel card.
//	We sample the display at a rate that's much slower than the 2ms tick, but
//	still fast enough so the perception is that the display is tracking the I/O
//	to the PAM display segment and digit select hardware closely.

#define	PANEL_TIMER				0x0001
#define	PANEL_TIMER_PERIOD		   100	//	Sampling rate for LED display
#define	PANEL_BEEP_FREQUENCY	  1000


//	============================================================================
//
//	This data structure represents the I/O resources of the front panel:

class Pam
{
//	Methods:

public:
					Pam();
					~Pam();
	void			Reset();

protected:

private:

//	Attributes:

public:
	USART			TapePort;
	unsigned char	ioPAM_IOW_360;
	unsigned char	ioPAM_IOR_360;
	unsigned char	ioPAM_IOW_361;
	BOOL			bI10;			//	is ioPAM_IOW_360 bit 6 = FALSE
	BOOL			bI20;			//	is ioPAM_IOW_360 bit 4 = TRUE
	BOOL			bSpkr;			//	is ioPAM_IOW_360 bit 7 = TRUE
	BOOL			bMonitorLED;	//	is ioPAM_IOW_360 bit 5 = TRUE
	int				nMAN1[9];
	BOOL			m_bReedSwitch;	//	A "hardware mod": when TRUE, inhibit 1 KHz tone
	BOOL			bUpdate;
	unsigned char	nKeycode;

protected:

private:

};

//	============================================================================
//
//	XCON Hardware -- used by PAM-37 but actually on Z-80 CPU Card.
//

#define	XCON_CTRL				0362
#define	XCON_CONFIG				0362


//	XCON Control Port Bits:

#define	XCON_CTRL_SIDE1SEL		0x0040
#define	XCON_CTRL_ROMDISABLE	0x0020
#define	XCON_CTRL_NMIMASK		0x0008


//	XCON Configuration Port Bits:

//	present setting for config port is:
#define	XCON_CFG_DATA			0x0008
//	but the following bits could be set:
#define	XCON_CFG_AUTOBOOT		0x0080
#define	XCON_CFG_BOOT_PRI_170	0x0010
#define	XCON_CFG_PORT170_IS_H67	0x0008
#define	XCON_CFG_PORT170_IS_H47	0x0004
#define	XCON_CFG_PORT170_IS_H37	0x0000
#define	XCON_CFG_PORT174_IS_H67	0x0002
#define	XCON_CFG_PORT174_IS_H47	0x0001
#define	XCON_CFG_PORT174_IS_H17	0x0000


//	============================================================================
//
//	This data structure represents the I/O resources of the XCON circuit:

struct XCON_IO {
	unsigned char	ioXCON_IOW_362;
	unsigned char	ioXCON_IOR_362;
	BOOL			bSideSelect;
	BOOL			bRomDisable;
	BOOL			bNmiMask;
};

#endif	//	#ifndef	_PAM_H_