/************************************************************************
 *																		*
 *	Copyright (c) 1998 - 2001 by	David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/


#include "stdafx.h"
#include <process.h>

#define	EXTERN		//	this is the only place where EXTERN isn't extern!

#include "virtual_h8.h"

#include "MainFrm.h"
#include "virtual_h8Doc.h"
#include "virtual_h8View.h"
#include "processor.h"
#include "pam.h"
#include "bus.h"
#include "globals.h"
#include "promload.h"
#include "cpuproc.h"
#include "ioproc.h"
#include "logicanalyzer.h"
#include "msgs.h"
#include "lasterrortext.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



/////////////////////////////////////////////////////////////////////////////
// Global objects



	HANDLE			g_hCPUThread;
	HANDLE			g_hTimerThread;
static UINT __stdcall	RxMsgThreadFunc(void * pParams);



/////////////////////////////////////////////////////////////////////////////
// CVirtual_h8App

BEGIN_MESSAGE_MAP(CVirtual_h8App, CWinApp)
	//{{AFX_MSG_MAP(CVirtual_h8App)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CVirtual_h8App construction

CVirtual_h8App::CVirtual_h8App()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
	m_nTimeSincePower = 0;
	m_nTau = 2;
	for (int nI = 0; nI < MAX_CONNECTIONS; ++nI)
		m_bConnected[nI] = FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// CVirtual_h8App initialization

BOOL CVirtual_h8App::InitInstance()
{
	unsigned int	wThreadId;
	char			szH8Directory[MAX_PATH];

	AfxEnableControlContainer();

// Standard initialization

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

//	For now, we let the application initialize the global logic analyzer objects
//	and the logic analyzer dialog, if run, modify them.

	tfp = NULL;
	g_bLABranchTrace = FALSE;
	g_bLAIOReadTrace = FALSE;
	g_bLAIOWriteTrace = FALSE;
	g_bLAIRQCycleTrace = FALSE;
	g_bLAIENCycleTrace = FALSE;
	g_bLAIACKCycleTrace = FALSE;
	g_bLAResetTrace = FALSE;
	g_bLADumpPAMArea = TRUE;

//	Initialize all the other global storage here, too, while we're at it.

	m_bTimerTickEnable = FALSE;
	m_fpTapeOut = NULL;
	m_fpTapeIn = NULL;
	m_strTapeInPath = "";
	m_strTapeOutPath = "";
	g_hCPUThread = NULL;
	g_hTimerThread = NULL;
	m_hSound = LoadResource(m_hInstance, FindResource(m_hInstance, MAKEINTRESOURCE(IDR_1KHZ),"WAVE"));
	if (!m_hSound)
	{
		AfxMessageBox(GetLastErrorText());
	}
	else
	{
		if (!LockResource(m_hSound))
		{
			AfxMessageBox(GetLastErrorText());
			m_hSound = NULL;
		}
	}

// Register the application's document templates.  Document templates
//  serve as the connection between documents, frame windows and views.

	CSingleDocTemplate* pDocTemplate;
	pDocTemplate =
		new CSingleDocTemplate(IDR_MAINFRAME,
								RUNTIME_CLASS(CVirtual_h8Doc),
								RUNTIME_CLASS(CMainFrame),  // main SDI frame window
								RUNTIME_CLASS(CVirtual_h8View));
	AddDocTemplate(pDocTemplate);

// Ignore command line by constructing default cmdInfo object but never setting it.

	CCommandLineInfo cmdInfo;
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;

//	Get H8 Environment directory from command line.

	m_strH8Directory = "";
	if (__argc >= 2)
	{
		strcpy(szH8Directory, __argv[1]);
		if (ExistsDirectory(szH8Directory))
		{
			m_strH8Directory = szH8Directory;
		}
	}
	if (m_strH8Directory.IsEmpty())
	{
		m_strH8Directory = "C:\\H8";
	}

//	Load the ROM contents from disk files.

	if (PromLoader(m_strH8Directory + "\\FIRMWARE\\PAM37.ROM") != 0)
	{
		AfxMessageBox("Could not load the PAM-37 ROM Image.");
		return FALSE;
	}
	if (PromLoader(m_strH8Directory + "\\FIRMWARE\\H17.ROM") != 0)
	{
		AfxMessageBox("Could not load the H-17 ROM Image.");
		return FALSE;
	}

//	Set up the initial state of the bus.
//	Activate the processor by deasserting WAIT and asserting RESET, but because
//	m_bSuspend is TRUE, neither the I/O nor CPU threads will do anything.

	m_Bus.m_bSuspend = TRUE;

//	Put the hardwar in power-on reset state.
	
	m_XCON_IO_PORTS.ioXCON_IOR_362 = XCON_CFG_DATA;
	m_XCON_IO_PORTS.ioXCON_IOW_362 = 0;
	m_PAM_IO_PORTS.Reset();

// The one and only window has been initialized, so show and update it.

	m_pMainWnd -> ShowWindow(SW_SHOW);
	m_pMainWnd -> UpdateWindow();

//	Create the IPC pipe pairs

	CreatePipes();

//	Spawn the task that emulates the processor and IO devices.

	if ((g_hCPUThread = ((HANDLE)
			_beginthreadex(NULL, 	            // security
							0, 					// stack size
							CPU_Proc,			// start addr
							NULL, 	            // param
							0, 					// run immediately
							&wThreadId) 		// thread id -- nobody cares
						)) == NULL )
	{
		AfxMessageBox("Create thread for CPU failed.");
		return FALSE;
	}

//	Spawn the task that starts the 2 ms timer

	if ((g_hTimerThread = ((HANDLE)
			_beginthreadex(NULL, 	            // security
							0, 					// stack size
							Timer_Proc,			// start addr
							NULL, 	            // param
							0, 					// run immediately
							&wThreadId) 		// thread id -- nobody cares
						)) == NULL )
	{
		AfxMessageBox("Create thread for Timer failed.");
		return FALSE;
	}
	theApp.m_Bus.m_bSuspend = FALSE;
	theApp.m_bTimerTickEnable = TRUE;
	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CVirtual_h8App::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// CVirtual_h8App message handlers



BOOL CVirtual_h8App::ExistsDirectory(CString strDirectory)
{
	CString	strPath = Dequote(strDirectory);
	DWORD	dAttribs = GetFileAttributes((LPCSTR) strPath);

	if (dAttribs == 0xFFFFFFFF)
	{
		return FALSE;
	}
	else return ((dAttribs & FILE_ATTRIBUTE_DIRECTORY) != 0);
}


CString	CVirtual_h8App::Dequote(CString strPath)
{
	CString	strTemp = strPath;
	
	if (strTemp.GetLength() >= 2)
	{
		strTemp = strTemp.Mid(1);
		strTemp = strTemp.Left(strTemp.GetLength() - 1);
	}
	if (strPath.IsEmpty()) return "";
	else if (strPath[0] == '\"')
	{
		return strTemp;
	}
	else
	{
		return strPath;
	}
}


int CVirtual_h8App::ExitInstance() 
{
	int			nConnection;
	POSITION	posP;
	Schedule	stSchedule;

	m_pRecentFileList = NULL;

#ifdef _DEBUG

//	Dump memory from 040.000 to 040.100 if the PAM Area Dump flag is set.

	if ((tfp != NULL) && (g_bLADumpPAMArea))
	{
		int				nAddr;
		int				nI;
		int				nData;

		fprintf (tfp,"\n\n===================================================\n"
						"\n\t\tPOST_MORTEM MEMORY DUMP\n\n"
						"\tPAM work area:\n");
		nAddr = 020000;
		while (nAddr < 020100)
		{
			fprintf(tfp, "\n%s\t", (LPCSTR) SplitOctal(nAddr));
			for (nI = 0; nI < 16; ++nI)
			{
				nData = m_Processor.m_MemMap.Read(nAddr++);
				fprintf(tfp, " %s", Octal(nData));
			}
		}
		fprintf(tfp, "\n\n");
		MemoryDump();
		fclose(tfp);
	}

#else

	if (tfp != NULL)
	{
		fclose(tfp);
	}

#endif


//	Signal any pending schedules.

	posP = m_clSchedules.GetHeadPosition();
	while(posP != NULL)
	{
		stSchedule = m_clSchedules.GetNext(posP);
		SetEvent(*stSchedule.pEvent);
	}

//	Signal a shutdown in process and send all satellite applications a
//	SHUTDOWN_NOW_CMD message if their pipes are connected.

	for (nConnection = 0;
			nConnection < (IPC_ARRAY_SIZE / 2);
			++nConnection)
	{
		if (!((m_IPC.stPipes[nConnection].hPipe == INVALID_HANDLE_VALUE) ||
			(m_IPC.stPipes[nConnection].ePipeState == PIPESTATE_CLOSED) ||
			(m_IPC.stPipes[nConnection].ePipeState == PIPESTATE_BROKEN)))
		{
			EmitMessage(nConnection, SHUTDOWN_NOW_CMD);
		}
	}
	theApp.m_Bus.m_bReset = TRUE;
	SetEvent(m_IPC.hShutdown);
	Sleep(1000);	//	wait one second so messages are received

//	Close all pipe connections.

	DestroyPipes();
	if (g_hCPUThread != NULL)
	{
		if (WaitForSingleObject(g_hCPUThread, 1000) == WAIT_TIMEOUT)
		{
			TerminateThread(g_hCPUThread, 3);
		}
		g_hCPUThread = NULL;
	}
	if (g_hTimerThread != NULL)
	{
		if (WaitForSingleObject(g_hTimerThread, 1000) == WAIT_TIMEOUT)
		{
			TerminateThread(g_hTimerThread, 3);
		}
		g_hTimerThread = NULL;
	}
	if (m_IPC.hPipeProc != NULL)
	{
		if (WaitForSingleObject(m_IPC.hPipeProc, 1000) == WAIT_TIMEOUT)
		{
			TerminateThread(m_IPC.hPipeProc, 3);
		}
		m_IPC.hPipeProc = NULL;
	}
	return CWinApp::ExitInstance();
}


BOOL	CVirtual_h8App::ScheduleWakeup(HANDLE * pWakeup, unsigned long int nDelay)
{
	Schedule	stSchedule;
	POSITION	posP;
	BOOL		bResult = TRUE;

//	Add a wakeup call at nDelay after current time.  Do not allow multiple
//	wakeups on the same handle simultaneously.  Ensure that the event is
//	unsignalled when the routine exits.  The scheduler will refuse any delays
//	of less than one millisecond or more than one day.

	EnterCriticalSection(&m_Bus.m_critTime);	//	use of m_critTime interlocks
												//	  with the 1ms timer thread.
	if ((*pWakeup == INVALID_HANDLE_VALUE) ||
		(nDelay < 1) || (nDelay > 86400000))
	{
		bResult = FALSE;	//	invalid arguments to scheduler
	}
	if (bResult)
	{
		stSchedule.pEvent = pWakeup;
		stSchedule.nExpirationTime = m_nTimeSincePower + nDelay;
		posP = m_clSchedules.GetHeadPosition();
		while(posP != NULL)
		{
			if (*stSchedule.pEvent == *m_clSchedules.GetNext(posP).pEvent)
			{
				bResult = FALSE;	//	duplicates existing schedule
				break;
			}
		}
	}
	if (bResult)
	{
		ResetEvent(*(stSchedule.pEvent));
		m_clSchedules.AddTail(stSchedule);
	}
	LeaveCriticalSection(&m_Bus.m_critTime);
	return bResult;
}


//	============================================================================
//
//	IPC Code -- part of the application for convenience; except for CreatePipes
//	and DestroyPipes, is actually used in CH8CtrlDlg object.
//
//	============================================================================

//	Create pipes function
//
//	Creates read pipe thread and RX and TX pipes for the IPC connections.
//

BOOL CVirtual_h8App::CreatePipes()
{
	UINT	dwThreadId;
	CString	strErrorMessage = "";

	if (m_IPC.CreatePipes(strErrorMessage))
	{

//	Create the read pipes thread.

		if((m_IPC.hPipeProc = (HANDLE)
			:: _beginthreadex(
					NULL,				// No security Attribs
	    			0L,					// Default stack
					&RxMsgThreadFunc,	// address of thread function
					NULL,				// argument for new thread 
					0,					// default creation flags (running, I hope!)
					&dwThreadId			// address of returned thread ID 
				)) == NULL)
		{
			strErrorMessage = 
				"Program error -- Unable to Create Pipe RX Thread.";
		}
	}
	if (!strErrorMessage.IsEmpty())
	{
		AfxMessageBox(strErrorMessage);
		DestroyPipes();
		return FALSE;
	}
	return TRUE;
}



//	Disconnect pipes function
//
//	Closes and re-opens the pipe pairs for one of the IPC pipe connections.
//	Parameter specifies which one:
//		0 - Audio Tape
//		1 - (reserved) H-19 Terminal
//		2 - (reserved) H-17 5-1/4 Hard-Sectored Diskette
//		3 - (reserved) H-14 Printer
//		4 - (reserved) H-37 5-1/4" Diskette
//		5 - (reserved) H-47/H-67 8" Diskette

BOOL CVirtual_h8App::DisconnectPipes(int nConnectionId)
{
	CString	strErrorMessage;

//	Disconnect and re-open the RX and TX pipes for the specified connection.

	if (m_IPC.DisconnectPipe(2 * nConnectionId, strErrorMessage))
	{
		if (m_IPC.DisconnectPipe(2 * nConnectionId + 1, strErrorMessage))
		{
			m_bConnected[nConnectionId] = FALSE;
			if (m_IPC.ConnectPipe(2 * nConnectionId, strErrorMessage))
			{
				m_IPC.ConnectPipe(2 * nConnectionId + 1, strErrorMessage);
			}
		}
	}
	if (!strErrorMessage.IsEmpty())
	{
		AfxMessageBox(strErrorMessage);
		DestroyPipes();
		return FALSE;
	}
	return TRUE;
}



//	Destroy pipes function
//
//	 Terminates the read pipe thread and closes all the IPC pipe pairs.

void CVirtual_h8App::DestroyPipes()
{
	int		nPipe;
	CString	strErrorMessage;

//	Disconnect all pipes without reconnecting them.

	for (nPipe = 0; nPipe < (IPC_ARRAY_SIZE - 1); ++nPipe)
	{
		if (!m_IPC.DisconnectPipe(nPipe, strErrorMessage))
		{
			AfxMessageBox(strErrorMessage);
		}
		m_bConnected[nPipe / 2] = FALSE;
	}
	
//	Signal kill to the reader thread

	SetEvent(m_IPC.hKillPipeProc);
	WaitForSingleObject(m_IPC.hPipeProc, 1000);
}


//	Write pipes function
//
//	Sends a message to the specified IPC RX pipe.  Destination codes, defined
//	like the enumerator of Disconnect pipes are:
//		0 - Audio Tape
//		1 - (reserved) H-19 Terminal
//		2 - (reserved) H-17 5-1/4 Hard-Sectored Diskette
//		3 - (reserved) H-14 Printer
//		4 - (reserved) H-37 5-1/4" Diskette
//		5 - (reserved) H-47/H-67 8" Diskette
//
//	Note: "Write pipes" does a synchronous write -- time stops while the
//	operation is in progress.

BOOL CVirtual_h8App::EmitMessage(int nConnectionId,
							int nCommandCode,
							CStringList & cslArgs)
{
	CString			strMessage = make_message(nCommandCode, cslArgs);
	DWORD			dwErrno;
	CString			strError;
	int				nPipeNum = nConnectionId * 2;	//	even number is write pipe

//	If the pipe's not connected, this is a NOP.  If the pipe is busy with a
//	previous write, we wait.  Otherwise, just write the message.

	if ((m_IPC.stPipes[nPipeNum].hPipe == INVALID_HANDLE_VALUE) ||
		(m_IPC.stPipes[nPipeNum].ePipeState == PIPESTATE_CLOSED) ||
		(m_IPC.stPipes[nPipeNum].ePipeState == PIPESTATE_BROKEN))
	{
		strError.Format("Attempting to send message on %s pipe # %d.",
							((m_IPC.stPipes[nPipeNum].hPipe ==
								INVALID_HANDLE_VALUE) ? "invalid" :
								(m_IPC.stPipes[nPipeNum].ePipeState == 
									PIPESTATE_CLOSED) ? "closed" : "broken"),
							nConnectionId);
		AfxMessageBox(strError);
		return TRUE;
	}
	else 
	{
		ResetEvent(m_Bus.m_hTimeRuns);	//	stop time for the H-8.

//	Ensure we own the outgoing message buffer before we proceed...

		WaitForSingleObject(m_IPC.stPipes[nPipeNum].overControl.hEvent,
							INFINITE);

//	Initialize the buffer to padding character

		memset((void *) &(m_IPC.stPipes[nPipeNum].szBuffer[0]), '\n',
				MAX_MESSAGE_LENGTH);
		strncpy(m_IPC.stPipes[nPipeNum].szBuffer, (LPCSTR) strMessage,
				strMessage.GetLength());
		m_IPC.stPipes[nPipeNum].dwBufferLength = strMessage.GetLength();
		m_IPC.stPipes[nPipeNum].dwXferCount = 0;
		if (WriteFile(m_IPC.stPipes[nPipeNum].hPipe,
					  m_IPC.stPipes[nPipeNum].szBuffer,
					  m_IPC.stPipes[nPipeNum].dwBufferLength,
					  &m_IPC.stPipes[nPipeNum].dwXferCount,
					  &m_IPC.stPipes[nPipeNum].overControl) == FALSE)
		{
			if ((dwErrno = GetLastError()) !=  ERROR_IO_PENDING)
			{
				strError.Format("Write to pipe failed.  Error was: %s",
							(LPCSTR) GetLastErrorText(dwErrno));
				AfxMessageBox(strError);
				return FALSE;
			}
		}
		SetEvent(m_Bus.m_hTimeRuns);	//	allow time to proceed.
	}
	return TRUE;
}

//	This overload makes EmitMessage more convenient to use when commands need
//	zero or one arguments

BOOL	CVirtual_h8App::EmitMessage(int nConnection, int nCommand, char * szArg)
{
	CStringList	cslArgs;

	cslArgs.RemoveAll();
	if (szArg != NULL) cslArgs.AddTail(szArg);
	return EmitMessage(nConnection, nCommand, cslArgs);
}


BOOL	CVirtual_h8App :: IsConnected(int nConnection)
{
	return m_bConnected[nConnection];
}


//	Read pipes thread
//
//	Does a round-robin read and dispatch of messages from the IPC TX pipes.
//	This is an asynchronous operation; if data is not yet ready on any pipe,
//	we go on to try the next pipe.  Each pipe has its own message buffer, and
//	each message buffer is copied to its own CStringList message queue.  But all
//	messages are dispatched from a common "OnIncomingMessage" handler -- the
//	wParam value specifies which connection caused the incoming handler call;
//	defined as in the Disconnect pipes function:
//		0 - Audio Tape
//		1 - (reserved) H-19 Terminal
//		2 - (reserved) H-17 5-1/4 Hard-Sectored Diskette
//		3 - (reserved) H-14 Printer
//		4 - (reserved) H-37 5-1/4" Diskette
//		5 - (reserved) H-47/H-67 8" Diskette
//
//	The OnIncomingMessage handler should dispatch from that list and then 
//	dispatch from all other lists until no more messages are found.
//
//	The lpParams pointer is a copy of m_pMainWnd -- the address of the main
//	window (a CWnd *) -- this is actually a (CH8CtlDlg *) and should be used
//	as such.
//
//	Note: this code isn't very graceful if pipes are broken by apps crashing.

UINT __stdcall	RxMsgThreadFunc(void * /* pParams */)
{
	{	//	Ensure that objects go out of scope before _endthreadex() is called.
		
		CVirtual_h8View *	pDlg = (CVirtual_h8View *) (theApp.m_pViewWnd);
		int					nPipeNum;
		DWORD				nResult;
		int					nCommandCode;
		DWORD				nByteCount;
		char				szMessage[MAX_MESSAGE_LENGTH + 10];
		CString				strMsg;
		int					nConnection;
		
//	Look for messages from all TX pipes every PIPE_SAMPLING_RATE milliseconds.

		for(;;)
		{

//	For every TX pipe...

			for (nPipeNum = 1; nPipeNum < IPC_ARRAY_SIZE - 1; nPipeNum += 2)
			{
				if ((WaitForSingleObject(theApp.m_IPC.hShutdown, 0) == WAIT_TIMEOUT) &&
					(WaitForSingleObject(theApp.m_IPC.hKillPipeProc,
											PIPE_SAMPLING_RATE) == WAIT_TIMEOUT))
				{
					switch(WaitForSingleObject(
							theApp.m_IPC.stPipes[nPipeNum].overControl.hEvent,
							0))
					{
					case WAIT_FAILED:

//	Fatal error.  Exit the thread.

						AfxMessageBox("wait failed.");
						goto DONE;
						break;
					case WAIT_OBJECT_0:

//	Event is signalled.  See why.  Could either be a completed connection or
//	receipt of a message.

						if (theApp.m_IPC.stPipes[nPipeNum].ePipeState ==
							PIPESTATE_CLOSED)
						{  
							if (!GetOverlappedResult(
									theApp.m_IPC.stPipes[nPipeNum].hPipe,
									&(theApp.m_IPC.stPipes[nPipeNum].overControl),
									&nByteCount,
									FALSE))
							{

//	What's happening?  I'm confused!

								AfxMessageBox("confused.");
								goto DONE;
							}
							else
							{

//	Connection created.  Presume that the application has also attached to the
//	second pipe of the pair as well.

								theApp.m_IPC.stPipes[nPipeNum].ePipeState =
									PIPESTATE_IDLE;
								theApp.m_IPC.stPipes[nPipeNum - 1].ePipeState =
									PIPESTATE_IDLE;
								theApp.m_bConnected[nPipeNum / 2] = TRUE;
							}
						}
						if (theApp.m_IPC.stPipes[nPipeNum].ePipeState ==
							PIPESTATE_IDLE)
						{

//	Connection established but no input yet asked for.  Ask for some.

							if (!ReadFile(
									theApp.m_IPC.stPipes[nPipeNum].hPipe,
									theApp.m_IPC.stPipes[nPipeNum].szBuffer,
									(theApp.m_IPC.stPipes[nPipeNum].dwBufferLength =
										MAX_MESSAGE_LENGTH),
									&(theApp.m_IPC.stPipes[nPipeNum].dwXferCount),
									&(theApp.m_IPC.stPipes[nPipeNum].overControl)))
							{
								if ((nResult = GetLastError()) !=
										ERROR_IO_PENDING)
								{

//	The pipe got an error.  Assume it broke.  Fatal error

									if (WaitForSingleObject(
										theApp.m_IPC.hShutdown,
										0) == WAIT_TIMEOUT)
									{
										AfxMessageBox("broke.");
									}
									goto DONE;
								}
								else
								{

//	The read is incomplete.

									theApp.m_IPC.stPipes[nPipeNum].ePipeState =
										PIPESTATE_XFER;
								}
							}
							else
							{

//	The read completed on first try.  Unusual, but okay.

								theApp.m_IPC.stPipes[nPipeNum].ePipeState =
									PIPESTATE_READY;
							}
						}
						if (theApp.m_IPC.stPipes[nPipeNum].ePipeState ==
							PIPESTATE_XFER)
						{

//	Had been doing an async I/O.  Check overlapped result.

 							if (!GetOverlappedResult(
									theApp.m_IPC.stPipes[nPipeNum].hPipe,
									&(theApp.m_IPC.stPipes[nPipeNum].overControl),
									&nByteCount,
									FALSE))
							{
				 				if ((nResult = GetLastError()) ==
										ERROR_IO_INCOMPLETE) 
								{

//	Incomplete is okay, though it's strange we woke up.  Go back to sleep.

								}
								else
								{

//	The pipe broke.

									if (WaitForSingleObject(
										theApp.m_IPC.hShutdown,
										0) == WAIT_TIMEOUT)
									{
										AfxMessageBox("broke.");
									}
									goto DONE;
								}
							}
							else
							{

//	The IO is complete.  Set Pipe State to READY.

								theApp.m_IPC.stPipes[nPipeNum].dwXferCount =
									nByteCount;
								theApp.m_IPC.stPipes[nPipeNum].ePipeState =
									PIPESTATE_READY;
							}
						}
						if (theApp.m_IPC.stPipes[nPipeNum].ePipeState ==
							PIPESTATE_READY)
						{

//	Message received.  Copy it and check the command code.

							memset(szMessage, 0, MAX_MESSAGE_LENGTH + 1);
							nByteCount =
								min(theApp.m_IPC.stPipes[nPipeNum].dwXferCount,
									MAX_MESSAGE_LENGTH);
							strncpy(szMessage,
									theApp.m_IPC.stPipes[nPipeNum].szBuffer,
									nByteCount);
							nCommandCode = atoi(szMessage);
							theApp.m_IPC.stPipes[nPipeNum].ePipeState = PIPESTATE_IDLE;

//	This thread must detect and act upon the SHUTDOWN_NOW_CMD and APP_IS_CLOSING
//	realtime messages -- in the first case, broadcast the message on all RX
//	pipes; in the second case, close and re-create the pipe pair for that
//	application.  Other messages can be handled asynchronously.  Put them into
//	the correct CStringList and send a message to the dialog window's attention.

							switch(nCommandCode)
							{
							case SHUTDOWN_NOW_CMD:
								for (nConnection = 0;
										nConnection < (IPC_ARRAY_SIZE / 2);
										++nConnection)
								{
									theApp.EmitMessage(nConnection,
														SHUTDOWN_NOW_CMD);
								}
								SetEvent(theApp.m_IPC.hShutdown);
								break;
							case APP_IS_CLOSING:
								if (!theApp.DisconnectPipes(nPipeNum / 2))
									goto DONE;
								break;
							default:
								nConnection = nPipeNum / 2;
								theApp.m_IPC.AddMessage(nConnection, szMessage);

//	Send UM_INCOMING message to *pDlg:
//		wParam = nConnection, lParam = tail position of the message buffer.

								if (pDlg != NULL)
								{
									if (IsWindow(pDlg -> m_hWnd))
									{
										pDlg ->
											PostMessage(UM_INCOMING,
														(WPARAM) nConnection,
														(LPARAM) 0);
									}
									else
									{
										AfxMessageBox("pDlg not a window");
									}
								}
								else
								{
									AfxMessageBox("pDlg NULL");
								}
								break;
							}
						}
						break;
					default:

//	Event is unsignalled.  Nothing to do.

						break;
					}
				}
				else
				{
					goto DONE;
				}
			}
		}
DONE:
		;

	}
	:: _endthreadex(0);
	return 0L;
}
