/************************************************************************
 *																		*
 *	Copyright (c) 1998 - 2001 by	David A. Wallace					*
 *									154 Dunstable Rd.					*
 *									N. Chelmsford, MA					*
 *											01863-1202					*
 *																		*
 ************************************************************************/


// virtual_h8View.cpp : implementation of the CVirtual_h8View class
//

#include "stdafx.h"
#include "virtual_h8.h"

#include "virtual_h8Doc.h"
#include "virtual_h8View.h"
#include "configdlg.h"
#include "keyhelpdlg.h"
#include "h8configdlg.h"


#define	EXTERN extern

#ifndef _BUS_H_
#include "bus.h"
#endif

#ifndef _GLOBALS_H_
#include "globals.h"
#endif

#ifndef	_PROCESSOR_H_
#include "processor.h"
#endif

#ifndef _LOGICANALYZER_H_
#include "logicanalyzer.h"
#endif

#include "ini_file.h"
#include "mainfrm.h"
#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define	H8_5_PROGRAM_NAME	"tapeui.exe"
#define	H17_PROGRAM_NAME	"h17ui.exe"


/////////////////////////////////////////////////////////////////////////////
// Global objects


extern	HANDLE			g_hCPUThread;



/////////////////////////////////////////////////////////////////////////////
// CVirtual_h8View

IMPLEMENT_DYNCREATE(CVirtual_h8View, CFormView)

BEGIN_MESSAGE_MAP(CVirtual_h8View, CFormView)
	//{{AFX_MSG_MAP(CVirtual_h8View)
	ON_WM_LBUTTONDOWN()
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_KBD_HELP_CK, OnKbdHelpCk)
	ON_BN_CLICKED(IDC_LOCAL, OnLocal)
	ON_COMMAND(IDM_VIEW_P3, OnViewP3)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_P3, OnUpdateViewP3)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_P2, OnUpdateViewP2)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_P1, OnUpdateViewP1)
	ON_UPDATE_COMMAND_UI(IDM_VIEW_P0, OnUpdateViewP0)
	ON_COMMAND(IDM_CONFIG_H19, OnConfigH19)
	ON_COMMAND(IDM_VIEW_P2, OnViewP2)
	ON_COMMAND(IDM_VIEW_P1, OnViewP1)
	ON_COMMAND(IDM_VIEW_P0, OnViewP0)
	ON_COMMAND(IDM_H8_5, OnH85)
	ON_UPDATE_COMMAND_UI(IDM_H8_5, OnUpdateH85)
	ON_COMMAND(IDM_CONFIG_H8, OnConfigH8)
	//}}AFX_MSG_MAP
    ON_MESSAGE(UM_INCOMING, OnIncoming)
    ON_MESSAGE(UM_CHAR_TO_H19, OnCharToH19)
    ON_MESSAGE(UM_BREAK_TO_H19, OnBreakToH19)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CVirtual_h8View construction/destruction

CVirtual_h8View::CVirtual_h8View()
	: CFormView(CVirtual_h8View::IDD)
{
	//{{AFX_DATA_INIT(CVirtual_h8View)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// TODO: add construction code here

//	The Photorealism control adjusts the presentation to allow the user to
//	choose between having a realistic-appearing presentation or saving screen
//	real-estate.  The plan is to support four possible values:
//
//	Value		Description		Notes
//	-----------	--------------- ------------------------------------------------
//	3 (default)	Photorealistic	The H-8 and H-19 displays are arranged to look
//								as much like the actual devices as possible and
//								show all the devices' visual features.
//	2			Space Saving	The empty areas to the left and right of the H-8
//								keypad are removed; The H-19's name-plate is
//								replaced with a simple "Key Help" button and the
//								key mapping table becomes a pop-up dialog.  The
//								"key help" and "on line" buttons are aligned
//								vertically close to the right edge of the "CRT".
//	1			Compact			The H-8's layout is extended vertically and the
//								keypad is moved under the display, displacing
//								the logo (and magnet) left a bit.
//	0			Minimalism		The presentation degenerates to the H-8 display
//								at the upper left part of the screen with the
//								H-19 "CRT" directly below.  To the right is the
//								H-8 keyboard (with the magnet directly under the
//								juncture of the "F" and "E" keys) The H-19's
//								buttons are positioned below "RTM" and "RESET"
//								keys.  The individual graphical elements remain
//								photorealistic, but background colors and logos
//								are deleted.
//
//	Mode 3 requires an area approximately 2000 pixels wide by 600 pixels high.
//	Mode 2 requires (estimate) 1600 by 600.
//	Mode 1 requires (estimate) 1200 by 600.
//	Mode 0 requires (estimate) 800 by 800.
//
//	I can't do much about the height -- the H-19 "CRT" display is 640 by 500
//	pixels and the H-8 display is 525 wide by 91 high.  So 600 pixels is always
//	the minimum height that shows everything and for mode 0 the display is even
//	taller.
//
//	Of these modes, initially only Mode 3 is implemented.  The rearrangement of
//	controls is progressively harder to program as we move from photorealistic
//	to minimalist, so I'll probably tackle the others in the order listed.

	m_nPhotorealismMode = 3;	//	3 (default) is photorealistic presentation
	m_pMagnetButton = NULL;
	m_bLocal = TRUE;
}

CVirtual_h8View::~CVirtual_h8View()
{
	if (m_pMagnetButton != NULL)
	{
		delete m_pMagnetButton;
		m_pMagnetButton = NULL;
	}
}

void CVirtual_h8View::DoDataExchange(CDataExchange* pDX)
{
	CFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CVirtual_h8View)
	DDX_Control(pDX, IDC_H19_FRAME, m_H19Frame);
	DDX_Control(pDX, IDC_H8_FRAME, m_H8Frame);
	DDX_Control(pDX, IDC_KEYHELP, m_KeyhelpGraphic);
	DDX_Control(pDX, IDC_KBD_HELP_CK, m_KbdHelpBtn);
	DDX_Control(pDX, IDC_LOCAL, m_Local);
	DDX_Control(pDX, IDC_H19_NAMEPLATE, m_Nameplate);
	DDX_Control(pDX, IDC_CPTHREAD_STATUS, m_CPThreadStatusWindow);
	DDX_Control(pDX, IDC_IOTHREAD_STATUS, m_IOThreadStatusWindow);
	//}}AFX_DATA_MAP
}

BOOL CVirtual_h8View::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CFormView::PreCreateWindow(cs);
}


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//
//		Set up the visual appearance for the H-8 and H-19 in our view.
//
//
//	The following #defines place all the graphical elements of the UI.
//	(There are certainly better ways to do this!  This stuff is highly dependent
//	on the size of a dialog unit in pixels! -- It won't work right if the user
//	elects to use small font, for example.)

/*

The H8 panel was tilted at about 60 degrees.  The actual
dimensions of the red plastic filter over the LEDs is 7-7/8 x 1-3/8; the

digits are centered vertically and are one inch high.

The keypad is 3" square.
Logo is 4-1/8x1/2

Panel is 15" wide x 4-1/2 high (keypad centered vertically) to bend;
bull-nose is 1" (at about 30 degrees).

LED display origin is is 1/2x1/2; keypad right margin is 2"

Logo is 1-7/8" from left edge and its bottom is 1/2" above fold.



*/

/*
	For the combined GUI, I'm allowing 20 pixels horizontally between the H-8
	(which is positioned first) and the H-19.
*/



//	These are the Layout template sets.  The scale for the H-8 was set such that
//	one inch = 67 pixels.  The H19 is almost to this scale, too, but it had to
//	stay in proportion to a CRT showing a 640x500-pixel display. The H-19's
//	controls and displays do not require a really elaborate relative-positioning
//	scheme like the H-8's.  [Yet.]


//	These are the default (Photorealistic) positions.  Subsequent layouts will
//	have identical names except be prefixed "P2_", "P1_" or "P0_" instead of
//	"P3_".  The exceptions are "BOTTOM_KEYSET_MARGIN" through "TEXT_SPACING",
//	which do not vary with layout.

#define	BOTTOM_KEYSET_MARGIN	  75
#define	KEYWIDTH				  50
#define	KEYHEIGHT				  50
#define DISPLAY_WIDTH			 525
#define DISPLAY_HEIGHT			  91
#define LOGO_HEIGHT				  33
#define MAGNET_HEIGHT			  16
#define BEVEL_HEIGHT			   2
#define	H19_SCREEN_WIDTH		 660
#define	H19_SCREEN_HEIGHT		 540
#define	H19_LOGO_WIDTH			 250
#define	H19_LOGO_HEIGHT			 425
#define	TEXT_MARGIN				   5
#define	TEXT_HEIGHT				  20
#define	TEXT_WIDTH				 300
#define	TEXT_SPACING			  15

//	----------------------------------------------------------------------------
//	PhotorealismMode = 3

//	for the H-8:

#define	P3_WINDOW_TOP			 190
#define	P3_INTER_APP_MARGIN		  20
#define P3_LEFT_MARGIN			  42
#define P3_RIGHT_MARGIN			 133
#define P3_TOP_MARGIN			  33
#define P3_BOTTOM_MARGIN		  17
#define P3_WINDOW_WIDTH			1000
#define P3_BEVEL_TOP_MARGIN		  32
#define P3_BEVEL_BOTTOM_MARGIN	  16
#define P3_BEVEL_POSITION		 300
#define P3_LOGO_POSITION		 120
#define P3_LOGO_WIDTH			 283
#define P3_MAGNET_POSITION			(P3_LOGO_POSITION + 65)
#define P3_MAGNET_WIDTH			  40
#define P3_SUBBEVEL_HEIGHT		  62
#define P3_WINDOW_HEIGHT			(P3_BEVEL_POSITION + BEVEL_HEIGHT + \
									P3_SUBBEVEL_HEIGHT + P3_BOTTOM_MARGIN + 20)
#define P3_KEYPAD_POSITION			(P3_BEVEL_POSITION / 2 - 2 * KEYHEIGHT)
#define P3_TEXT_POSITION			(P3_LEFT_MARGIN + 45)

//	for the H19:

#define	P3_H19_TOP_MARGIN		  20
#define	P3_H19_LEFT_MARGIN		  20
#define	P3_H19_LOGO_LEFT_SPACE	  20
#define	P3_H19_LOGO_RIGHT_SPACE	  20
#define	P3_H19_BOTTOM_SPACE		  20
#define	P3_H19_BOTTOM_MARGIN	  20
#define	P3_H19_WINDOW_WIDTH			(P3_H19_LEFT_MARGIN + H19_SCREEN_WIDTH + \
										H19_LOGO_WIDTH + \
										P3_H19_LOGO_LEFT_SPACE + \
										P3_H19_LOGO_RIGHT_SPACE)
#define	P3_H19_WINDOW_HEIGHT		(P3_H19_TOP_MARGIN + H19_SCREEN_HEIGHT + \
										P3_H19_BOTTOM_MARGIN)



//	----------------------------------------------------------------------------
//	PhotorealismMode = 2

//	for the H-8:

#define	P2_WINDOW_TOP			 190
#define	P2_INTER_APP_MARGIN		  10
#define P2_LEFT_MARGIN			  10
#define P2_RIGHT_MARGIN			  10
#define P2_TOP_MARGIN			  33
#define P2_BOTTOM_MARGIN		  17
#define P2_WINDOW_WIDTH			1000 - 155 - 100 + 10
#define P2_BEVEL_TOP_MARGIN		  32
#define P2_BEVEL_BOTTOM_MARGIN	  16
#define P2_BEVEL_POSITION		 300
#define P2_LOGO_POSITION		    P2_LEFT_MARGIN
#define P2_LOGO_WIDTH			 283
#define P2_MAGNET_POSITION			(P2_LOGO_POSITION + 65)
#define P2_MAGNET_WIDTH			  40
#define P2_SUBBEVEL_HEIGHT		  62
#define P2_WINDOW_HEIGHT			(P2_BEVEL_POSITION + BEVEL_HEIGHT + \
									P2_SUBBEVEL_HEIGHT + P2_BOTTOM_MARGIN + 20)
#define P2_KEYPAD_POSITION			(P2_BEVEL_POSITION / 2 - 2 * KEYHEIGHT)
#define P2_TEXT_POSITION			P2_LEFT_MARGIN

//	for the H19:

#define	P2_H19_TOP_MARGIN		  20
#define	P2_H19_LEFT_MARGIN		  10
#define	P2_H19_LOGO_LEFT_SPACE	  20
#define	P2_H19_LOGO_RIGHT_SPACE	  20
#define	P2_H19_BOTTOM_SPACE		  20
#define	P2_H19_BOTTOM_MARGIN	  20
#define	P2_H19_WINDOW_WIDTH			(P2_H19_LEFT_MARGIN + H19_SCREEN_WIDTH + \
										H19_LOGO_WIDTH + \
										P2_H19_LOGO_LEFT_SPACE + \
										P2_H19_LOGO_RIGHT_SPACE)
#define	P2_H19_WINDOW_HEIGHT		(P2_H19_TOP_MARGIN + H19_SCREEN_HEIGHT + \
										P2_H19_BOTTOM_MARGIN)


//	----------------------------------------------------------------------------
//	PhotorealismMode = 1

//	for the H-8:

#define	P1_WINDOW_TOP				(190 - DISPLAY_HEIGHT)
#define	P1_INTER_APP_MARGIN		  10
#define P1_LEFT_MARGIN			  10
#define P1_RIGHT_MARGIN			  10
#define P1_TOP_MARGIN			  33
#define P1_BOTTOM_MARGIN		  17
#define P1_WINDOW_WIDTH			1000 - 155 - 100 - KEYWIDTH * 4
#define P1_BEVEL_TOP_MARGIN		  32
#define P1_BEVEL_BOTTOM_MARGIN	  16
#define P1_BEVEL_POSITION			(300 + DISPLAY_HEIGHT)
#define P1_LOGO_POSITION		    P1_LEFT_MARGIN
#define P1_LOGO_WIDTH			 283
#define P1_MAGNET_POSITION			(P1_LOGO_POSITION + 65)
#define P1_MAGNET_WIDTH			  40
#define P1_SUBBEVEL_HEIGHT		  62
#define P1_WINDOW_HEIGHT			(P1_BEVEL_POSITION + BEVEL_HEIGHT + \
									P1_SUBBEVEL_HEIGHT + P1_BOTTOM_MARGIN + 20)
#define P1_KEYPAD_POSITION			(DISPLAY_HEIGHT + \
										(P1_BEVEL_POSITION - \
										DISPLAY_HEIGHT)/ 2 - 2 * KEYHEIGHT)
#define P1_TEXT_POSITION			P1_LEFT_MARGIN

//	for the H19:

#define	P1_H19_TOP_MARGIN		  20
#define	P1_H19_LEFT_MARGIN		  10
#define	P1_H19_LOGO_LEFT_SPACE	  20
#define	P1_H19_LOGO_RIGHT_SPACE	  20
#define	P1_H19_BOTTOM_SPACE		  20
#define	P1_H19_BOTTOM_MARGIN	  20
#define	P1_H19_WINDOW_WIDTH			(P1_H19_LEFT_MARGIN + H19_SCREEN_WIDTH + \
										H19_LOGO_WIDTH + \
										P1_H19_LOGO_LEFT_SPACE + \
										P1_H19_LOGO_RIGHT_SPACE)
#define	P1_H19_WINDOW_HEIGHT		(P1_H19_TOP_MARGIN + H19_SCREEN_HEIGHT + \
										P1_H19_BOTTOM_MARGIN)


//	----------------------------------------------------------------------------
//	PhotorealismMode = 0

//	for the H-8:

#define	P0_WINDOW_TOP			   0
#define	P0_INTER_APP_MARGIN		   0
#define P0_LEFT_MARGIN			  10
#define P0_RIGHT_MARGIN			  10
#define P0_TOP_MARGIN			  10
#define P0_BOTTOM_MARGIN		  10
#define P0_WINDOW_WIDTH				(H19_SCREEN_WIDTH + 4 * KEYWIDTH + P0_LEFT_MARGIN + P0_RIGHT_MARGIN)
#define P0_BEVEL_TOP_MARGIN		   0
#define P0_BEVEL_BOTTOM_MARGIN	   0
#define P0_BEVEL_POSITION		   0
#define P0_LOGO_POSITION		   P0_LEFT_MARGIN
#define P0_LOGO_WIDTH			 283
#define P0_MAGNET_WIDTH			  40
#define P0_SUBBEVEL_HEIGHT		   0
#define P0_WINDOW_HEIGHT			(P0_TOP_MARGIN + DISPLAY_HEIGHT + 10 + H19_SCREEN_HEIGHT + P0_BOTTOM_MARGIN)
#define P0_KEYPAD_POSITION			P0_TOP_MARGIN + DISPLAY_HEIGHT + 2 * KEYHEIGHT
#define P0_MAGNET_POSITION			(P0_LOGO_POSITION +  65)
#define P0_TEXT_POSITION			P0_LEFT_MARGIN

//	for the H19:

#define	P0_H19_TOP_MARGIN		   0
#define	P0_H19_LEFT_MARGIN		   0
#define	P0_H19_LOGO_LEFT_SPACE	   0
#define	P0_H19_LOGO_RIGHT_SPACE	   0
#define	P0_H19_BOTTOM_SPACE		   0
#define	P0_H19_BOTTOM_MARGIN	   0
#define	P0_H19_WINDOW_WIDTH		   H19_SCREEN_WIDTH + 10
#define	P0_H19_WINDOW_HEIGHT	   0


void CVirtual_h8View::OnInitialUpdate()
{
	CMenu*	pSysMenu = GetSystemMenu(FALSE);
	char	szWd[MAX_PATH];
	int		nI;

	CFormView::OnInitialUpdate();

//	Retrieve visual state and set m_nPhotorealismMode

	GetCurrentDirectory(MAX_PATH, szWd);
	m_strIniFile = szWd;
	m_strIniFile += "\\" INI_FILE_NAME;
	theApp.m_nTau = GetPrivateProfileInt(INI_KEY_TAU, 2, (LPCSTR) m_strIniFile);
	m_nPhotorealismMode = GetPrivateProfileInt(INI_KEY_APPEARANCE,
												3,
												(LPCSTR) m_strIniFile);

//	Create graphical objects of dummy sizes.

	m_wDisplay.Create(NULL,
						"H-8 Front Panel LED Display",
						WS_CHILD | WS_VISIBLE | WS_GROUP,
						CRect(0,0,240,100),
						this,
						IDW_DISPLAY,
						NULL);
	m_pMagnetButton = (CStatic *) new CStatic;
	m_pMagnetButton -> Create(NULL,
								WS_CHILD | WS_VISIBLE | WS_GROUP | SS_BITMAP,
								CRect(0,0,30,10),
								this,
								IDC_MAGNET);
	m_Screen.Create(NULL, "H19 Screen",
						WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP,
						CRect(0,0,640,500), this, IDC_SCREEN);

//	Initialize the bitmap buttons.

	if (!m_MagnetBmp.LoadBitmap(IDB_MANGET))
		AfxMessageBox("Unable to load IDB_MAGNET.");
	if (!m_NoMagnetBmp.LoadBitmap(IDB_NOMAGNET))
		AfxMessageBox("Unable to load IDB_NOMAGNET.");
	m_Keypad1Btn.AutoLoad(IDC_BUTTON1, this);
	m_Keypad1Btn.m_pSignal = OnSevenButton;
	m_Keypad2Btn.AutoLoad(IDC_BUTTON2, this);
	m_Keypad2Btn.m_pSignal = OnEightButton;
	m_Keypad3Btn.AutoLoad(IDC_BUTTON3, this);
	m_Keypad3Btn.m_pSignal = OnNineButton;
	m_Keypad4Btn.AutoLoad(IDC_BUTTON4, this);
	m_Keypad4Btn.m_pSignal = OnAButton;
	m_Keypad5Btn.AutoLoad(IDC_BUTTON5, this);
	m_Keypad5Btn.m_pSignal = OnFourButton;
	m_Keypad6Btn.AutoLoad(IDC_BUTTON6, this);
	m_Keypad6Btn.m_pSignal = OnFiveButton;
	m_Keypad7Btn.AutoLoad(IDC_BUTTON7, this);
	m_Keypad7Btn.m_pSignal = OnSixButton;
	m_Keypad8Btn.AutoLoad(IDC_BUTTON8, this);
	m_Keypad8Btn.m_pSignal = OnBButton;
	m_Keypad9Btn.AutoLoad(IDC_BUTTON9, this);
	m_Keypad9Btn.m_pSignal = OnOneButton;
	m_Keypad10Btn.AutoLoad(IDC_BUTTON10, this);
	m_Keypad10Btn.m_pSignal = OnTwoButton;
	m_Keypad11Btn.AutoLoad(IDC_BUTTON11, this);
	m_Keypad11Btn.m_pSignal = OnThreeButton;
	m_Keypad12Btn.AutoLoad(IDC_BUTTON12, this);
	m_Keypad12Btn.m_pSignal = OnCButton;
	m_Keypad13Btn.AutoLoad(IDC_BUTTON13, this);
	m_Keypad13Btn.m_pSignal = OnZeroButton;
	m_Keypad14Btn.AutoLoad(IDC_BUTTON14, this);
	m_Keypad14Btn.m_pSignal = OnFButton;
	m_Keypad15Btn.AutoLoad(IDC_BUTTON15, this);
	m_Keypad15Btn.m_pSignal = OnEButton;
	m_Keypad16Btn.AutoLoad(IDC_BUTTON16, this);
	m_Keypad16Btn.m_pSignal = OnDButton;
	m_Keypad17Btn.AutoLoad(IDC_BUTTON17, this);
	m_Keypad17Btn.m_pSignal = OnRTMButton;
	m_Keypad18Btn.AutoLoad(IDC_BUTTON18, this);
	m_Keypad18Btn.m_pSignal = OnResetButton;

//	Now calculate the actual positions of things and position them.

	CalculateLayout();

	m_IOThreadStatusWindow.SetWindowText("");
	m_IOThreadStatusWindow.ShowWindow(SW_HIDE);
	m_CPThreadStatusWindow.SetWindowText("CPU suspended");
	m_CPThreadStatusWindow.ShowWindow(SW_SHOW);
	for (nI = 0; nI < 9; ++nI)
	{
		theApp.m_PAM_IO_PORTS.nMAN1[nI] = PANEL_MAN1_SPACE;
	}
	m_wDisplay.m_bPowerLED = TRUE;
	m_wDisplay.ShowWindow(SW_SHOW);

////////////////////////////////////////////////////////////////////////////////
//
//	Finish the view's initialization by configuring the logic analyzer and
//	setting up the H-19's power-on configuration.

	m_Screen.m_bReverseVideo = GetPrivateProfileInt(INI_KEY_REVERSE_VIDEO,
								FALSE,
								(LPCSTR) m_strIniFile);
	m_Screen.RedrawWindow();
	m_Local.SetCheck(!(m_bLocal = !GetPrivateProfileInt(INI_KEY_ONLINE,
											FALSE,
											(LPCSTR) m_strIniFile)));
	m_Screen.m_bLocal = m_bLocal;

#ifdef _DEBUG
	m_Screen.DisplayTest();
#endif



#ifdef _DEBUG

//	Configure the optional logic analyzer.

	ConfigLogicAnalyzer();

#endif



////////////////////////////////////////////////////////////////////////////////
//
//	Activate the timer and un-suspend the bus. This will release the H-8's
//	emulation threads.

	theApp.m_pViewWnd = this;
	OnTimer(0);
	m_nTimerID = SetTimer(PANEL_TIMER, PANEL_TIMER_PERIOD, NULL);
	GetDocument() -> SetTitle("Virtual H8");
}

void	CVirtual_h8View :: CalculateLayout()
{
	CRect	rDlg;
	CRect	rWnd;
	CRect	rPowerSwitch;
	CRect	rLogo;
	CRect	rMagnet;
	CRect	rKeypad;
	CRect	rBevel;
	CRect	rIOStatus;
	CRect	rCPStatus;
	CRect	rScreen;
	CString	strTemp;
	int		nH19Left;
	int		nH19Top;
	CRect	rNameplate;
	CRect	rControl;
	CRect	rKbdMap;
	int		nWidth;
	int		nHeight;
	int		nBottom;
	int		nViewHeight;
	int		nViewWidth;
	CRect	rDesktop;

	ScrollToPosition(CPoint(0,0));

	GetDesktopWindow() -> GetWindowRect(&rDesktop);

////////////////////////////////////////////////////////////////////////////////
//
//	First place all the graphical elements for the H-8.

	m_H8Frame.GetWindowRect(&rDlg);
	ScreenToClient(&rDlg);
	nWidth = rDlg.Width();
	rDlg.left = 10;
	rDlg.right = rDlg.left + nWidth;
	switch(m_nPhotorealismMode)
	{
	default:
	case 0:	//	Minimalism
		GetDlgItem(IDC_H8NAMEPLATE) -> ShowWindow(SW_SHOW);
		m_H8Frame.ShowWindow(SW_SHOW);
		GetDlgItem(IDC_BEVEL) -> ShowWindow(SW_HIDE);
		rDlg.top = P0_WINDOW_TOP;
		rDlg.bottom = P0_WINDOW_HEIGHT;
		rDlg.right = P0_WINDOW_WIDTH;
		rWnd = CRect(rDlg.left + P0_LEFT_MARGIN,
						rDlg.top + P0_TOP_MARGIN,
						rDlg.left + P0_LEFT_MARGIN + DISPLAY_WIDTH,
						rDlg.top + P0_TOP_MARGIN + DISPLAY_HEIGHT);
		rKeypad = CRect(rDlg.left + P0_LEFT_MARGIN + TEXT_WIDTH / 2 - 2 * KEYWIDTH,
						rDlg.top + P0_KEYPAD_POSITION,
						rDlg.left + P0_LEFT_MARGIN + TEXT_WIDTH / 2 - 2 * KEYWIDTH + KEYWIDTH,
						rDlg.top + P0_KEYPAD_POSITION + KEYHEIGHT);
		rBevel = CRect(0,0,0,0);
		rIOStatus = CRect(rDlg.left + P0_TEXT_POSITION,
							rDlg.top + P0_KEYPAD_POSITION + 7 * KEYHEIGHT,
							rDlg.left + P0_TEXT_POSITION + TEXT_WIDTH,
							rDlg.top + P0_KEYPAD_POSITION + 7 * KEYHEIGHT +
								TEXT_HEIGHT);
		rCPStatus =  CRect(rDlg.left + P0_TEXT_POSITION,
							rDlg.top + P0_KEYPAD_POSITION + 7 * KEYHEIGHT +
								TEXT_HEIGHT + TEXT_MARGIN,
							rDlg.left + P0_TEXT_POSITION + TEXT_WIDTH,
							rDlg.top + P0_KEYPAD_POSITION + 7 * KEYHEIGHT +
								TEXT_HEIGHT + TEXT_MARGIN +
								TEXT_HEIGHT);
		rLogo = CRect(rDlg.left + P0_LOGO_POSITION,
						rDlg.top + P0_KEYPAD_POSITION + 7 * KEYHEIGHT +
							2 * TEXT_HEIGHT + 5 * TEXT_MARGIN,
						rDlg.left + P0_LOGO_POSITION + P0_LOGO_WIDTH,
						rDlg.top + P0_KEYPAD_POSITION + 7 * KEYHEIGHT +
							2 * TEXT_HEIGHT + 5 * TEXT_MARGIN + LOGO_HEIGHT);
		rMagnet = CRect(rDlg.left + P0_MAGNET_POSITION,
						rLogo.top - MAGNET_HEIGHT - 2,
						rDlg.left + P0_MAGNET_POSITION + P0_MAGNET_WIDTH,
						rLogo.top - 2);
		nH19Left = rCPStatus.right + 10;
		nH19Top = rWnd.bottom + 10;
		break;
	case 1:	//	Compact
		m_H8Frame.ShowWindow(SW_SHOW);
		rDlg.top = P1_WINDOW_TOP;
		rDlg.right = rDlg.left + P1_WINDOW_WIDTH;
		rDlg.bottom = rDlg.top + P1_WINDOW_HEIGHT;
		m_H8Frame.MoveWindow(&rDlg);
		rWnd = CRect(rDlg.left + P1_LEFT_MARGIN,
						rDlg.top + P1_TOP_MARGIN,
						rDlg.left + P1_LEFT_MARGIN + DISPLAY_WIDTH,
						rDlg.top + P1_TOP_MARGIN + DISPLAY_HEIGHT);
		rLogo = CRect(rDlg.left + P1_LOGO_POSITION,
						rDlg.top + P1_BEVEL_POSITION - P1_BEVEL_TOP_MARGIN -
							3 * LOGO_HEIGHT / 2,
						rDlg.left + P1_LOGO_POSITION + P1_LOGO_WIDTH,
						rDlg.top + P1_BEVEL_POSITION - P1_BEVEL_TOP_MARGIN -
						LOGO_HEIGHT / 2);
		rMagnet = CRect(rDlg.left + P1_MAGNET_POSITION,
							rDlg.top + P1_BEVEL_POSITION - P1_BEVEL_TOP_MARGIN -
								3 * LOGO_HEIGHT / 2 - MAGNET_HEIGHT,
							rDlg.left + P1_MAGNET_POSITION + P1_MAGNET_WIDTH,
							rDlg.top + P1_BEVEL_POSITION - P1_BEVEL_TOP_MARGIN -
								3 * LOGO_HEIGHT / 2);
		rKeypad = CRect(rDlg.left + P1_WINDOW_WIDTH - P1_RIGHT_MARGIN -
							4 * KEYWIDTH,
						rDlg.top + P1_KEYPAD_POSITION,
						rDlg.left + P1_WINDOW_WIDTH - P1_RIGHT_MARGIN -
							4 * KEYWIDTH + KEYWIDTH,
						rDlg.top + P1_KEYPAD_POSITION + KEYHEIGHT);
		rBevel = CRect(rDlg.left + 1,
						rDlg.top + P1_BEVEL_POSITION,
						rDlg.left + P1_WINDOW_WIDTH - 2,
						rDlg.top + P1_BEVEL_POSITION + BEVEL_HEIGHT);
		rIOStatus = CRect(rDlg.left + P1_TEXT_POSITION,
							rDlg.top + P1_BEVEL_POSITION + BEVEL_HEIGHT +
								P1_BEVEL_BOTTOM_MARGIN + TEXT_MARGIN,
							rDlg.left + P1_TEXT_POSITION + TEXT_WIDTH,
							rDlg.top + P1_BEVEL_POSITION + BEVEL_HEIGHT +
								P1_BEVEL_BOTTOM_MARGIN + TEXT_MARGIN +
								TEXT_HEIGHT);
		rCPStatus = CRect(rDlg.left + P1_TEXT_POSITION,
							rDlg.top + P1_BEVEL_POSITION + BEVEL_HEIGHT +
								P1_BEVEL_BOTTOM_MARGIN + TEXT_HEIGHT +
								TEXT_MARGIN + TEXT_SPACING,
							rDlg.left + P1_TEXT_POSITION + TEXT_WIDTH,
							rDlg.top + P1_BEVEL_POSITION + BEVEL_HEIGHT +
								P1_BEVEL_BOTTOM_MARGIN +
								TEXT_HEIGHT + TEXT_MARGIN + TEXT_SPACING +
								TEXT_HEIGHT);
		nH19Top = 10;
		nH19Left = rDlg.right + P1_INTER_APP_MARGIN;
		GetDlgItem(IDC_H8NAMEPLATE) -> ShowWindow(SW_SHOW);
		GetDlgItem(IDC_BEVEL) -> ShowWindow(SW_SHOW);
		break;
	case 2:	//	Space Saving
		m_H8Frame.ShowWindow(SW_SHOW);
		rDlg.top = P2_WINDOW_TOP;
		rDlg.right = rDlg.left + P2_WINDOW_WIDTH;
		rDlg.bottom = rDlg.top + P2_WINDOW_HEIGHT;
		m_H8Frame.MoveWindow(&rDlg);
		rWnd = CRect(rDlg.left + P2_LEFT_MARGIN,
						rDlg.top + P2_TOP_MARGIN,
						rDlg.left + P2_LEFT_MARGIN + DISPLAY_WIDTH,
						rDlg.top + P2_TOP_MARGIN + DISPLAY_HEIGHT);
		rLogo = CRect(rDlg.left + P2_LOGO_POSITION,
						rDlg.top + P2_BEVEL_POSITION - P2_BEVEL_TOP_MARGIN -
							LOGO_HEIGHT,
						rDlg.left + P2_LOGO_POSITION + P2_LOGO_WIDTH,
						rDlg.top + P2_BEVEL_POSITION - P2_BEVEL_TOP_MARGIN);
		rMagnet = CRect(rDlg.left + P2_MAGNET_POSITION,
							rDlg.top + P2_BEVEL_POSITION - P2_BEVEL_TOP_MARGIN -
								LOGO_HEIGHT - MAGNET_HEIGHT,
							rDlg.left + P2_MAGNET_POSITION + P2_MAGNET_WIDTH,
							rDlg.top + P2_BEVEL_POSITION - P2_BEVEL_TOP_MARGIN -
								LOGO_HEIGHT);
		rKeypad = CRect(rDlg.left + P2_WINDOW_WIDTH - P2_RIGHT_MARGIN -
							4 * KEYWIDTH,
						rDlg.top + P2_KEYPAD_POSITION,
						rDlg.left + P2_WINDOW_WIDTH - P2_RIGHT_MARGIN -
							4 * KEYWIDTH + KEYWIDTH,
						rDlg.top + P2_KEYPAD_POSITION + KEYHEIGHT);
		rBevel = CRect(rDlg.left + 1,
						rDlg.top + P2_BEVEL_POSITION,
						rDlg.left + P2_WINDOW_WIDTH - 2,
						rDlg.top + P2_BEVEL_POSITION + BEVEL_HEIGHT);
		rIOStatus = CRect(rDlg.left + P2_TEXT_POSITION,
							rDlg.top + P2_BEVEL_POSITION + BEVEL_HEIGHT +
								P2_BEVEL_BOTTOM_MARGIN + TEXT_MARGIN,
							rDlg.left + P2_TEXT_POSITION + TEXT_WIDTH,
							rDlg.top + P2_BEVEL_POSITION + BEVEL_HEIGHT +
								P2_BEVEL_BOTTOM_MARGIN + TEXT_MARGIN +
								TEXT_HEIGHT);
		rCPStatus = CRect(rDlg.left + P2_TEXT_POSITION,
							rDlg.top + P2_BEVEL_POSITION + BEVEL_HEIGHT +
								P2_BEVEL_BOTTOM_MARGIN + TEXT_HEIGHT +
								TEXT_MARGIN + TEXT_SPACING,
							rDlg.left + P2_TEXT_POSITION + TEXT_WIDTH,
							rDlg.top + P2_BEVEL_POSITION + BEVEL_HEIGHT +
								P2_BEVEL_BOTTOM_MARGIN +
								TEXT_HEIGHT + TEXT_MARGIN + TEXT_SPACING +
								TEXT_HEIGHT);
		nH19Top = 10;
		nH19Left = rDlg.right + P2_INTER_APP_MARGIN;
		GetDlgItem(IDC_H8NAMEPLATE) -> ShowWindow(SW_SHOW);
		GetDlgItem(IDC_BEVEL) -> ShowWindow(SW_SHOW);
		break;
	case 3:	//	Photorealism
		m_H8Frame.ShowWindow(SW_SHOW);
		rDlg.top = P3_WINDOW_TOP;
		rDlg.right = rDlg.left + P3_WINDOW_WIDTH;
		rDlg.bottom = rDlg.top + P3_WINDOW_HEIGHT;
		m_H8Frame.MoveWindow(&rDlg);
		rWnd = CRect(rDlg.left + P3_LEFT_MARGIN,
						rDlg.top + P3_TOP_MARGIN,
						rDlg.left + P3_LEFT_MARGIN + DISPLAY_WIDTH,
						rDlg.top + P3_TOP_MARGIN + DISPLAY_HEIGHT);
		rLogo = CRect(rDlg.left + P3_LOGO_POSITION,
						rDlg.top + P3_BEVEL_POSITION - P3_BEVEL_TOP_MARGIN -
							LOGO_HEIGHT,
						rDlg.left + P3_LOGO_POSITION + P3_LOGO_WIDTH,
						rDlg.top + P3_BEVEL_POSITION - P3_BEVEL_TOP_MARGIN);
		rMagnet = CRect(rDlg.left + P3_MAGNET_POSITION,
							rDlg.top + P3_BEVEL_POSITION - P3_BEVEL_TOP_MARGIN -
								LOGO_HEIGHT - MAGNET_HEIGHT,
							rDlg.left + P3_MAGNET_POSITION + P3_MAGNET_WIDTH,
							rDlg.top + P3_BEVEL_POSITION - P3_BEVEL_TOP_MARGIN -
								LOGO_HEIGHT);
		rKeypad = CRect(rDlg.left + P3_WINDOW_WIDTH - P3_RIGHT_MARGIN -
							4 * KEYWIDTH,
						rDlg.top + P3_KEYPAD_POSITION,
						rDlg.left + P3_WINDOW_WIDTH - P3_RIGHT_MARGIN -
							4 * KEYWIDTH + KEYWIDTH,
						rDlg.top + P3_KEYPAD_POSITION + KEYHEIGHT);
		rBevel = CRect(rDlg.left + 1,
						rDlg.top + P3_BEVEL_POSITION,
						rDlg.left + P3_WINDOW_WIDTH - 2,
						rDlg.top + P3_BEVEL_POSITION + BEVEL_HEIGHT);
		rIOStatus = CRect(rDlg.left + P3_TEXT_POSITION,
							rDlg.top + P3_BEVEL_POSITION + BEVEL_HEIGHT +
								P3_BEVEL_BOTTOM_MARGIN + TEXT_MARGIN,
							rDlg.left + P3_TEXT_POSITION + TEXT_WIDTH,
							rDlg.top + P3_BEVEL_POSITION + BEVEL_HEIGHT +
								P3_BEVEL_BOTTOM_MARGIN + TEXT_MARGIN +
								TEXT_HEIGHT);
		rCPStatus = CRect(rDlg.left + P3_TEXT_POSITION,
							rDlg.top + P3_BEVEL_POSITION + BEVEL_HEIGHT +
								P3_BEVEL_BOTTOM_MARGIN + TEXT_HEIGHT +
								TEXT_MARGIN + TEXT_SPACING,
							rDlg.left + P3_TEXT_POSITION + TEXT_WIDTH,
							rDlg.top + P3_BEVEL_POSITION + BEVEL_HEIGHT +
								P3_BEVEL_BOTTOM_MARGIN +
								TEXT_HEIGHT + TEXT_MARGIN + TEXT_SPACING +
								TEXT_HEIGHT);
		nH19Top = 10;
		nH19Left = rDlg.right + P3_INTER_APP_MARGIN;
		GetDlgItem(IDC_H8NAMEPLATE) -> ShowWindow(SW_SHOW);
		GetDlgItem(IDC_BEVEL) -> ShowWindow(SW_SHOW);
		break;
	}

/*	Create and show the display window.	*/

	m_wDisplay.MoveWindow(&rWnd);

//	Now position everything by pixels! (m_wDisplay already is.)

	GetDlgItem(IDC_H8NAMEPLATE) -> MoveWindow(&rLogo);
	m_pMagnetButton -> MoveWindow(rMagnet);
	m_pMagnetButton -> SetBitmap((HBITMAP) m_NoMagnetBmp);
	m_Keypad1Btn.MoveWindow(rKeypad);
	rKeypad.left += KEYWIDTH;
	rKeypad.right += KEYWIDTH;
	m_Keypad2Btn.MoveWindow(rKeypad);
	rKeypad.left += KEYWIDTH;
	rKeypad.right += KEYWIDTH;
	m_Keypad3Btn.MoveWindow(rKeypad);
	rKeypad.left += KEYWIDTH;
	rKeypad.right += KEYWIDTH;
	m_Keypad4Btn.MoveWindow(rKeypad);
	rKeypad.left -= 3 * KEYWIDTH;
	rKeypad.right -= 3 * KEYWIDTH;
	rKeypad.top += KEYHEIGHT;
	rKeypad.bottom += KEYHEIGHT;
	m_Keypad5Btn.MoveWindow(rKeypad);
	rKeypad.left += KEYWIDTH;
	rKeypad.right += KEYWIDTH;
	m_Keypad6Btn.MoveWindow(rKeypad);
	rKeypad.left += KEYWIDTH;
	rKeypad.right += KEYWIDTH;
	m_Keypad7Btn.MoveWindow(rKeypad);
	rKeypad.left += KEYWIDTH;
	rKeypad.right += KEYWIDTH;
	m_Keypad8Btn.MoveWindow(rKeypad);
	rKeypad.left -= 3 * KEYWIDTH;
	rKeypad.right -= 3 * KEYWIDTH;
	rKeypad.top += KEYHEIGHT;
	rKeypad.bottom += KEYHEIGHT;
	m_Keypad9Btn.MoveWindow(rKeypad);
	rKeypad.left += KEYWIDTH;
	rKeypad.right += KEYWIDTH;
	m_Keypad10Btn.MoveWindow(rKeypad);
	rKeypad.left += KEYWIDTH;
	rKeypad.right += KEYWIDTH;
	m_Keypad11Btn.MoveWindow(rKeypad);
	rKeypad.left += KEYWIDTH;
	rKeypad.right += KEYWIDTH;
	m_Keypad12Btn.MoveWindow(rKeypad);
	rKeypad.left -= 3 * KEYWIDTH;
	rKeypad.right -= 3 * KEYWIDTH;
	rKeypad.top += KEYHEIGHT;
	rKeypad.bottom += KEYHEIGHT;
	m_Keypad13Btn.MoveWindow(rKeypad);
	rKeypad.left += KEYWIDTH;
	rKeypad.right += KEYWIDTH;
	m_Keypad14Btn.MoveWindow(rKeypad);
	rKeypad.left += KEYWIDTH;
	rKeypad.right += KEYWIDTH;
	m_Keypad15Btn.MoveWindow(rKeypad);
	rKeypad.left += KEYWIDTH;
	rKeypad.right += KEYWIDTH;

//	The RTM and RESET buttons go below the bevel.

	m_Keypad16Btn.MoveWindow(rKeypad);
	rKeypad.left -= 3 * KEYWIDTH;
	rKeypad.right -= 3 * KEYWIDTH;
	rKeypad.top = rKeypad.bottom + BOTTOM_KEYSET_MARGIN;
	rKeypad.bottom = rKeypad.top + KEYHEIGHT;
	m_Keypad17Btn.MoveWindow(rKeypad);
	rKeypad.left += 3 * KEYWIDTH;
	rKeypad.right += 3 * KEYWIDTH;
	m_Keypad18Btn.MoveWindow(rKeypad);
	GetDlgItem(IDC_BEVEL) -> MoveWindow(&rBevel);
	m_IOThreadStatusWindow.MoveWindow(&rIOStatus);
	m_CPThreadStatusWindow.MoveWindow(&rCPStatus);

////////////////////////////////////////////////////////////////////////////////
//
//	Now place all the graphical elements for the H-19.

	m_H19Frame.GetWindowRect(&rDlg);
	ScreenToClient(&rDlg);
	rDlg.left = nH19Left;
	rDlg.top = nH19Top;
	m_Local.GetWindowRect(&rControl);
	switch(m_nPhotorealismMode)
	{
	default:
	case 0:	//	Minimalism
		m_H19Frame.ShowWindow(SW_HIDE);
		nHeight = rControl.Height();
		nWidth = rControl.Width();
		m_KbdHelpBtn.SetWindowText("Key Help");
		rDlg.right = rDlg.left + P0_H19_WINDOW_WIDTH;
		rDlg.bottom = rDlg.top + H19_SCREEN_HEIGHT;
		rScreen = CRect(rDlg.left,
						rDlg.top,
						rDlg.right,
						rDlg.top + H19_SCREEN_HEIGHT);
		rNameplate = CRect(0,0,rScreen.right,P0_TOP_MARGIN + DISPLAY_HEIGHT / 2 - nHeight / 2);
		rControl.top = rNameplate.bottom;
		rControl.bottom = rControl.top + nHeight;
		rControl.right = rNameplate.right - nWidth;
		rControl.left = rControl.right - nWidth;
		nViewHeight = rDlg.bottom + 10;
		nViewWidth = nH19Left + rDlg.Width() + 10;
		break;
	case 1:	//	Compact
		m_H19Frame.ShowWindow(SW_SHOW);
		rDlg.right = rDlg.left + P1_H19_WINDOW_WIDTH;
		rDlg.bottom = rDlg.top + P1_H19_WINDOW_HEIGHT;
		rScreen = CRect(rDlg.left + P1_H19_LEFT_MARGIN,
						rDlg.top + P1_H19_TOP_MARGIN,
						rDlg.left + P1_H19_LEFT_MARGIN + H19_SCREEN_WIDTH,
						rDlg.top + P1_H19_TOP_MARGIN + H19_SCREEN_HEIGHT);
		rNameplate = CRect(rDlg.left + P1_H19_LEFT_MARGIN + H19_SCREEN_WIDTH + 
							P1_H19_LOGO_LEFT_SPACE,
							rDlg.top + P1_H19_TOP_MARGIN,
							rDlg.left + P1_H19_LEFT_MARGIN + H19_SCREEN_WIDTH + 
							P1_H19_LOGO_LEFT_SPACE + H19_LOGO_WIDTH,
							rDlg.top + P1_H19_TOP_MARGIN + H19_LOGO_HEIGHT);
		m_KbdHelpBtn.SetWindowText("Key Help");
		nHeight = rControl.Height();
		nWidth = rControl.Width();
		rControl.top = rNameplate.bottom + P1_H19_BOTTOM_SPACE;
		rControl.bottom = rControl.top + nHeight;
		rControl.left = rScreen.right + 10;
		rControl.right = rControl.left + nWidth;
		rDlg.right = rControl.right + P1_H19_LEFT_MARGIN;
		nViewHeight = rDlg.bottom + 10;
		nViewWidth = nH19Left + rDlg.Width() + 10;
		break;
	case 2:	//	Space Saving
		m_H19Frame.ShowWindow(SW_SHOW);
		rDlg.right = rDlg.left + P2_H19_WINDOW_WIDTH;
		rDlg.bottom = rDlg.top + P2_H19_WINDOW_HEIGHT;
		rScreen = CRect(rDlg.left + P2_H19_LEFT_MARGIN,
						rDlg.top + P2_H19_TOP_MARGIN,
						rDlg.left + P2_H19_LEFT_MARGIN + H19_SCREEN_WIDTH,
						rDlg.top + P2_H19_TOP_MARGIN + H19_SCREEN_HEIGHT);
		rNameplate = CRect(rDlg.left + P2_H19_LEFT_MARGIN + H19_SCREEN_WIDTH + 
							P2_H19_LOGO_LEFT_SPACE,
							rDlg.top + P2_H19_TOP_MARGIN,
							rDlg.left + P2_H19_LEFT_MARGIN + H19_SCREEN_WIDTH + 
							P2_H19_LOGO_LEFT_SPACE + H19_LOGO_WIDTH,
							rDlg.top + P2_H19_TOP_MARGIN + H19_LOGO_HEIGHT);
		m_KbdHelpBtn.SetWindowText("Key Help");
		nHeight = rControl.Height();
		nWidth = rControl.Width();
		rControl.top = rNameplate.bottom + P2_H19_BOTTOM_SPACE;
		rControl.bottom = rControl.top + nHeight;
		rControl.left = rScreen.right + 10;
		rControl.right = rControl.left + nWidth;
		rDlg.right = rControl.right + P2_H19_LEFT_MARGIN;
		nViewHeight = rDlg.bottom + 10;
		nViewWidth = nH19Left + rDlg.Width() + 10;
		break;
	case 3:	//	Photorealism
		m_H19Frame.ShowWindow(SW_SHOW);
		rDlg.right = rDlg.left + P3_H19_WINDOW_WIDTH;
		rDlg.bottom = rDlg.top + P3_H19_WINDOW_HEIGHT;
		rScreen = CRect(rDlg.left + P3_H19_LEFT_MARGIN,
						rDlg.top + P3_H19_TOP_MARGIN,
						rDlg.left + P3_H19_LEFT_MARGIN + H19_SCREEN_WIDTH,
						rDlg.top + P3_H19_TOP_MARGIN + H19_SCREEN_HEIGHT);
		rNameplate = CRect(rDlg.left + P3_H19_LEFT_MARGIN + H19_SCREEN_WIDTH + 
							P3_H19_LOGO_LEFT_SPACE,
							rDlg.top + P3_H19_TOP_MARGIN,
							rDlg.left + P3_H19_LEFT_MARGIN + H19_SCREEN_WIDTH + 
							P3_H19_LOGO_LEFT_SPACE + H19_LOGO_WIDTH,
							rDlg.top + P3_H19_TOP_MARGIN + H19_LOGO_HEIGHT);
		m_KbdHelpBtn.SetWindowText("Show Keyboard Mapping");
		nHeight = rControl.Height();
		nWidth = rControl.Width();
		rControl.top = rNameplate.bottom + P3_H19_BOTTOM_SPACE;
		rControl.bottom = rControl.top + nHeight;
		rControl.left = (rNameplate.left + rNameplate.right - nWidth) / 2;
		rControl.right = rControl.left + nWidth;
		nViewHeight = rDlg.bottom + 10;
		nViewWidth = nH19Left + rDlg.Width() + 10;
		break;
	}
	m_H19Frame.MoveWindow(&rDlg);
	m_Screen.MoveWindow(&rScreen);
	m_Nameplate.MoveWindow(&rNameplate);
	if (m_nPhotorealismMode != 3)
	{

//	Position the Keyboard Map button over the online button and make it
//	the same size.
	
		nHeight = rControl.Height();
		rKbdMap = rControl;
		nWidth = rControl.Width();
		if (m_nPhotorealismMode == 0)
		{
			rKbdMap.right = rKbdMap.right - TEXT_WIDTH + nWidth;
			rKbdMap.left = rKbdMap.right - nWidth;
		}
		else
		{
			rKbdMap.bottom = rKbdMap.top - 10;
			rKbdMap.top = rKbdMap.bottom - nHeight;
		}
		m_KbdHelpBtn.MoveWindow(&rKbdMap);
		m_Nameplate.ShowWindow(SW_HIDE);
		m_KeyhelpGraphic.ShowWindow(SW_HIDE);
		m_KbdHelpBtn.SetCheck(FALSE);
	}
	else
	{

//	Position the Keyboard Map button over the nameplate and size it to fit.
	
		m_Nameplate.ShowWindow(SW_SHOW);
		m_KbdHelpBtn.GetWindowRect(&rKbdMap);
		rKbdMap.right = rKbdMap.left + rKbdMap.Width();
		nWidth = max(rNameplate.Width() - 10, rKbdMap.Width());
		nHeight = 2 * rControl.Height() / 3;
		rKbdMap.top = rNameplate.top + 5;
		rKbdMap.bottom = rKbdMap.top + nHeight;
		rKbdMap.left = (rNameplate.left + rNameplate.right) / 2 -
						nWidth / 2;
		rKbdMap.right = rKbdMap.left + nWidth;
		m_KbdHelpBtn.MoveWindow(&rKbdMap);
		nBottom = rKbdMap.bottom + 5;

//	Position and hide the keyboard mapping window.  (This graphic is displayed
//	when the keyboard help button is depressed.)

		m_KeyhelpGraphic.GetWindowRect(&rKbdMap);
		rKbdMap.bottom = rKbdMap.top + rKbdMap.Height();
		rKbdMap.right = rKbdMap.left + rKbdMap.Width();
		nWidth = rKbdMap.Width();
		nHeight = rKbdMap.Height();
		rKbdMap.top = nBottom;
		rKbdMap.bottom = rKbdMap.top + nHeight;
		rKbdMap.left = (rNameplate.left + rNameplate.right) / 2 -
						nWidth / 2;
		rKbdMap.right = rKbdMap.left + nWidth;
		m_KeyhelpGraphic.MoveWindow(&rKbdMap);
		m_KeyhelpGraphic.ShowWindow(SW_HIDE);
		m_KbdHelpBtn.SetCheck(FALSE);
	}

//	Position the Local button centered with respect to the nameplate.  Line it
//	up with the bottom of the "tube".

	m_Local.MoveWindow(&rControl);
	GetClientRect(&rWnd);
	rWnd.bottom = rWnd.top + nViewHeight;
	rWnd.right = rWnd.left + nViewWidth;
	SetWindowPos(NULL,
					0,
					0,
					rWnd.Width(),
					rWnd.Height(),
					SWP_NOACTIVATE | SWP_NOZORDER | SWP_NOMOVE);
	if (m_nPhotorealismMode == 0)
	{
		m_H8Frame.MoveWindow(rWnd);
	}
	SetScrollSizes(MM_TEXT, CSize(rWnd.Width(), rWnd.Height()));
	GetParentFrame() -> RecalcLayout();

//	Grow/shrink the application to hold the view, but ensure that we don't
//	grow to exceed the size of the desktop.

	ResizeParentToFit((rWnd.Width() > rDesktop.Width() - 10) ||
						(rWnd.Height() > rDesktop.Height() - 10));
}

/////////////////////////////////////////////////////////////////////////////
// CVirtual_h8View diagnostics

#ifdef _DEBUG
void CVirtual_h8View::AssertValid() const
{
	CFormView::AssertValid();
}

void CVirtual_h8View::Dump(CDumpContext& dc) const
{
	CFormView::Dump(dc);
}

CVirtual_h8Doc* CVirtual_h8View::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CVirtual_h8Doc)));
	return (CVirtual_h8Doc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CVirtual_h8View message handlers


void CVirtual_h8View::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CRect	rMagnet;

	m_pMagnetButton -> GetWindowRect(rMagnet);
	ScreenToClient(&rMagnet);

	if (rMagnet.PtInRect(point))
	{
		if (m_pMagnetButton -> GetBitmap() == (HBITMAP) m_NoMagnetBmp)
		{
			m_pMagnetButton -> SetBitmap((HBITMAP) m_MagnetBmp);
			theApp.m_PAM_IO_PORTS.m_bReedSwitch = TRUE;
		}
		else
		{
			m_pMagnetButton -> SetBitmap((HBITMAP) m_NoMagnetBmp);
			theApp.m_PAM_IO_PORTS.m_bReedSwitch = FALSE;
		}
	}

	CFormView::OnLButtonDown(nFlags, point);
}




void CVirtual_h8View::OnKbdHelpCk() 
{
	KeyhelpDlg	myDlg;

	BOOL	m_bState = m_KbdHelpBtn.GetCheck();
	
	if (m_bState)
	{
		if (m_nPhotorealismMode == 3)
		{

//	Show the keyboard mapping graphic.

			m_KeyhelpGraphic.ShowWindow(SW_SHOW);
		}
		else
		{
			myDlg.DoModal();
			m_KbdHelpBtn.SetCheck(FALSE);
		}
	}
	else
	{

//	Hide the keyboard mapping graphic.

		m_KeyhelpGraphic.ShowWindow(SW_HIDE);
	}
		
}


void CVirtual_h8View :: OnLocal() 
{
	m_bLocal= !m_bLocal;
	m_Screen.m_bLocal = m_bLocal;
	WritePrivateProfileString(INI_KEY_ONLINE,
								m_bLocal ? "0" : "1",
								(LPCSTR) m_strIniFile);
}

/////////////////////////////////////////////////////////////////////////////
//


void OnZeroButton(BOOL bState) 
{
	theApp.m_PAM_IO_PORTS.nKeycode = (char)(bState ? PANEL_KEY_0 : 0x0FF);
}

void OnOneButton(BOOL bState) 
{
	theApp.m_PAM_IO_PORTS.nKeycode = (char)(bState ? PANEL_KEY_1 : 0x0FF);
}

void OnTwoButton(BOOL bState) 
{
	theApp.m_PAM_IO_PORTS.nKeycode = (char)(bState ? PANEL_KEY_2 : 0x0FF);
}

void OnThreeButton(BOOL bState) 
{
	theApp.m_PAM_IO_PORTS.nKeycode = (char)(bState ? PANEL_KEY_3 : 0x0FF);
}

void OnFourButton(BOOL bState) 
{
	theApp.m_PAM_IO_PORTS.nKeycode = (char)(bState ? PANEL_KEY_4 : 0x0FF);
}

void OnFiveButton(BOOL bState) 
{
	theApp.m_PAM_IO_PORTS.nKeycode = (char)(bState ? PANEL_KEY_5 : 0x0FF);
}

void OnSixButton(BOOL bState) 
{
	theApp.m_PAM_IO_PORTS.nKeycode = (char)(bState ? PANEL_KEY_6 : 0x0FF);
}

void OnSevenButton(BOOL bState) 
{
	theApp.m_PAM_IO_PORTS.nKeycode = (char)(bState ? PANEL_KEY_7 : 0x0FF);
}

void OnEightButton(BOOL bState) 
{
	theApp.m_PAM_IO_PORTS.nKeycode = (char)(bState ? PANEL_KEY_8 : 0x0FF);
}

void OnNineButton(BOOL bState) 
{
	theApp.m_PAM_IO_PORTS.nKeycode = (char)(bState ? PANEL_KEY_9 : 0x0FF);
}

void OnAButton(BOOL bState) 
{
	theApp.m_PAM_IO_PORTS.nKeycode = (char)(bState ? PANEL_KEY_A : 0x0FF);
}

void OnBButton(BOOL bState) 
{
	theApp.m_PAM_IO_PORTS.nKeycode = (char)(bState ? PANEL_KEY_B : 0x0FF);
}

void OnCButton(BOOL bState) 
{
	theApp.m_PAM_IO_PORTS.nKeycode = (char)(bState ? PANEL_KEY_C : 0x0FF);
}

void OnDButton(BOOL bState) 
{
	theApp.m_PAM_IO_PORTS.nKeycode = (char)(bState ? PANEL_KEY_D : 0x0FF);
}

void OnEButton(BOOL bState) 
{
	theApp.m_PAM_IO_PORTS.nKeycode = (char)(bState ? PANEL_KEY_E : 0x0FF);
}

void OnFButton(BOOL bState) 
{
	theApp.m_PAM_IO_PORTS.nKeycode = (char)(bState ? PANEL_KEY_F : 0x0FF);
}

void OnRTMButton(BOOL bState)
{

//	RTM key is zero plus E keys; set keycode accordingly.

	theApp.m_PAM_IO_PORTS.nKeycode = (char)(bState ? (PANEL_KEY_0 & PANEL_KEY_E & 0x0FF) : 0x0FF);

//	Also set interrupt 10.

	EnterCriticalSection(&theApp.m_Bus.m_critBus);
	theApp.m_Bus.m_bRTMSignal  = bState;
	LeaveCriticalSection(&theApp.m_Bus.m_critBus);
}

void OnResetButton(BOOL bState) 
{
	CStringList	cslArgs;

	if (bState)
	{

//	RESET key -- allow processor to start by signalling RESET and removing WAIT.
//	(No point in setting the keycode, since nobody can read it while reset is
//	asserted.)

		EnterCriticalSection(&theApp.m_Bus.m_critBus);
		theApp.m_Bus.m_bReset = TRUE;
		theApp.m_Bus.m_bNewInput = TRUE;
		LeaveCriticalSection(&theApp.m_Bus.m_critBus);

//	Reset the hardware

		theApp.m_XCON_IO_PORTS.ioXCON_IOR_362 = XCON_CFG_DATA;
		theApp.m_XCON_IO_PORTS.ioXCON_IOW_362 = 0;
		theApp.m_PAM_IO_PORTS.Reset();
		theApp.m_PAM_IO_PORTS.TapePort.bCommand_State = FALSE;
		theApp.m_PAM_IO_PORTS.TapePort.nCommand = 0;
		theApp.m_PAM_IO_PORTS.TapePort.nMode = 0;
		theApp.m_PAM_IO_PORTS.TapePort.nStatus = 0;
		if (theApp.m_fpTapeOut != NULL)
		{
			fclose(theApp.m_fpTapeOut);
			theApp.m_fpTapeOut = NULL;
			cslArgs.RemoveAll();
			cslArgs.AddTail("1");
			cslArgs.AddTail("0");
			theApp.EmitMessage(CASSETTE_CONNECTION, CASSETTE_LOCK, cslArgs);
		}
		if (theApp.m_fpTapeIn != NULL)
		{
			fclose(theApp.m_fpTapeIn);
			theApp.m_fpTapeIn = NULL;
			cslArgs.RemoveAll();
			cslArgs.AddTail("0");
			cslArgs.AddTail("0");
			theApp.EmitMessage(CASSETTE_CONNECTION, CASSETTE_LOCK, cslArgs);
		}

//	If we get here, processor has been reset.

	}
}


void CVirtual_h8View::OnTimer(UINT nIDEvent) 
{
	BOOL		bRunBit;
	BOOL		bIENBit;
	static BOOL	bPreviousIENBit[3] = {FALSE, FALSE, FALSE};
	int			nDigit;
	static int	nTickCount = 0;
	BOOL		bTimeToBlink;
	static BOOL	bFirst = TRUE;

//	This timer routine must serve two purposes: it is activated 10 times a
//	second so that the H-8's display is updated.  But it also must blink the
//	H-19's cursor at about two per second, so we use the nTickCount to track how
//	many times through the timer we go and change the cursor state on a pattern
//	that goes 0 = on, 2 = off, 5 = on, 7 = off, 10 =on, 12 = off, ...

	bTimeToBlink = (((nTickCount % 5) == 0) || ((nTickCount % 5) == 2));
	++nTickCount;
	if (nIDEvent != 0)
	{
		if (bFirst)
		{

//	The first time through, we command a full paint of the display.  This is
//	especially necessary if we start up in reverse video or are using the debug
//	mode's display test to pre-load the raster.  We have to wait for the timer
//	because trying to do this from OnInitDialog won't succeed because the screen
//	window isn't yet displayed.

			bFirst = FALSE;
			m_Screen.RedrawWindow();
			m_Screen.RepaintScreen();
		}
		if (bTimeToBlink)
		{
			m_Screen.m_bCursorBlinkState = ! m_Screen.m_bCursorBlinkState;
			m_Screen.RedrawCursor();
		}
	}

//	This part of the timer routine handles the sampling of the bus state for
//	display purposes.

	EnterCriticalSection(&theApp.m_Bus.m_critBus);

//	Test the state of the run flag and theApp.m_Bus.hInterruptsEnabled.  Set m_bRunLED
//	and m_bInterruptEnabledLED accordingly.

	bRunBit = theApp.m_Bus.m_bRun;
	bIENBit = theApp.m_Bus.m_bInterruptsEnabled;
	m_wDisplay.m_bRunLED = bRunBit;
	m_wDisplay.m_bInterruptEnabledLED = bIENBit

#if 0

//	Simulate a four-bit serial shift register with all stages asynchronously preset
//	and synchronously reset.  This will make the ION light stop flickering while
//	still making it respond to longer periods of interrupts being disabled.

										 |
										bPreviousIENBit[0] |
										bPreviousIENBit[1] |
										bPreviousIENBit[2]

#endif
										
										;
	bPreviousIENBit[2] = bPreviousIENBit[1] | bIENBit;
	bPreviousIENBit[1] = bPreviousIENBit[0] | bIENBit;
	bPreviousIENBit[0] = bIENBit;
	if (!theApp.m_PAM_IO_PORTS.bUpdate)
	{

//	If the "update" flag is not set, clear the MAN1s.  This simulates the
//	circuitry on the control panel board that resets the segment select regsiter
//	every two milliseconds, while at the same time simulating human persistence
//	of vision.

		for (nDigit = 0; nDigit < 9; ++nDigit)
		{
			theApp.m_PAM_IO_PORTS.nMAN1[nDigit] = PANEL_MAN1_SPACE;
		}
	}
	else
	{
		theApp.m_PAM_IO_PORTS.bUpdate = FALSE;
	}

//	Redraw the display.

	m_wDisplay.RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);
	theApp.m_PAM_IO_PORTS.bMonitorLED = FALSE;
	if (nIDEvent != 0)
	{

//	Timer has expired.  Re-start it.

		m_nTimerID = SetTimer(PANEL_TIMER, PANEL_TIMER_PERIOD, NULL);
		CFormView::OnTimer(nIDEvent);
	}
	LeaveCriticalSection(&theApp.m_Bus.m_critBus);
}

void	CVirtual_h8View :: ConfigLogicAnalyzer()
{
	LogicAnalyzer	myDlg;
	CString			strLogfile;

	if (AfxMessageBox("Do you want to enable and configure\r\n"
						"the built-in Logic Analyzer?",
						MB_YESNO) == IDYES)
	{
		if (myDlg.DoModal() == IDOK)
		{
			strLogfile = theApp.m_strH8Directory + "\\trace.txt";
			if ((tfp = fopen((LPCSTR) strLogfile, "w")) != NULL)
			{

//	Capture all the cycle logging data.

				g_bLACycleTrace = myDlg.m_bLogAllCycles;
				g_bLAResetTrace = myDlg.m_bLogResets || g_bLACycleTrace;
				g_bLADumpPAMArea = myDlg.m_bDumpPAMWorkArea;
				g_bLABranchTrace = myDlg.m_bTraceBranches;
				g_bLAIOReadTrace = myDlg.m_bLogIORead || g_bLACycleTrace;
				g_bLAIOWriteTrace = myDlg.m_bLogIOWrite || g_bLACycleTrace;
//				if (myDlg.m_bQualifyIORad)
//					g_nLAIOReadQualifier = m_nIOReadPort;
//				else
//					g_nLAIOReadQualifier = 0x0100;
//				if (myDlg.m_bQualifyIOWrite)
//					g_nLAIOWriteQualifier = m_nIOWritePort;
//				else
//					g_nLAIOWriteQualifier = 0x0100;
				g_bLAIACKCycleTrace = myDlg.m_bLogIACK || g_bLACycleTrace;
				g_bLAIENCycleTrace = myDlg.m_bLogIntEnable || g_bLACycleTrace;
				g_bLAIRQCycleTrace = myDlg.m_bLogIRQ || g_bLACycleTrace;
				g_bLAM1CycleTrace = myDlg.m_bLogM1 || g_bLACycleTrace;
			}
			else
			{
				AfxMessageBox("Cannot create " + strLogfile);
			}
		}
		else tfp = NULL;
	}
}


void CVirtual_h8View::OnViewP3() 
{
	m_nPhotorealismMode = 3;
	WritePrivateProfileString(INI_KEY_APPEARANCE,
							  "3",
							  (LPCSTR) m_strIniFile);
	CalculateLayout();
	RedrawWindow();
}


void CVirtual_h8View::OnViewP2() 
{
	m_nPhotorealismMode = 2;
	WritePrivateProfileString(INI_KEY_APPEARANCE,
							  "2",
							  (LPCSTR) m_strIniFile);
	CalculateLayout();
	RedrawWindow();
}


void CVirtual_h8View::OnViewP1() 
{
	m_nPhotorealismMode = 1;
	WritePrivateProfileString(INI_KEY_APPEARANCE,
							  "1",
							  (LPCSTR) m_strIniFile);
	CalculateLayout();
	RedrawWindow();
}


void CVirtual_h8View::OnViewP0() 
{
	m_nPhotorealismMode = 0;
	WritePrivateProfileString(INI_KEY_APPEARANCE,
							  "0",
							  (LPCSTR) m_strIniFile);
	CalculateLayout();
	RedrawWindow();
}


void CVirtual_h8View::OnUpdateViewP3(CCmdUI* pCmdUI) 
{
	pCmdUI -> SetCheck(m_nPhotorealismMode == 3);
}


void CVirtual_h8View::OnUpdateViewP2(CCmdUI* pCmdUI) 
{
	pCmdUI -> SetCheck(m_nPhotorealismMode == 2);
}


void CVirtual_h8View::OnUpdateViewP1(CCmdUI* pCmdUI) 
{
	pCmdUI -> SetCheck(m_nPhotorealismMode == 1);
}


void CVirtual_h8View::OnUpdateViewP0(CCmdUI* pCmdUI) 
{
	pCmdUI -> SetCheck(m_nPhotorealismMode == 0);
}


void CVirtual_h8View::OnConfigH19() 
{
	ConfigDlg	myDlg;

	myDlg.m_strPath = m_strIniFile;

	if (myDlg.DoModal() == IDOK)
	{
		m_Screen.SetPowerupConfiguration();
		m_Screen.RepaintScreen();
		RedrawWindow();
	}
}


void CVirtual_h8View::OnH85() 
{
	CString		strCommand = theApp.m_strH8Directory + "\\" H8_5_PROGRAM_NAME;
	HANDLE		hFile;
	CWnd *		pWnd;

//	Launch the H8-5 Tape UI application

//	1.	Try to open "TAPEUI.EXE" in the working directory for read/write
//		exclusive access.

	if ((hFile = CreateFile((LPCSTR) strCommand,
							 GENERIC_READ | GENERIC_WRITE,
							 0,
							 NULL,
							 OPEN_EXISTING,
							 FILE_ATTRIBUTE_NORMAL,
							 NULL)) == INVALID_HANDLE_VALUE)
	{

//	1a.	If this fails, either the program is already running or it does not
//		exist.  See which.

		if ((pWnd =
				FindWindow(NULL, "Virtual H8-5 Tape Drive Emulation")) == NULL)
		{


//	1a1.	File does not exist.  In that case, return without complaint.

		}
		else
		{

//	1a2.	Program already running -- switch to it.

			pWnd -> ShowWindow(SW_RESTORE);
			pWnd -> SetWindowPos(&wndTop, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
		}
	}
	else
	{

//	1b.	Otherwise, to a WinExec to start the program.

		CloseHandle(hFile);
		WinExec((LPCSTR) strCommand, SW_SHOW);
	}
}


void CVirtual_h8View::OnUpdateH85(CCmdUI* pCmdUI) 
{

//	No work required at the moment.

	pCmdUI -> Enable(TRUE);
}


afx_msg LRESULT CVirtual_h8View :: OnBreakToH19(WPARAM wParam, LPARAM /* lParam */)
{
	if (!m_bLocal)
	{
		m_Screen.ReceiveBreak();
	}
	return 0L;
}

afx_msg LRESULT CVirtual_h8View :: OnCharToH19(WPARAM wParam, LPARAM /* lParam */)
{
	if (!m_bLocal)
	{
		m_Screen.CharacterIn((char) wParam);
	}
	return 0L;
}

afx_msg LRESULT CVirtual_h8View :: OnIncoming(WPARAM wParam, LPARAM /* lParam */)
{
	int			nConnection = wParam;
	CString		strTemp;

//	TODO: This code presumes that messages don't get "stacked up".  Should probably
//	do a while loop until the specified message buffer contains no more strings.

	if (theApp.m_IPC.GetMessage(nConnection, strTemp))
	{
		ResetEvent(theApp.m_Bus.m_hTimeRuns);
		MessageDispatch(strTemp, nConnection);
		SetEvent(theApp.m_Bus.m_hTimeRuns);
	}
	else
	{

//	We were called but the message buffer was empty!

		AfxMessageBox("Program Error -- empty message buffer.");
	}
 	return 0L;
}

void	CVirtual_h8View :: MessageDispatch(CString strMsg, int nConnection)
{
	int			nCommandCode;
	CStringList	cslArgs;
	CString		strArg[MAX_ARG_COUNT];

	if (parse_message(strMsg, nCommandCode, cslArgs))
	{
		switch(nCommandCode)
		{
		case CASSETTE_CHANGE:
//	nConnection should be 0.
//	Message Syntax:	"CASSETTE_CHANGE {0 | 1} [path]"
//	Type:			QUEUED
//	Issued By:		TAPEUI
//	To:				H8CTRL
//	Semantics:		CASSETTE_CHANGE 0 pertains to the Reader device;
//					CASSETTE_CHANGE 1 pertains to the Recorder device.  If no
//					path argument is supplied, the notification is that the
//					device has no cassette "mounted" (e.g.: at start-up or
//					when the operator presses EJECT on the device).
//	Response:		none directly, though a CASSETTE_LOCK will occur if the
//					emulator wants to do IO to the device.

			if (cslArgs.GetCount() > 2)
			{
				AfxMessageBox("CASSETTE_CHANGE: too many arguments.");
			}
			if (cslArgs.GetCount() > 1)
			{

//	A "mount" operation

				strArg[0] = cslArgs.RemoveHead();
				strArg[1] = cslArgs.RemoveHead();
				if (atoi((LPCSTR) strArg[0]) == 0)
				{
					theApp.m_strTapeInPath = strArg[1];
				}
				else if (atoi((LPCSTR) strArg[0]) == 1)
				{
					theApp.m_strTapeOutPath = strArg[1];
				}
				else
				{
					AfxMessageBox("Undefined cassette number!");
				}
			}
			else
			{

//	A "dismount" operation.

				strArg[0] = cslArgs.RemoveHead();
				if (atoi((LPCSTR) strArg[0]) == 0)
				{
					theApp.m_strTapeInPath = "";
				}
				else if (atoi((LPCSTR) strArg[0]) == 1)
				{
					theApp.m_strTapeOutPath = "";
				}
				else
				{
					AfxMessageBox("Undefined cassette number!");
				}
			}
			break;
		case CONTEXT_INQUIRY:
//	nConnection could be any.  arg[0] should be zero.
//	Message Syntax:	"CONTEXT_INQUIRY index"
//					where index is a number.  Currently-defined values are:
//					0 - The path for the H8Enviroment
//
//	Type:			QUEUED
//	Issued By:		any slave application
//	To:				H8CTRL
//	Semantics:		This message asks H8CTRL, the applciation which keeps all
//					such information, to return a context value.  Context values
//					are pieces of information (usually paths or configuration
//					settings) needed by other applications in order to
//					initialize or access data in a way consistent with the rest
//					of the virtual computer system.
//	Response:		CONTEXT_VALUE

			cslArgs.RemoveAll();
			cslArgs.AddTail("0");
			cslArgs.AddTail(theApp.m_strH8Directory);
			theApp.EmitMessage(nConnection, CONTEXT_VALUE, cslArgs);
			break;
		default:

//	Unrecognized message.  Ignore it.

			AfxMessageBox("Unrecognized message.");
			break;
		}
	}
	else
	{
		AfxMessageBox("Syntax error in message.");
	}
}


////////////////////////////////////////////////////////////////////////////////
//	The rest of this code is to catch keystrokes and shift keys from the PC
//	keyboard so they can be passed to the terminal emulator.

BOOL CVirtual_h8View::PreTranslateMessage(MSG* pMsg) 
{
	int	nKeyCode;

	switch(pMsg -> message)
	{
	case WM_SYSKEYDOWN:
		switch (nKeyCode = ((pMsg -> lParam >> 16) & 0x1FF))
		{
		case 0x44:	//	F10
			FunctionKey(nKeyCode);
			return TRUE;
			break;
		}
		break;
	case WM_KEYDOWN:
		
//	First, we must dispatch some keys based on scan code because that's the only
//	way to identify, for example, the Keypad Enter as distinct from the main
//	section's Enter key.  (REQUIRES 102-Key PC/AT STYLE KEYBOARD!!!!)

		switch (nKeyCode = ((pMsg -> lParam >> 16) & 0x1FF))
		{

//	These are "dead" keys -- throw them away.

		case 0x037:	//	Keypad *
		case 0x045:	//	Num Lock
		case 0x04A:	//	Keypad -
		case 0x04E:	//	Keypad +
		case 0x135:	//	Keypad /
		case 0x057:	//	F11
		case 0x058:	//	F12
		case 0x147:	//	Home
		case 0x148:	//	Up Arrow
		case 0x149:	//	Page Up
		case 0x14B:	//	Left Arrow
		case 0x14D:	//	Right Arrow
		case 0x14F:	//	End
		case 0x150:	//	Down Arrow
		case 0x151:	//	Page Down
		case 0x152:	//	Insert
		case 0x153:	//	Delete
			return TRUE;
			break;

//	I must handle these keys:

		case 0x3B:	//	F1
		case 0x3C:	//	F2
		case 0x3D:	//	F3
		case 0x3E:	//	F4
		case 0x3F:	//	F5
		case 0x40:	//	F6
		case 0x41:	//	F7
		case 0x42:	//	F8
		case 0x43:	//	F9
			FunctionKey(nKeyCode);
			return TRUE;
			break;
		default:

//	No special handling required for most keys.

			break;

//	Here are all the keys that require special handling at this level:

		case 0x47:	//	Keypad 7
		case 0x48:	//	Keypad 8
		case 0x49:	//	Keypad 9
		case 0x4B:	//	Keypad 4
		case 0x4C:	//	Keypad 5
		case 0x4D:	//	Keypad 6
		case 0x4F:	//	Keypad 1
		case 0x50:	//	Keypad 2
		case 0x51:	//	Keypad 3
		case 0x52:	//	Keypad 0
		case 0x53:	//	Keypad .
		case 0x11C:	//	Keypad Enter
			KeypadKey(nKeyCode);
			return TRUE;
			break;
		}

//	Anything not already caught can be dispatched on its VK_xxx code

		switch(pMsg -> wParam)
		{
		case VK_BACK:		//	backspace
		case VK_TAB:		//	TAB key
		case VK_RETURN:		//	ENTER key
		case VK_ESCAPE:		//	escape key

//	Strip any special meaning from these keys and pass them along as simple characters

			SendMessage(WM_CHAR, pMsg -> wParam, pMsg -> lParam);
			return FALSE;
			break;
		case VK_SPACE:
		
//	The spacebar must be handled here to avoid pressing buttons.

			Spacebar();	//	for now, just beep.
			return TRUE;
			break;
		case VK_CANCEL:		//	ctl-break
			Break();
			return TRUE;
			break;
		case VK_SCROLL:		//	scroll lock
			ScrollLock();
			return TRUE;
			break;
		default:

//	This key is harmless and needs no translation.

			SendMessage(WM_CHAR, pMsg -> wParam, pMsg -> lParam);
			return FALSE;
			break;
		}
		break;
	case WM_CHAR:
		switch(nKeyCode = (pMsg -> wParam))
		{
		case 0:
		case 1:
		case 2:
		case 3:
		case 4:
		case 5:
		case 6:
		case 7:
		case 8:
		case 9:
		case 10:
		case 11:
		case 12:
		case 13:
		case 14:
		case 15:
		case 16:
		case 17:
		case 18:
		case 19:
		case 20:
		case 21:
		case 22:
		case 23:
		case 24:
		case 25:
		case 26:
		case 28:
		case 29:
		case 30:
		case 31:
		case 127:
			CtrlChar(nKeyCode);
			break;
		case 27:
			EscapeChar();
			break;
		default:

//	Here, we handle the character messages (translated for us from virt-keys to
//	ascii.  We pass this code to the keyboard emulation.

			Character(nKeyCode);
			break;
		}
		return TRUE;
		break;
	default:
		break;
	}

//	Any message not caught is fair game.

	return CFormView::PreTranslateMessage(pMsg);
}


//	Various flavors of keystrokes:

void	CVirtual_h8View :: FunctionKey(int nKeyCode)	//	nKeyCode here is a scan code.
{
	int		nShiftState = TestLockAndShiftState();
	int		nCode;

	switch(nKeyCode)
	{
	case 0x3B:	//	F1	-	f1
	case 0x3C:	//	F2	-	f2
	case 0x3D:	//	F3	-	f3
	case 0x3E:	//	F4	-	f4
	case 0x3F:	//	F5	-	f5
	case 0x40:	//	F6	-	erase
	case 0x41:	//	F7	-	blue
	case 0x42:	//	F8	-	red
	case 0x43:	//	F9	-	gray
	case 0x44:	//	F10	-	reset

//	Combine keycode with shift state to produce unique code values in the range
//	128 - 137 (no modifiers), 138 - 147 (shift), 148 - 157 (control) and 158 -
//	167 (control-shift).

		nCode = (nKeyCode - 0x3B) + 128;
		nCode = nCode;
		nCode +=  (0x44 - 0x3B + 1) * (nShiftState >> 4);
		m_Screen.Keystroke(nCode);
		break;
	default:

//	Should not be here.  Ignore keystroke.

		break;
	}
}


void	CVirtual_h8View :: KeypadKey(int nKeyCode)	//	nKeyCode here is a scan code.
{
	int		nShiftState = TestLockAndShiftState();
	int		nCode;

//	Combine keycode with shift state to produce unique code values in the range
//	168 - 179 (no modifiers), 188 - 199 (shift), 208 - 219 (control) and 228 -
//	239 (control-shift).

	switch(nKeyCode)
	{
	case 0x47:	//	Keypad 7
	case 0x48:	//	Keypad 8
	case 0x49:	//	Keypad 9
		nCode = (nKeyCode - 0x47) + 168;
		nCode += 20 * (nShiftState >> 4);
		m_Screen.Keystroke(nCode);
		break;
	case 0x4B:	//	Keypad 4
	case 0x4C:	//	Keypad 5
	case 0x4D:	//	Keypad 6
		nCode = (nKeyCode - 0x4B) + 171;
		nCode += 20 * (nShiftState >> 4);
		m_Screen.Keystroke(nCode);
		break;
	case 0x4F:	//	Keypad 1
	case 0x50:	//	Keypad 2
	case 0x51:	//	Keypad 3
	case 0x52:	//	Keypad 0
	case 0x53:	//	Keypad .
		nCode = (nKeyCode - 0x4F) + 174;
		nCode += 20 * (nShiftState >> 4);
		m_Screen.Keystroke(nCode);
		break;
	case 0x11C:	//	Keypad Enter
		nCode = 179;
		nCode += 20 * (nShiftState >> 4);
		m_Screen.Keystroke(nCode);
		break;
	default:

//	Should not be here.  Ignore the keystroke.

		break;
	}
}


void	CVirtual_h8View :: Spacebar()
{

//	Shift states don't apply.

	m_Screen.Keystroke(' ');
}


void	CVirtual_h8View :: ScrollLock()
{

//	Shift states don't apply.  Pass as keystroke # 240

	m_Screen.Keystroke(240);
}


void	CVirtual_h8View :: Break()
{

//	Shift states don't apply.  Pass as keystroke #241

	m_Screen.Keystroke(241);
}


void	CVirtual_h8View :: EscapeChar()
{

//	Shift states don't apply.  Pass as keystroke 27.

	m_Screen.Keystroke(27);
}


void	CVirtual_h8View :: Character(int nKeyCode)	//	nKeyCode here is the ascii.
{
	int		nShiftState = TestLockAndShiftState();
	BOOL	bCapsState = ((nShiftState & 32) != 0) || ((nShiftState & 2) != 0);

//	Note: "bCapsState" makes the caps lock key work like it did on the H-19.

	if ((!bCapsState) && (isalpha(nKeyCode)))
	{
		nKeyCode |= 'a' - 'A';
	}
	else if ((bCapsState) && (isalpha(nKeyCode)))
	{
		nKeyCode &= ~('a' - 'A');
	}
	m_Screen.Keystroke(nKeyCode);
}


void	CVirtual_h8View :: CtrlChar(int nKeyCode)	//	nKeyCode here is the ascii.
{

//	Shift states don't apply.  Pass as code value.

	m_Screen.Keystroke(nKeyCode);
}


void CVirtual_h8View :: ClearAllLocks()
{ 
	BYTE keyState[256];

	GetKeyboardState((LPBYTE)&keyState);
	if (keyState[VK_NUMLOCK] & 1)
	{

// Simulate a key press followed by a key release to toggle the shift state.

		keybd_event(VK_NUMLOCK, 0x45,
						KEYEVENTF_EXTENDEDKEY | 0, 0);
		keybd_event(VK_NUMLOCK, 0x45,
					KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP, 0);
	}
	if (keyState[VK_SCROLL] & 1)
	{
		keybd_event(VK_SCROLL, 0x46,
						KEYEVENTF_EXTENDEDKEY | 0, 0);
		keybd_event(VK_SCROLL, 0x46,
					KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP, 0);
	}
	if (keyState[VK_CAPITAL] & 1)
	{
		keybd_event(VK_CAPITAL, 0x3A,
						KEYEVENTF_EXTENDEDKEY | 0, 0);
		keybd_event(VK_CAPITAL, 0x3A,
						KEYEVENTF_EXTENDEDKEY | KEYEVENTF_KEYUP, 0);
	}
	m_bCapsLock = FALSE;
	m_bScrollLock = FALSE;
	m_bNumLock = FALSE;
}


int	CVirtual_h8View :: TestLockAndShiftState()
{
	BYTE keyState[256];

	GetKeyboardState((LPBYTE)&keyState);
	m_bNumLock = (keyState[VK_NUMLOCK] & 1);
	m_bScrollLock =  (keyState[VK_SCROLL] & 1);
	m_bCapsLock = (keyState[VK_CAPITAL] & 1);
	return (((keyState[VK_SHIFT] & 0x80) ? 32 : 0) +
			((keyState[VK_CONTROL] & 0x80) ? 16 : 0) +
			((keyState[VK_MENU] & 0x80) ? 8 : 0) +
			(m_bNumLock ? 4 : 0) +
			(m_bCapsLock ? 2 : 0) +
			(m_bScrollLock ? 1 : 0));
}

void CVirtual_h8View::OnConfigH8() 
{
	H8ConfigDlg	myDlg;
	CString		strValue;

	myDlg.m_nTau = theApp.m_nTau - 1;
	if (myDlg.DoModal() == IDOK)
	{
		theApp.m_nTau = myDlg.m_nTau + 1;
		strValue.Format("%d", myDlg.m_nTau + 1);
		WritePrivateProfileString(INI_KEY_TAU, (LPCSTR) strValue, (LPCSTR) m_strIniFile);
		AfxMessageBox("The value of Tau has been changed.\r\n"
						"You must re-start the Virtual H8 Application\r\n"
						"for this change to take effect.");
	}
}
